/* ---------------------------------------------------------------------------------------
 *  @file:    YTM32Z1ME0_startup_gcc.S
 *  @purpose: GNU Compiler Collection Startup File
 *            YTM32Z1ME0
 *  @version: 1.0
 *  @date:    2025-03-06
 * ---------------------------------------------------------------------------------------*/
/*****************************************************************************/
/* Version: GNU Compiler Collection                                          */
/*****************************************************************************/
    .syntax unified
    .arch armv6-m

    .section .isr_vector, "a"
    .align 2
    .globl __isr_vector
__isr_vector:
    .long   __StackTop                                     /* Top of Stack */
    .long   Reset_Handler                                  /* Reset Handler */
    .long   0                                              /* Reserved */
    .long   HardFault_Handler                              /* HardFault Handler */
    .long   0                                              /* Reserved */
    .long   0                                              /* Reserved */
    .long   0                                              /* Reserved */
    .long   0                                              /* Reserved */
    .long   0                                              /* Reserved */
    .long   0                                              /* Reserved */
    .long   0                                              /* Reserved */
    .long   SVC_Handler                                    /* SVC Handler */
    .long   0                                              /* Reserved */
    .long   0                                              /* Reserved */
    .long   PendSV_Handler                                 /* PendSV Handler */
    .long   SysTick_Handler                                /* SysTick Handler */

/* External Interrupts*/
    .long   Reserved10_IRQHandler                          /* Reserved10 IRQHandler */
    .long   Reserved11_IRQHandler                          /* Reserved11 IRQHandler */
    .long   Reserved12_IRQHandler                          /* Reserved12 IRQHandler */
    .long   Reserved13_IRQHandler                          /* Reserved13 IRQHandler */
    .long   Reserved14_IRQHandler                          /* Reserved14 IRQHandler */
    .long   Reserved15_IRQHandler                          /* Reserved15 IRQHandler */
    .long   Reserved16_IRQHandler                          /* Reserved16 IRQHandler */
    .long   Reserved17_IRQHandler                          /* Reserved17 IRQHandler */
    .long   Reserved18_IRQHandler                          /* Reserved18 IRQHandler */
    .long   GPIOA_IRQHandler                               /* GPIOA IRQHandler */
    .long   MSPWM_Ch_IRQHandler                            /* MSPWM_Ch IRQHandler */
    .long   MSPWM_Ovf_IRQHandler                           /* MSPWM_Ovf IRQHandler */
    .long   CCTMR0_Ch_IRQHandler                           /* CCTMR0_Ch IRQHandler */
    .long   CCTMR0_Ovf_IRQHandler                          /* CCTMR0_Ovf IRQHandler */
    .long   Reserved19_IRQHandler                          /* Reserved19 IRQHandler */
    .long   Reserved20_IRQHandler                          /* Reserved20 IRQHandler */
    .long   DRIVER_Fault_IRQHandler                        /* DRIVER_Fault IRQHandler */
    .long   DRIVER_Vds_IRQHandler                          /* DRIVER_Vds IRQHandler */
    .long   EFM_IRQHandler                                 /* EFM IRQHandler */
    .long   Reserved21_IRQHandler                          /* Reserved21 IRQHandler */
    .long   pTMR_Ch0_IRQHandler                            /* pTMR_Ch0 IRQHandler */
    .long   pTMR_Ch1_IRQHandler                            /* pTMR_Ch1 IRQHandler */
    .long   pTMR_Ch2_IRQHandler                            /* pTMR_Ch2 IRQHandler */
    .long   pTMR_Ch3_IRQHandler                            /* pTMR_Ch3 IRQHandler */
    .long   PCU_IRQHandler                                 /* PCU IRQHandler */
    .long   WDG_IRQHandler                                 /* WDG IRQHandler */
    .long   SPI0_IRQHandler                                /* SPI0 IRQHandler */
    .long   LINPHY_CTRL_IRQHandler                         /* LINPHY_CTRL IRQHandler */
    .long   ADC0_IRQHandler                                /* ADC0 IRQHandler */
    .long   ZDU_IRQHandler                                 /* ZDU IRQHandler */
    .long   Reserved22_IRQHandler                          /* Reserved22 IRQHandler */
    .long   UART0_IRQHandler                               /* UART0 IRQHandler */

    .size    __isr_vector, . - __isr_vector

    .text
    .thumb

/* Reset Handler */

    .thumb_func
    .align 2
    .globl   Reset_Handler
    .weak    Reset_Handler
    .type    Reset_Handler, %function
Reset_Handler:
    cpsid   i               /* Mask interrupts */

    /* Init the rest of the registers */
    ldr     r1,=0
    ldr     r2,=0
    ldr     r3,=0
    ldr     r4,=0
    ldr     r5,=0
    ldr     r6,=0
    ldr     r7,=0
    mov     r8,r7
    mov     r9,r7
    mov     r10,r7
    mov     r11,r7
    mov     r12,r7

#ifndef START_DISABLE_VDDA_SWITCH
    ldr     r0, =0x4007D02C
    ldr     r1, =0x00000031
    str     r1, [r0]
    str     r1, [r0]
    ldr     r1, =0x00000030
    str     r1, [r0]
#endif

#ifndef START_NO_ECC_INIT

    /* Init ECC RAM */
    ldr     r1, =__RAM_START
    ldr     r2, =__RAM_END

    subs    r2, r1
    subs    r2, #1
    ble     SRAM_LOOP_END

    movs    r0, #0
    movs    r3, #4
SRAM_LOOP:
    str     r0, [r1]
    add     r1, r1, r3
    subs    r2, #4
    bge     SRAM_LOOP
SRAM_LOOP_END:
#endif
    
    /* Initialize the stack pointer */
    ldr     r0,=__StackTop
    mov     r13,r0

#ifndef __NO_SYSTEM_INIT
    /* Call the system init routine */
    ldr     r0,=SystemInit
    blx     r0
#endif

    /* Init .data and .bss sections */
    ldr     r0,=init_data_bss
    blx     r0
    cpsie   i               /* Unmask interrupts */
    bl      main
JumpToSelf:
    b       JumpToSelf

    .pool
    .size Reset_Handler, . - Reset_Handler

    .align  1
    .thumb_func
    .weak DefaultISR
    .type DefaultISR, %function
DefaultISR:
    b       DefaultISR
    .size DefaultISR, . - DefaultISR

/*    Macro to define default handlers. Default handler
 *    will be weak symbol and just dead loops. They can be
 *    overwritten by other handlers */
    .macro def_irq_handler  handler_name
    .weak \handler_name
    .set  \handler_name, DefaultISR
    .endm

    /* Exception Handlers */
    def_irq_handler    Reserved0_Handler
    def_irq_handler    HardFault_Handler
    def_irq_handler    Reserved1_Handler
    def_irq_handler    Reserved2_Handler
    def_irq_handler    Reserved3_Handler
    def_irq_handler    Reserved4_Handler
    def_irq_handler    Reserved5_Handler
    def_irq_handler    Reserved6_Handler
    def_irq_handler    Reserved7_Handler
    def_irq_handler    SVC_Handler
    def_irq_handler    Reserved8_Handler
    def_irq_handler    Reserved9_Handler
    def_irq_handler    PendSV_Handler
    def_irq_handler    SysTick_Handler

    /* Device Exception Handlers */

    def_irq_handler    Reserved10_IRQHandler
    def_irq_handler    Reserved11_IRQHandler
    def_irq_handler    Reserved12_IRQHandler
    def_irq_handler    Reserved13_IRQHandler
    def_irq_handler    Reserved14_IRQHandler
    def_irq_handler    Reserved15_IRQHandler
    def_irq_handler    Reserved16_IRQHandler
    def_irq_handler    Reserved17_IRQHandler
    def_irq_handler    Reserved18_IRQHandler
    def_irq_handler    GPIOA_IRQHandler
    def_irq_handler    MSPWM_Ch_IRQHandler
    def_irq_handler    MSPWM_Ovf_IRQHandler
    def_irq_handler    CCTMR0_Ch_IRQHandler
    def_irq_handler    CCTMR0_Ovf_IRQHandler
    def_irq_handler    Reserved19_IRQHandler
    def_irq_handler    Reserved20_IRQHandler
    def_irq_handler    DRIVER_Fault_IRQHandler
    def_irq_handler    DRIVER_Vds_IRQHandler
    def_irq_handler    EFM_IRQHandler
    def_irq_handler    Reserved21_IRQHandler
    def_irq_handler    pTMR_Ch0_IRQHandler
    def_irq_handler    pTMR_Ch1_IRQHandler
    def_irq_handler    pTMR_Ch2_IRQHandler
    def_irq_handler    pTMR_Ch3_IRQHandler
    def_irq_handler    PCU_IRQHandler
    def_irq_handler    WDG_IRQHandler
    def_irq_handler    SPI0_IRQHandler
    def_irq_handler    LINPHY_CTRL_IRQHandler
    def_irq_handler    ADC0_IRQHandler
    def_irq_handler    ZDU_IRQHandler
    def_irq_handler    Reserved22_IRQHandler
    def_irq_handler    UART0_IRQHandler

    .end