/*
** ###################################################################
**     Processor:           YTM32Z1ME0
**     Version:             rev. 1.0, 2025-04-03
**
**     Abstract:
**         Peripheral Access Layer for YTM32Z1ME0
**
**     Copyright (c) 2020 - 2025 Yuntu Microelectronics co.,ltd
**     All rights reserved.
**
** ###################################################################
*/

/*!
 * @file YTM32Z1ME0.h
 * @version 1.0
 * @date 2025-04-03
 * @brief Peripheral Access Layer for YTM32Z1ME0
 *
 * This file contains register definitions and macros for easy access to their
 * bit fields.
 *
 * This file assumes LITTLE endian system.
 */

/* ----------------------------------------------------------------------------
   -- MCU activation
   ---------------------------------------------------------------------------- */

/* Prevention from multiple including the same memory map */
#if !defined(YTM32Z1ME0_H)  /* Check if memory map has not been already included */
#define YTM32Z1ME0_H
#define MCU_YTM32Z1ME0

/* Check if another memory map has not been also included */
#if (defined(MCU_ACTIVE))
  #error YTM32Z1ME0 memory map: There is already included another memory map. Only one memory map can be included.
#endif /* (defined(MCU_ACTIVE)) */
#define MCU_ACTIVE

/** Memory map major version (memory maps with equal major version number are
 * compatible) */
#define MCU_MEM_MAP_VERSION 0x0100u
/** Memory map minor version */
#define MCU_MEM_MAP_VERSION_MINOR 0x0003u

#include <stdint.h>

/* ----------------------------------------------------------------------------
   -- Generic macros
   ---------------------------------------------------------------------------- */

/* IO definitions (access restrictions to peripheral registers) */
/**
*   IO Type Qualifiers are used
*   \li to specify the access to peripheral variables.
*   \li for automatic generation of peripheral register debug information.
*/
#ifndef __IO
#ifdef __cplusplus
  #define   __I     volatile             /*!< Defines 'read only' permissions                 */
#else
  #define   __I     volatile const       /*!< Defines 'read only' permissions                 */
#endif
#define     __O     volatile             /*!< Defines 'write only' permissions                */
#define     __IO    volatile             /*!< Defines 'read / write' permissions              */
#endif


/**
* @brief 32 bits memory read macro.
*/
#if !defined(REG_READ32)
  #define REG_READ32(address)               (*(volatile uint32_t*)(address))
#endif

/**
* @brief 32 bits memory write macro.
*/
#if !defined(REG_WRITE32)
  #define REG_WRITE32(address, value)       ((*(volatile uint32_t*)(address))= (uint32_t)(value))
#endif

/**
* @brief 32 bits bits setting macro.
*/
#if !defined(REG_BIT_SET32)
  #define REG_BIT_SET32(address, mask)      ((*(volatile uint32_t*)(address))|= (uint32_t)(mask))
#endif

/**
* @brief 32 bits bits clearing macro.
*/
#if !defined(REG_BIT_CLEAR32)
  #define REG_BIT_CLEAR32(address, mask)    ((*(volatile uint32_t*)(address))&= ((uint32_t)~((uint32_t)(mask))))
#endif

/**
* @brief 32 bit clear bits and set with new value
* @note It is user's responsability to make sure that value has only "mask" bits set - (value&~mask)==0
*/
#if !defined(REG_RMW32)
  #define REG_RMW32(address, mask, value)   (REG_WRITE32((address), ((REG_READ32(address)& ((uint32_t)~((uint32_t)(mask))))| ((uint32_t)(value)))))
#endif
/* ----------------------------------------------------------------------------
   -- Interrupt vector numbers for YTM32Z1ME0
   ---------------------------------------------------------------------------- */

/*!
 * @addtogroup Interrupt_vector_numbers_YTM32B1L Interrupt vector numbers for YTM32B1L
 * @{
 */

/** Interrupt Number Definitions */
#define NUMBER_OF_INT_VECTORS 48u          /**< Number of interrupts in the Vector table */

/**
 * @brief Defines the Interrupt Numbers definitions
 *
 * This enumeration is used to configure the interrupts.
 *
 * Implements : IRQn_Type_Class
 */
typedef enum
{
  /* Auxiliary constants */
  NotAvail_IRQn     = -128,                /**< Not available device specific interrupt */

  /* Core interrupts */
  HardFault_IRQn              =  -13,                /**< HardFault Interrupt */ 
  SVC_IRQn                    =   -5,                /**< SVC Interrupt */ 
  PendSV_IRQn                 =   -2,                /**< PendSV Interrupt */ 
  SysTick_IRQn                =   -1,                /**< SysTick Interrupt */ 
  /* Device specific interrupts */
  Reserved10_IRQn             =    0,                /**< Reserved10 Interrupt */ 
  Reserved11_IRQn             =    1,                /**< Reserved11 Interrupt */ 
  Reserved12_IRQn             =    2,                /**< Reserved12 Interrupt */ 
  Reserved13_IRQn             =    3,                /**< Reserved13 Interrupt */ 
  Reserved14_IRQn             =    4,                /**< Reserved14 Interrupt */ 
  Reserved15_IRQn             =    5,                /**< Reserved15 Interrupt */ 
  Reserved16_IRQn             =    6,                /**< Reserved16 Interrupt */ 
  Reserved17_IRQn             =    7,                /**< Reserved17 Interrupt */ 
  Reserved18_IRQn             =    8,                /**< Reserved18 Interrupt */ 
  GPIOA_IRQn                  =    9,                /**< GPIOA Interrupt */ 
  MSPWM_Ch_IRQn               =   10,                /**< MSPWM_Ch Interrupt */ 
  MSPWM_Ovf_IRQn              =   11,                /**< MSPWM_Ovf Interrupt */ 
  CCTMR0_Ch_IRQn              =   12,                /**< CCTMR0_Ch Interrupt */ 
  CCTMR0_Ovf_IRQn             =   13,                /**< CCTMR0_Ovf Interrupt */ 
  Reserved19_IRQn             =   14,                /**< Reserved19 Interrupt */ 
  Reserved20_IRQn             =   15,                /**< Reserved20 Interrupt */ 
  DRIVER_Fault_IRQn           =   16,                /**< DRIVER_Fault Interrupt */ 
  DRIVER_Vds_IRQn             =   17,                /**< DRIVER_Vds Interrupt */ 
  EFM_IRQn                    =   18,                /**< EFM Interrupt */ 
  Reserved21_IRQn             =   19,                /**< Reserved21 Interrupt */ 
  pTMR_Ch0_IRQn               =   20,                /**< pTMR_Ch0 Interrupt */ 
  pTMR_Ch1_IRQn               =   21,                /**< pTMR_Ch1 Interrupt */ 
  pTMR_Ch2_IRQn               =   22,                /**< pTMR_Ch2 Interrupt */ 
  pTMR_Ch3_IRQn               =   23,                /**< pTMR_Ch3 Interrupt */ 
  PCU_IRQn                    =   24,                /**< PCU Interrupt */ 
  WDG_IRQn                    =   25,                /**< WDG Interrupt */ 
  SPI0_IRQn                   =   26,                /**< SPI0 Interrupt */ 
  LINPHY_CTRL_IRQn            =   27,                /**< LINPHY_CTRL Interrupt */ 
  ADC0_IRQn                   =   28,                /**< ADC0 Interrupt */ 
  ZDU_IRQn                    =   29,                /**< ZDU Interrupt */ 
  Reserved22_IRQn             =   30,                /**< Reserved22 Interrupt */ 
  UART0_IRQn                  =   31,                /**< UART0 Interrupt */ 
} IRQn_Type;

/*!
 * @}
 */ /* end of group Interrupt_vector_numbers YTM32Z1ME0 */

/* ================================================================================ */
/* ================      Processor and Core Peripheral Section     ================ */
/* ================================================================================ */

/* -------  Start of section using anonymous unions and disabling warnings  ------- */
#if   defined (__CC_ARM)
  #pragma push
  #pragma anon_unions
#elif defined (__ICCARM__)
  #pragma language=extended
#elif defined(__ARMCC_VERSION) && (__ARMCC_VERSION >= 6010050)
  #pragma clang diagnostic push
  #pragma clang diagnostic ignored "-Wc11-extensions"
  #pragma clang diagnostic ignored "-Wreserved-id-macro"
#elif defined (__GNUC__)
  /* anonymous unions are enabled by default */
#elif defined (__TMS470__)
  /* anonymous unions are enabled by default */
#elif defined (__TASKING__)
  #pragma warning 586
#elif defined (__CSMC__)
  /* anonymous unions are enabled by default */
#elif defined (__ghs__)
  /* anonymous unions are enabled by default */
#else
  #warning Not supported compiler type
#endif


/* --------  Configuration of Core Peripherals  ----------------------------------- */
#define __CM0PLUS_REV             
#define __MPU_PRESENT             0U        // MPU present
#define __VTOR_PRESENT            1U        // VTOR present
#define __NVIC_PRIO_BITS          2U        // Number of Bits used for Priority Levels
#define __Vendor_SysTickConfig    0U        // Set to 1 if different SysTick Config is used 

#include "core_cm0plus.h"                                   /* Processor and core peripherals */
#include "core_common.h"                                    /* Processor common */
#include "system_YTM32Z1ME0.h"                              /* System Header */


/* --------  End of section using anonymous unions and disabling warnings  -------- */
#if   defined (__CC_ARM)
  #pragma pop
#elif defined (__ICCARM__)
  /* leave anonymous unions enabled */
#elif (defined(__ARMCC_VERSION) && (__ARMCC_VERSION >= 6010050))
  #pragma clang diagnostic pop
#elif defined (__GNUC__)
  /* anonymous unions are enabled by default */
#elif defined (__TMS470__)
  /* anonymous unions are enabled by default */
#elif defined (__TASKING__)
  #pragma warning restore
#elif defined (__CSMC__)
  /* anonymous unions are enabled by default */
#elif defined (__ghs__)
  /* anonymous unions are enabled by default */
#else
  #warning Not supported compiler type
#endif
/* ----------------------------------------------------------------------------
   -- Device Peripheral Access Layer for YTM32Z1ME0
   ---------------------------------------------------------------------------- */

/*!
 * @addtogroup Peripheral_access_layer_YTM32Z1ME0 Device Peripheral Access Layer for YTM32Z1ME0
 * @{
 */

/* @brief This module covers memory mapped registers available on SoC */

/* -------------------------------------------------------------
   -- EFM Peripheral Access Layer
   ------------------------------------------------------------- */

/*!
 * @addtogroup EFM_Peripheral_Access_Layer EFM Peripheral Access Layer
 * @{
 */


/** EFM - Size of Registers Arrays */
#define EFM_ADDR_PROT_COUNT                                                   (1)

/* EFM Register Layout Typedef */
typedef struct {
    __IO uint32_t CTRL;                                                       /**< Control Register, offset: 0x0000 */
    __IO uint32_t STS;                                                        /**< Status Register, offset: 0x0004 */
    __IO uint32_t CMD;                                                        /**< Command Register, offset: 0x0008 */
    __IO uint32_t CMD_PROT;                                                   /**< Command Protect Register, offset: 0x000C */
    __IO uint32_t TIMING1;                                                    /**< Timing 1 Control Register, offset: 0x0010 */
    __IO uint32_t TIMING2;                                                    /**< Timing 2 Control Register, offset: 0x0014 */
    __IO uint8_t  RESERVED6[ 8];                                              /**< RESERVED6, offset: 0x0018 */
    __IO uint32_t ERR_INJECT_CTRL;                                            /**< Error Inject Control, offset: 0x0020 */
    __IO uint32_t ADDR_PROT[ 1];                                              /**< Flash Partition Protect Register, offset: 0x0024 */
    __IO uint8_t  RESERVED9[ 8];                                              /**< RESERVED9, offset: 0x0028 */
    __IO uint32_t ECCERR_ADDR;                                                /**< ECC Error Address, offset: 0x0030 */
    __IO uint32_t ECCERR_CNT;                                                 /**< ECC Error Count, offset: 0x0034 */
    __IO uint8_t  RESERVED12[456];                                            /**< RESERVED12, offset: 0x0038 */
    __O  uint32_t CMD_UNLOCK;                                                 /**< EFM Command Unlock, offset: 0x0200 */
    __IO uint32_t CUS_KEY;                                                    /**< EFM Customer Key, offset: 0x0204 */
    __IO uint8_t  RESERVED15[248];                                            /**< RESERVED15, offset: 0x0208 */
    __IO uint32_t CTRL_OVRD;                                                  /**< Override Control Register, offset: 0x0300 */
    __IO uint32_t RESERVED17;                                                 /**< RESERVED17, offset: 0x0304 */
    __IO uint32_t RESERVED18[ 1];                                             /**< RESERVED18, offset: 0x0308 */
    __IO uint8_t  RESERVED19[116];                                            /**< RESERVED19, offset: 0x030C */
    __IO uint32_t RESERVED20;                                                 /**< RESERVED20, offset: 0x0380 */
    __IO uint32_t RESERVED21;                                                 /**< RESERVED21, offset: 0x0384 */

} EFM_Type, *EFM_MemMapPtr;

/** Number of instances of the EFM module. */
#define EFM_INSTANCE_COUNT                                                    (1u)

/* EFM base address */
#define EFM_BASE                                                              (0x40020000)
#define EFM                                                                   ((EFM_Type *)(EFM_BASE))

/** Array initializer of EFM peripheral base addresses */
#define EFM_BASE_ADDRS                                                        { EFM_BASE }
/** Array initializer of EFM peripheral base pointers */
#define EFM_BASE_PTRS                                                         { EFM }
/** Number of interrupt vector arrays for the EFM module. */
#define EFM_IRQS_ARR_COUNT                                                    (1u)
/** Number of interrupt channels for the EFM. */
#define EFM_IRQS_CH_COUNT                                                     (1u)
/** Interrupt vectors for the EFM peripheral type */
#define EFM_IRQS                                                              { EFM_IRQn }

/* EFM Register Mask */
/* EFM_CTRL Register */
#define EFM_CTRL_LPEN_MASK                                                    (0x80000000U)
#define EFM_CTRL_LPEN_SHIFT                                                   (31U)
#define EFM_CTRL_LPEN(x)                                                      (((uint32_t)(((uint32_t)(x)) << EFM_CTRL_LPEN_SHIFT)) & EFM_CTRL_LPEN_MASK)
#define EFM_CTRL_PRESCALER_MASK                                               (0x3F0000U)
#define EFM_CTRL_PRESCALER_SHIFT                                              (16U)
#define EFM_CTRL_PRESCALER(x)                                                 (((uint32_t)(((uint32_t)(x)) << EFM_CTRL_PRESCALER_SHIFT)) & EFM_CTRL_PRESCALER_MASK)
#define EFM_CTRL_WE_MASK                                                      (0x1000U)
#define EFM_CTRL_WE_SHIFT                                                     (12U)
#define EFM_CTRL_WE(x)                                                        (((uint32_t)(((uint32_t)(x)) << EFM_CTRL_WE_SHIFT)) & EFM_CTRL_WE_MASK)
#define EFM_CTRL_RWS_MASK                                                     (0x300U)
#define EFM_CTRL_RWS_SHIFT                                                    (8U)
#define EFM_CTRL_RWS(x)                                                       (((uint32_t)(((uint32_t)(x)) << EFM_CTRL_RWS_SHIFT)) & EFM_CTRL_RWS_MASK)
#define EFM_CTRL_CMD_VERIFY_EN_MASK                                           (0x10U)
#define EFM_CTRL_CMD_VERIFY_EN_SHIFT                                          (4U)
#define EFM_CTRL_CMD_VERIFY_EN(x)                                             (((uint32_t)(((uint32_t)(x)) << EFM_CTRL_CMD_VERIFY_EN_SHIFT)) & EFM_CTRL_CMD_VERIFY_EN_MASK)
#define EFM_CTRL_ACCERRIE_MASK                                                (0x8U)
#define EFM_CTRL_ACCERRIE_SHIFT                                               (3U)
#define EFM_CTRL_ACCERRIE(x)                                                  (((uint32_t)(((uint32_t)(x)) << EFM_CTRL_ACCERRIE_SHIFT)) & EFM_CTRL_ACCERRIE_MASK)
#define EFM_CTRL_UNRECOVERRIE_MASK                                            (0x4U)
#define EFM_CTRL_UNRECOVERRIE_SHIFT                                           (2U)
#define EFM_CTRL_UNRECOVERRIE(x)                                              (((uint32_t)(((uint32_t)(x)) << EFM_CTRL_UNRECOVERRIE_SHIFT)) & EFM_CTRL_UNRECOVERRIE_MASK)
#define EFM_CTRL_RECOVERRIE_MASK                                              (0x2U)
#define EFM_CTRL_RECOVERRIE_SHIFT                                             (1U)
#define EFM_CTRL_RECOVERRIE(x)                                                (((uint32_t)(((uint32_t)(x)) << EFM_CTRL_RECOVERRIE_SHIFT)) & EFM_CTRL_RECOVERRIE_MASK)
#define EFM_CTRL_DONEIE_MASK                                                  (0x1U)
#define EFM_CTRL_DONEIE_SHIFT                                                 (0U)
#define EFM_CTRL_DONEIE(x)                                                    (((uint32_t)(((uint32_t)(x)) << EFM_CTRL_DONEIE_SHIFT)) & EFM_CTRL_DONEIE_MASK)
/* EFM_STS Register */
#define EFM_STS_CI_RDVIOLERR_MASK                                             (0x4000000U)
#define EFM_STS_CI_RDVIOLERR_SHIFT                                            (26U)
#define EFM_STS_CI_RDVIOLERR(x)                                               (((uint32_t)(((uint32_t)(x)) << EFM_STS_CI_RDVIOLERR_SHIFT)) & EFM_STS_CI_RDVIOLERR_MASK)
#define EFM_STS_CI_WEERR_MASK                                                 (0x2000000U)
#define EFM_STS_CI_WEERR_SHIFT                                                (25U)
#define EFM_STS_CI_WEERR(x)                                                   (((uint32_t)(((uint32_t)(x)) << EFM_STS_CI_WEERR_SHIFT)) & EFM_STS_CI_WEERR_MASK)
#define EFM_STS_CI_BUSYERR_MASK                                               (0x1000000U)
#define EFM_STS_CI_BUSYERR_SHIFT                                              (24U)
#define EFM_STS_CI_BUSYERR(x)                                                 (((uint32_t)(((uint32_t)(x)) << EFM_STS_CI_BUSYERR_SHIFT)) & EFM_STS_CI_BUSYERR_MASK)
#define EFM_STS_IDLE_MASK                                                     (0x80U)
#define EFM_STS_IDLE_SHIFT                                                    (7U)
#define EFM_STS_IDLE(x)                                                       (((uint32_t)(((uint32_t)(x)) << EFM_STS_IDLE_SHIFT)) & EFM_STS_IDLE_MASK)
#define EFM_STS_ARRAY_SELECTED_MASK                                           (0x40U)
#define EFM_STS_ARRAY_SELECTED_SHIFT                                          (6U)
#define EFM_STS_ARRAY_SELECTED(x)                                             (((uint32_t)(((uint32_t)(x)) << EFM_STS_ARRAY_SELECTED_SHIFT)) & EFM_STS_ARRAY_SELECTED_MASK)
#define EFM_STS_CMD_FAIL_MASK                                                 (0x10U)
#define EFM_STS_CMD_FAIL_SHIFT                                                (4U)
#define EFM_STS_CMD_FAIL(x)                                                   (((uint32_t)(((uint32_t)(x)) << EFM_STS_CMD_FAIL_SHIFT)) & EFM_STS_CMD_FAIL_MASK)
#define EFM_STS_ACCERR_MASK                                                   (0x8U)
#define EFM_STS_ACCERR_SHIFT                                                  (3U)
#define EFM_STS_ACCERR(x)                                                     (((uint32_t)(((uint32_t)(x)) << EFM_STS_ACCERR_SHIFT)) & EFM_STS_ACCERR_MASK)
#define EFM_STS_UNRECOVERR_MASK                                               (0x4U)
#define EFM_STS_UNRECOVERR_SHIFT                                              (2U)
#define EFM_STS_UNRECOVERR(x)                                                 (((uint32_t)(((uint32_t)(x)) << EFM_STS_UNRECOVERR_SHIFT)) & EFM_STS_UNRECOVERR_MASK)
#define EFM_STS_RECOVERR_MASK                                                 (0x2U)
#define EFM_STS_RECOVERR_SHIFT                                                (1U)
#define EFM_STS_RECOVERR(x)                                                   (((uint32_t)(((uint32_t)(x)) << EFM_STS_RECOVERR_SHIFT)) & EFM_STS_RECOVERR_MASK)
#define EFM_STS_DONE_MASK                                                     (0x1U)
#define EFM_STS_DONE_SHIFT                                                    (0U)
#define EFM_STS_DONE(x)                                                       (((uint32_t)(((uint32_t)(x)) << EFM_STS_DONE_SHIFT)) & EFM_STS_DONE_MASK)
/* EFM_CMD Register */
#define EFM_CMD_CMD_MASK                                                      (0xFFU)
#define EFM_CMD_CMD_SHIFT                                                     (0U)
#define EFM_CMD_CMD(x)                                                        (((uint32_t)(((uint32_t)(x)) << EFM_CMD_CMD_SHIFT)) & EFM_CMD_CMD_MASK)
/* EFM_CMD_PROT Register */
#define EFM_CMD_PROT_CMD_PROT_MASK                                            (0xFFFFFFFFU)
#define EFM_CMD_PROT_CMD_PROT_SHIFT                                           (0U)
#define EFM_CMD_PROT_CMD_PROT(x)                                              (((uint32_t)(((uint32_t)(x)) << EFM_CMD_PROT_CMD_PROT_SHIFT)) & EFM_CMD_PROT_CMD_PROT_MASK)
/* EFM_TIMING1 Register */
#define EFM_TIMING1_TPROG_SEL_MASK                                            (0x3U)
#define EFM_TIMING1_TPROG_SEL_SHIFT                                           (0U)
#define EFM_TIMING1_TPROG_SEL(x)                                              (((uint32_t)(((uint32_t)(x)) << EFM_TIMING1_TPROG_SEL_SHIFT)) & EFM_TIMING1_TPROG_SEL_MASK)
/* EFM_TIMING2 Register */
#define EFM_TIMING2_TERASE_SEC_SEL_MASK                                       (0x3U)
#define EFM_TIMING2_TERASE_SEC_SEL_SHIFT                                      (0U)
#define EFM_TIMING2_TERASE_SEC_SEL(x)                                         (((uint32_t)(((uint32_t)(x)) << EFM_TIMING2_TERASE_SEC_SEL_SHIFT)) & EFM_TIMING2_TERASE_SEC_SEL_MASK)
/* EFM_ERR_INJECT_CTRL Register */
#define EFM_ERR_INJECT_CTRL_ECCERR_INJ_EN_MASK                                (0xC0000000U)
#define EFM_ERR_INJECT_CTRL_ECCERR_INJ_EN_SHIFT                               (30U)
#define EFM_ERR_INJECT_CTRL_ECCERR_INJ_EN(x)                                  (((uint32_t)(((uint32_t)(x)) << EFM_ERR_INJECT_CTRL_ECCERR_INJ_EN_SHIFT)) & EFM_ERR_INJECT_CTRL_ECCERR_INJ_EN_MASK)
/* EFM_ADDR_PROT Register */
#define EFM_ADDR_PROT_ADDR_PROT_MASK                                          (0x7FFFU)
#define EFM_ADDR_PROT_ADDR_PROT_SHIFT                                         (0U)
#define EFM_ADDR_PROT_ADDR_PROT(x)                                            (((uint32_t)(((uint32_t)(x)) << EFM_ADDR_PROT_ADDR_PROT_SHIFT)) & EFM_ADDR_PROT_ADDR_PROT_MASK)
/* EFM_ECCERR_ADDR Register */
#define EFM_ECCERR_ADDR_ERR_FLASH_ADDR_MASK                                   (0xFFFFFFFFU)
#define EFM_ECCERR_ADDR_ERR_FLASH_ADDR_SHIFT                                  (0U)
#define EFM_ECCERR_ADDR_ERR_FLASH_ADDR(x)                                     (((uint32_t)(((uint32_t)(x)) << EFM_ECCERR_ADDR_ERR_FLASH_ADDR_SHIFT)) & EFM_ECCERR_ADDR_ERR_FLASH_ADDR_MASK)
/* EFM_ECCERR_CNT Register */
#define EFM_ECCERR_CNT_ECCERR_CNT_MASK                                        (0x1FFFU)
#define EFM_ECCERR_CNT_ECCERR_CNT_SHIFT                                       (0U)
#define EFM_ECCERR_CNT_ECCERR_CNT(x)                                          (((uint32_t)(((uint32_t)(x)) << EFM_ECCERR_CNT_ECCERR_CNT_SHIFT)) & EFM_ECCERR_CNT_ECCERR_CNT_MASK)
/* EFM_CMD_UNLOCK Register */
#define EFM_CMD_UNLOCK_CMD_UNLOCK_KEY_MASK                                    (0xFFFFFFFFU)
#define EFM_CMD_UNLOCK_CMD_UNLOCK_KEY_SHIFT                                   (0U)
#define EFM_CMD_UNLOCK_CMD_UNLOCK_KEY(x)                                      (((uint32_t)(((uint32_t)(x)) << EFM_CMD_UNLOCK_CMD_UNLOCK_KEY_SHIFT)) & EFM_CMD_UNLOCK_CMD_UNLOCK_KEY_MASK)
/* EFM_CUS_KEY Register */
#define EFM_CUS_KEY_CUS_KEY_MATCH_MASK                                        (0x80000000U)
#define EFM_CUS_KEY_CUS_KEY_MATCH_SHIFT                                       (31U)
#define EFM_CUS_KEY_CUS_KEY_MATCH(x)                                          (((uint32_t)(((uint32_t)(x)) << EFM_CUS_KEY_CUS_KEY_MATCH_SHIFT)) & EFM_CUS_KEY_CUS_KEY_MATCH_MASK)
#define EFM_CUS_KEY_CUS_KEY_MISMATCH_MASK                                     (0x40000000U)
#define EFM_CUS_KEY_CUS_KEY_MISMATCH_SHIFT                                    (30U)
#define EFM_CUS_KEY_CUS_KEY_MISMATCH(x)                                       (((uint32_t)(((uint32_t)(x)) << EFM_CUS_KEY_CUS_KEY_MISMATCH_SHIFT)) & EFM_CUS_KEY_CUS_KEY_MISMATCH_MASK)
#define EFM_CUS_KEY_CUS_KEY_MASK                                              (0xFFFFFFU)
#define EFM_CUS_KEY_CUS_KEY_SHIFT                                             (0U)
#define EFM_CUS_KEY_CUS_KEY(x)                                                (((uint32_t)(((uint32_t)(x)) << EFM_CUS_KEY_CUS_KEY_SHIFT)) & EFM_CUS_KEY_CUS_KEY_MASK)
/* EFM_CTRL_OVRD Register */
#define EFM_CTRL_OVRD_PFLASH_PREREAD_DIS_MASK                                 (0x20U)
#define EFM_CTRL_OVRD_PFLASH_PREREAD_DIS_SHIFT                                (5U)
#define EFM_CTRL_OVRD_PFLASH_PREREAD_DIS(x)                                   (((uint32_t)(((uint32_t)(x)) << EFM_CTRL_OVRD_PFLASH_PREREAD_DIS_SHIFT)) & EFM_CTRL_OVRD_PFLASH_PREREAD_DIS_MASK)
#define EFM_CTRL_OVRD_DBG_DISABLE_MASK                                        (0x8U)
#define EFM_CTRL_OVRD_DBG_DISABLE_SHIFT                                       (3U)
#define EFM_CTRL_OVRD_DBG_DISABLE(x)                                          (((uint32_t)(((uint32_t)(x)) << EFM_CTRL_OVRD_DBG_DISABLE_SHIFT)) & EFM_CTRL_OVRD_DBG_DISABLE_MASK)
#define EFM_CTRL_OVRD_PFLASH_ECC_DIS_MASK                                     (0x1U)
#define EFM_CTRL_OVRD_PFLASH_ECC_DIS_SHIFT                                    (0U)
#define EFM_CTRL_OVRD_PFLASH_ECC_DIS(x)                                       (((uint32_t)(((uint32_t)(x)) << EFM_CTRL_OVRD_PFLASH_ECC_DIS_SHIFT)) & EFM_CTRL_OVRD_PFLASH_ECC_DIS_MASK)


/*!
 * @}
 */ /* end of group EFM_Register_Masks */


/*!
 * @}
 */ /* end of group EFM_Peripheral_Access_Layer */


/* -------------------------------------------------------------
   -- GPIO Peripheral Access Layer
   ------------------------------------------------------------- */

/*!
 * @addtogroup GPIO_Peripheral_Access_Layer GPIO Peripheral Access Layer
 * @{
 */


/** GPIO - Size of Registers Arrays */
#define GPIO_PCR_COUNT                                                        (12)

/* GPIO Register Layout Typedef */
typedef struct {
    __IO uint32_t PDOR;                                                       /**< Port Data Output Register, offset: 0x0000 */
    __O  uint32_t PSOR;                                                       /**< Port Set Output Register, offset: 0x0004 */
    __O  uint32_t PCOR;                                                       /**< Port Clear Output Register, offset: 0x0008 */
    __O  uint32_t PTOR;                                                       /**< Port Toggle Output Register, offset: 0x000C */
    __I  uint32_t PDIR;                                                       /**< Port Data Input Register, offset: 0x0010 */
    __IO uint32_t POER;                                                       /**< Port Output Enable Register, offset: 0x0014 */
    __IO uint32_t PIER;                                                       /**< Port Input Enable Register, offset: 0x0018 */
    __IO uint32_t PIFR;                                                       /**< Port Interrupt Status Flag Register, offset: 0x001C */
    __IO uint8_t  RESERVED8[96];                                              /**< RESERVED8, offset: 0x0020 */
    __IO uint32_t PCR[12];                                                    /**< Port Control Register, offset: 0x0080 */

} GPIO_Type, *GPIO_MemMapPtr;

/** Number of instances of the GPIO module. */
#define GPIO_INSTANCE_COUNT                                                   (1u)

/* GPIOA  base address */
#define GPIOA_BASE                                                            (0x4000F000)
#define GPIOA                                                                 ((GPIO_Type *)(GPIOA_BASE))

/** Array initializer of GPIO peripheral base addresses */
#define GPIO_BASE_ADDRS                                                       { GPIOA_BASE }
/** Array initializer of GPIO peripheral base pointers */
#define GPIO_BASE_PTRS                                                        { GPIOA }
/** Number of interrupt vector arrays for the GPIO module. */
#define GPIO_IRQS_ARR_COUNT                                                   (1u)
/** Number of interrupt channels for the GPIO. */
#define GPIO_IRQS_CH_COUNT                                                    (1u)
/** Interrupt vectors for the GPIO peripheral type */
#define GPIO_IRQS                                                             { GPIOA_IRQn, GPIOB_IRQn, GPIOC_IRQn, GPIOD_IRQn, GPIOE_IRQn }

/* GPIO Register Mask */
/* GPIO_PDOR Register */
#define GPIO_PDOR_PDO_MASK                                                    (0xFFFU)
#define GPIO_PDOR_PDO_SHIFT                                                   (0U)
#define GPIO_PDOR_PDO(x)                                                      (((uint32_t)(((uint32_t)(x)) << GPIO_PDOR_PDO_SHIFT)) & GPIO_PDOR_PDO_MASK)
/* GPIO_PSOR Register */
#define GPIO_PSOR_PSO_MASK                                                    (0xFFFU)
#define GPIO_PSOR_PSO_SHIFT                                                   (0U)
#define GPIO_PSOR_PSO(x)                                                      (((uint32_t)(((uint32_t)(x)) << GPIO_PSOR_PSO_SHIFT)) & GPIO_PSOR_PSO_MASK)
/* GPIO_PCOR Register */
#define GPIO_PCOR_PCO_MASK                                                    (0xFFFU)
#define GPIO_PCOR_PCO_SHIFT                                                   (0U)
#define GPIO_PCOR_PCO(x)                                                      (((uint32_t)(((uint32_t)(x)) << GPIO_PCOR_PCO_SHIFT)) & GPIO_PCOR_PCO_MASK)
/* GPIO_PTOR Register */
#define GPIO_PTOR_PTO_MASK                                                    (0xFFFU)
#define GPIO_PTOR_PTO_SHIFT                                                   (0U)
#define GPIO_PTOR_PTO(x)                                                      (((uint32_t)(((uint32_t)(x)) << GPIO_PTOR_PTO_SHIFT)) & GPIO_PTOR_PTO_MASK)
/* GPIO_PDIR Register */
#define GPIO_PDIR_PDI_MASK                                                    (0xFFFU)
#define GPIO_PDIR_PDI_SHIFT                                                   (0U)
#define GPIO_PDIR_PDI(x)                                                      (((uint32_t)(((uint32_t)(x)) << GPIO_PDIR_PDI_SHIFT)) & GPIO_PDIR_PDI_MASK)
/* GPIO_POER Register */
#define GPIO_POER_POE_MASK                                                    (0xFFFU)
#define GPIO_POER_POE_SHIFT                                                   (0U)
#define GPIO_POER_POE(x)                                                      (((uint32_t)(((uint32_t)(x)) << GPIO_POER_POE_SHIFT)) & GPIO_POER_POE_MASK)
/* GPIO_PIER Register */
#define GPIO_PIER_PIE_MASK                                                    (0xFFFU)
#define GPIO_PIER_PIE_SHIFT                                                   (0U)
#define GPIO_PIER_PIE(x)                                                      (((uint32_t)(((uint32_t)(x)) << GPIO_PIER_PIE_SHIFT)) & GPIO_PIER_PIE_MASK)
/* GPIO_PIFR Register */
#define GPIO_PIFR_PIF_MASK                                                    (0xFFFU)
#define GPIO_PIFR_PIF_SHIFT                                                   (0U)
#define GPIO_PIFR_PIF(x)                                                      (((uint32_t)(((uint32_t)(x)) << GPIO_PIFR_PIF_SHIFT)) & GPIO_PIFR_PIF_MASK)
/* GPIO_PCR Register */
#define GPIO_PCR_IRQC_MASK                                                    (0xFU)
#define GPIO_PCR_IRQC_SHIFT                                                   (0U)
#define GPIO_PCR_IRQC(x)                                                      (((uint32_t)(((uint32_t)(x)) << GPIO_PCR_IRQC_SHIFT)) & GPIO_PCR_IRQC_MASK)


/*!
 * @}
 */ /* end of group GPIO_Register_Masks */


/*!
 * @}
 */ /* end of group GPIO_Peripheral_Access_Layer */


/* -------------------------------------------------------------
   -- PCTRL Peripheral Access Layer
   ------------------------------------------------------------- */

/*!
 * @addtogroup PCTRL_Peripheral_Access_Layer PCTRL Peripheral Access Layer
 * @{
 */


/** PCTRL - Size of Registers Arrays */
#define PCTRL_PCR_COUNT                                                       (11)

/* PCTRL Register Layout Typedef */
typedef struct {
    __IO uint32_t PCR[11];                                                    /**< Port Control Register, offset: 0x0000 */

} PCTRL_Type, *PCTRL_MemMapPtr;

/** Number of instances of the PCTRL module. */
#define PCTRL_INSTANCE_COUNT                                                  (1u)

/* PCTRLA  base address */
#define PCTRLA_BASE                                                           (0x40049000)
#define PCTRLA                                                                ((PCTRL_Type *)(PCTRLA_BASE))

/** Array initializer of PCTRL peripheral base addresses */
#define PCTRL_BASE_ADDRS                                                      { PCTRLA_BASE }
/** Array initializer of PCTRL peripheral base pointers */
#define PCTRL_BASE_PTRS                                                       { PCTRLA }

/* PCTRL Register Mask */
/* PCTRL_PCR Register */
#define PCTRL_PCR_LOCK_MASK                                                   (0x80000000U)
#define PCTRL_PCR_LOCK_SHIFT                                                  (31U)
#define PCTRL_PCR_LOCK(x)                                                     (((uint32_t)(((uint32_t)(x)) << PCTRL_PCR_LOCK_SHIFT)) & PCTRL_PCR_LOCK_MASK)
#define PCTRL_PCR_MUX_WIDTH                                                   (4)
#define PCTRL_PCR_MUX_MASK                                                    (0xF00U)
#define PCTRL_PCR_MUX_SHIFT                                                   (8U)
#define PCTRL_PCR_MUX(x)                                                      (((uint32_t)(((uint32_t)(x)) << PCTRL_PCR_MUX_SHIFT)) & PCTRL_PCR_MUX_MASK)
#define PCTRL_PCR_ODE_MASK                                                    (0x20U)
#define PCTRL_PCR_ODE_SHIFT                                                   (5U)
#define PCTRL_PCR_ODE(x)                                                      (((uint32_t)(((uint32_t)(x)) << PCTRL_PCR_ODE_SHIFT)) & PCTRL_PCR_ODE_MASK)
#define PCTRL_PCR_PE_MASK                                                     (0x2U)
#define PCTRL_PCR_PE_SHIFT                                                    (1U)
#define PCTRL_PCR_PE(x)                                                       (((uint32_t)(((uint32_t)(x)) << PCTRL_PCR_PE_SHIFT)) & PCTRL_PCR_PE_MASK)
#define PCTRL_PCR_PS_MASK                                                     (0x1U)
#define PCTRL_PCR_PS_SHIFT                                                    (0U)
#define PCTRL_PCR_PS(x)                                                       (((uint32_t)(((uint32_t)(x)) << PCTRL_PCR_PS_SHIFT)) & PCTRL_PCR_PS_MASK)


/*!
 * @}
 */ /* end of group PCTRL_Register_Masks */


/*!
 * @}
 */ /* end of group PCTRL_Peripheral_Access_Layer */


/* -------------------------------------------------------------
   -- DIVSQRT Peripheral Access Layer
   ------------------------------------------------------------- */

/*!
 * @addtogroup DIVSQRT_Peripheral_Access_Layer DIVSQRT Peripheral Access Layer
 * @{
 */


/** DIVSQRT - Size of Registers Arrays */

/* DIVSQRT Register Layout Typedef */
typedef struct {
    __O  uint32_t DEND;                                                       /**< Dividend Register, offset: 0x0000 */
    __O  uint32_t DSOR;                                                       /**< Divisor Register, offset: 0x0004 */
    __IO uint32_t CSR;                                                        /**< Control and Status Register, offset: 0x0008 */
    __IO uint32_t RES;                                                        /**< Result Register, offset: 0x000C */
    __O  uint32_t RCND;                                                       /**< Radicand Register, offset: 0x0010 */

} DIVSQRT_Type, *DIVSQRT_MemMapPtr;

/** Number of instances of the DIVSQRT module. */
#define DIVSQRT_INSTANCE_COUNT                                                (1u)

/* DIVSQRT base address */
#define DIVSQRT_BASE                                                          (0x4000E000)
#define DIVSQRT                                                               ((DIVSQRT_Type *)(DIVSQRT_BASE))

/** Array initializer of DIVSQRT peripheral base addresses */
#define DIVSQRT_BASE_ADDRS                                                    { DIVSQRT_BASE }
/** Array initializer of DIVSQRT peripheral base pointers */
#define DIVSQRT_BASE_PTRS                                                     { DIVSQRT }
/** Number of interrupt vector arrays for the DIVSQRT module. */
#define DIVSQRT_IRQS_ARR_COUNT                                                (1u)
/** Number of interrupt channels for the DIVSQRT. */
#define DIVSQRT_IRQS_CH_COUNT                                                 (1u)
/** Interrupt vectors for the DIVSQRT peripheral type */
#define DIVSQRT_IRQS                                                          { DIVSQRT_IRQn }

/* DIVSQRT Register Mask */
/* DIVSQRT_DEND Register */
#define DIVSQRT_DEND_DIVIDEND_MASK                                            (0xFFFFFFFFU)
#define DIVSQRT_DEND_DIVIDEND_SHIFT                                           (0U)
#define DIVSQRT_DEND_DIVIDEND(x)                                              (((uint32_t)(((uint32_t)(x)) << DIVSQRT_DEND_DIVIDEND_SHIFT)) & DIVSQRT_DEND_DIVIDEND_MASK)
/* DIVSQRT_DSOR Register */
#define DIVSQRT_DSOR_DIVISOR_MASK                                             (0xFFFFFFFFU)
#define DIVSQRT_DSOR_DIVISOR_SHIFT                                            (0U)
#define DIVSQRT_DSOR_DIVISOR(x)                                               (((uint32_t)(((uint32_t)(x)) << DIVSQRT_DSOR_DIVISOR_SHIFT)) & DIVSQRT_DSOR_DIVISOR_MASK)
/* DIVSQRT_CSR Register */
#define DIVSQRT_CSR_BUSY_MASK                                                 (0x80000000U)
#define DIVSQRT_CSR_BUSY_SHIFT                                                (31U)
#define DIVSQRT_CSR_BUSY(x)                                                   (((uint32_t)(((uint32_t)(x)) << DIVSQRT_CSR_BUSY_SHIFT)) & DIVSQRT_CSR_BUSY_MASK)
#define DIVSQRT_CSR_DIV_MASK                                                  (0x40000000U)
#define DIVSQRT_CSR_DIV_SHIFT                                                 (30U)
#define DIVSQRT_CSR_DIV(x)                                                    (((uint32_t)(((uint32_t)(x)) << DIVSQRT_CSR_DIV_SHIFT)) & DIVSQRT_CSR_DIV_MASK)
#define DIVSQRT_CSR_SQRT_MASK                                                 (0x20000000U)
#define DIVSQRT_CSR_SQRT_SHIFT                                                (29U)
#define DIVSQRT_CSR_SQRT(x)                                                   (((uint32_t)(((uint32_t)(x)) << DIVSQRT_CSR_SQRT_SHIFT)) & DIVSQRT_CSR_SQRT_MASK)
#define DIVSQRT_CSR_DFS_MASK                                                  (0x20U)
#define DIVSQRT_CSR_DFS_SHIFT                                                 (5U)
#define DIVSQRT_CSR_DFS(x)                                                    (((uint32_t)(((uint32_t)(x)) << DIVSQRT_CSR_DFS_SHIFT)) & DIVSQRT_CSR_DFS_MASK)
#define DIVSQRT_CSR_DZ_MASK                                                   (0x10U)
#define DIVSQRT_CSR_DZ_SHIFT                                                  (4U)
#define DIVSQRT_CSR_DZ(x)                                                     (((uint32_t)(((uint32_t)(x)) << DIVSQRT_CSR_DZ_SHIFT)) & DIVSQRT_CSR_DZ_MASK)
#define DIVSQRT_CSR_DZE_MASK                                                  (0x8U)
#define DIVSQRT_CSR_DZE_SHIFT                                                 (3U)
#define DIVSQRT_CSR_DZE(x)                                                    (((uint32_t)(((uint32_t)(x)) << DIVSQRT_CSR_DZE_SHIFT)) & DIVSQRT_CSR_DZE_MASK)
#define DIVSQRT_CSR_REM_MASK                                                  (0x4U)
#define DIVSQRT_CSR_REM_SHIFT                                                 (2U)
#define DIVSQRT_CSR_REM(x)                                                    (((uint32_t)(((uint32_t)(x)) << DIVSQRT_CSR_REM_SHIFT)) & DIVSQRT_CSR_REM_MASK)
#define DIVSQRT_CSR_USGN_MASK                                                 (0x2U)
#define DIVSQRT_CSR_USGN_SHIFT                                                (1U)
#define DIVSQRT_CSR_USGN(x)                                                   (((uint32_t)(((uint32_t)(x)) << DIVSQRT_CSR_USGN_SHIFT)) & DIVSQRT_CSR_USGN_MASK)
#define DIVSQRT_CSR_SRT_MASK                                                  (0x1U)
#define DIVSQRT_CSR_SRT_SHIFT                                                 (0U)
#define DIVSQRT_CSR_SRT(x)                                                    (((uint32_t)(((uint32_t)(x)) << DIVSQRT_CSR_SRT_SHIFT)) & DIVSQRT_CSR_SRT_MASK)
/* DIVSQRT_RES Register */
#define DIVSQRT_RES_RESULT_MASK                                               (0xFFFFFFFFU)
#define DIVSQRT_RES_RESULT_SHIFT                                              (0U)
#define DIVSQRT_RES_RESULT(x)                                                 (((uint32_t)(((uint32_t)(x)) << DIVSQRT_RES_RESULT_SHIFT)) & DIVSQRT_RES_RESULT_MASK)
/* DIVSQRT_RCND Register */
#define DIVSQRT_RCND_RADICAND_MASK                                            (0xFFFFFFFFU)
#define DIVSQRT_RCND_RADICAND_SHIFT                                           (0U)
#define DIVSQRT_RCND_RADICAND(x)                                              (((uint32_t)(((uint32_t)(x)) << DIVSQRT_RCND_RADICAND_SHIFT)) & DIVSQRT_RCND_RADICAND_MASK)


/*!
 * @}
 */ /* end of group DIVSQRT_Register_Masks */


/*!
 * @}
 */ /* end of group DIVSQRT_Peripheral_Access_Layer */


/* -------------------------------------------------------------
   -- TMU Peripheral Access Layer
   ------------------------------------------------------------- */

/*!
 * @addtogroup TMU_Peripheral_Access_Layer TMU Peripheral Access Layer
 * @{
 */


/** TMU - Size of Registers Arrays */
#define TMU_MUX_COUNT                                                         (4)

/* TMU Register Layout Typedef */
typedef struct {
    __IO uint32_t MUX[ 4];                                                    /**< MUX, offset: 0x0000 */

} TMU_Type, *TMU_MemMapPtr;

/** Number of instances of the TMU module. */
#define TMU_INSTANCE_COUNT                                                    (1u)

/* TMU base address */
#define TMU_BASE                                                              (0x40063000)
#define TMU                                                                   ((TMU_Type *)(TMU_BASE))

/** Array initializer of TMU peripheral base addresses */
#define TMU_BASE_ADDRS                                                        { TMU_BASE }
/** Array initializer of TMU peripheral base pointers */
#define TMU_BASE_PTRS                                                         { TMU }

/* TMU Register Mask */
/* TMU_MUX Register */
#define TMU_MUX_LOCK_MASK                                                     (0x80000000U)
#define TMU_MUX_LOCK_SHIFT                                                    (31U)
#define TMU_MUX_LOCK(x)                                                       (((uint32_t)(((uint32_t)(x)) << TMU_MUX_LOCK_SHIFT)) & TMU_MUX_LOCK_MASK)
#define TMU_MUX_SEL3_WIDTH                                                    (5)
#define TMU_MUX_SEL3_MASK                                                     (0x1F000000U)
#define TMU_MUX_SEL3_SHIFT                                                    (24U)
#define TMU_MUX_SEL3(x)                                                       (((uint32_t)(((uint32_t)(x)) << TMU_MUX_SEL3_SHIFT)) & TMU_MUX_SEL3_MASK)
#define TMU_MUX_SEL2_WIDTH                                                    (5)
#define TMU_MUX_SEL2_MASK                                                     (0x1F0000U)
#define TMU_MUX_SEL2_SHIFT                                                    (16U)
#define TMU_MUX_SEL2(x)                                                       (((uint32_t)(((uint32_t)(x)) << TMU_MUX_SEL2_SHIFT)) & TMU_MUX_SEL2_MASK)
#define TMU_MUX_SEL1_WIDTH                                                    (5)
#define TMU_MUX_SEL1_MASK                                                     (0x1F00U)
#define TMU_MUX_SEL1_SHIFT                                                    (8U)
#define TMU_MUX_SEL1(x)                                                       (((uint32_t)(((uint32_t)(x)) << TMU_MUX_SEL1_SHIFT)) & TMU_MUX_SEL1_MASK)
#define TMU_MUX_SEL0_WIDTH                                                    (5)
#define TMU_MUX_SEL0_MASK                                                     (0x1FU)
#define TMU_MUX_SEL0_SHIFT                                                    (0U)
#define TMU_MUX_SEL0(x)                                                       (((uint32_t)(((uint32_t)(x)) << TMU_MUX_SEL0_SHIFT)) & TMU_MUX_SEL0_MASK)


/*!
 * @}
 */ /* end of group TMU_Register_Masks */


/*!
 * @}
 */ /* end of group TMU_Peripheral_Access_Layer */


/* -------------------------------------------------------------
   -- CIM Peripheral Access Layer
   ------------------------------------------------------------- */

/*!
 * @addtogroup CIM_Peripheral_Access_Layer CIM Peripheral Access Layer
 * @{
 */


/** CIM - Size of Registers Arrays */

/* CIM Register Layout Typedef */
typedef struct {
    __I  uint32_t DIEINFO;                                                    /**< Die Information Register, offset: 0x0000 */
    __IO uint8_t  RESERVED1[ 4];                                              /**< RESERVED1, offset: 0x0004 */
    __I  uint32_t CHIPINFO;                                                   /**< Chip Information Register, offset: 0x0008 */
    __IO uint8_t  RESERVED3[36];                                              /**< RESERVED3, offset: 0x000C */
    __IO uint32_t CTRL;                                                       /**< Control Register, offset: 0x0030 */
    __IO uint32_t SWTRIG;                                                     /**< Software Trigger Register, offset: 0x0034 */
    __IO uint32_t RESERVED6;                                                  /**< RESERVED6, offset: 0x0038 */

} CIM_Type, *CIM_MemMapPtr;

/** Number of instances of the CIM module. */
#define CIM_INSTANCE_COUNT                                                    (1u)

/* CIM base address */
#define CIM_BASE                                                              (0x40048000)
#define CIM                                                                   ((CIM_Type *)(CIM_BASE))

/** Array initializer of CIM peripheral base addresses */
#define CIM_BASE_ADDRS                                                        { CIM_BASE }
/** Array initializer of CIM peripheral base pointers */
#define CIM_BASE_PTRS                                                         { CIM }

/* CIM Register Mask */
/* CIM_DIEINFO Register */
#define CIM_DIEINFO_FAMID_MASK                                                (0xF0000000U)
#define CIM_DIEINFO_FAMID_SHIFT                                               (28U)
#define CIM_DIEINFO_FAMID(x)                                                  (((uint32_t)(((uint32_t)(x)) << CIM_DIEINFO_FAMID_SHIFT)) & CIM_DIEINFO_FAMID_MASK)
#define CIM_DIEINFO_GENID_MASK                                                (0xF000000U)
#define CIM_DIEINFO_GENID_SHIFT                                               (24U)
#define CIM_DIEINFO_GENID(x)                                                  (((uint32_t)(((uint32_t)(x)) << CIM_DIEINFO_GENID_SHIFT)) & CIM_DIEINFO_GENID_MASK)
#define CIM_DIEINFO_SUBFAMIDH_MASK                                            (0xF00000U)
#define CIM_DIEINFO_SUBFAMIDH_SHIFT                                           (20U)
#define CIM_DIEINFO_SUBFAMIDH(x)                                              (((uint32_t)(((uint32_t)(x)) << CIM_DIEINFO_SUBFAMIDH_SHIFT)) & CIM_DIEINFO_SUBFAMIDH_MASK)
#define CIM_DIEINFO_SUBFAMIDL_MASK                                            (0xF0000U)
#define CIM_DIEINFO_SUBFAMIDL_SHIFT                                           (16U)
#define CIM_DIEINFO_SUBFAMIDL(x)                                              (((uint32_t)(((uint32_t)(x)) << CIM_DIEINFO_SUBFAMIDL_SHIFT)) & CIM_DIEINFO_SUBFAMIDL_MASK)
#define CIM_DIEINFO_MAJVER_MASK                                               (0xF0U)
#define CIM_DIEINFO_MAJVER_SHIFT                                              (4U)
#define CIM_DIEINFO_MAJVER(x)                                                 (((uint32_t)(((uint32_t)(x)) << CIM_DIEINFO_MAJVER_SHIFT)) & CIM_DIEINFO_MAJVER_MASK)
#define CIM_DIEINFO_MINVER_MASK                                               (0xFU)
#define CIM_DIEINFO_MINVER_SHIFT                                              (0U)
#define CIM_DIEINFO_MINVER(x)                                                 (((uint32_t)(((uint32_t)(x)) << CIM_DIEINFO_MINVER_SHIFT)) & CIM_DIEINFO_MINVER_MASK)
/* CIM_CHIPINFO Register */
#define CIM_CHIPINFO_PKGS_MASK                                                (0xF00000U)
#define CIM_CHIPINFO_PKGS_SHIFT                                               (20U)
#define CIM_CHIPINFO_PKGS(x)                                                  (((uint32_t)(((uint32_t)(x)) << CIM_CHIPINFO_PKGS_SHIFT)) & CIM_CHIPINFO_PKGS_MASK)
#define CIM_CHIPINFO_SRAMS_MASK                                               (0xF0000U)
#define CIM_CHIPINFO_SRAMS_SHIFT                                              (16U)
#define CIM_CHIPINFO_SRAMS(x)                                                 (((uint32_t)(((uint32_t)(x)) << CIM_CHIPINFO_SRAMS_SHIFT)) & CIM_CHIPINFO_SRAMS_MASK)
#define CIM_CHIPINFO_PFLASHS_MASK                                             (0xF00U)
#define CIM_CHIPINFO_PFLASHS_SHIFT                                            (8U)
#define CIM_CHIPINFO_PFLASHS(x)                                               (((uint32_t)(((uint32_t)(x)) << CIM_CHIPINFO_PFLASHS_SHIFT)) & CIM_CHIPINFO_PFLASHS_MASK)
/* CIM_CTRL Register */
#define CIM_CTRL_CAPSEL_MASK                                                  (0x80U)
#define CIM_CTRL_CAPSEL_SHIFT                                                 (7U)
#define CIM_CTRL_CAPSEL(x)                                                    (((uint32_t)(((uint32_t)(x)) << CIM_CTRL_CAPSEL_SHIFT)) & CIM_CTRL_CAPSEL_MASK)
#define CIM_CTRL_LIN_GPIO_SEL_MASK                                            (0x30U)
#define CIM_CTRL_LIN_GPIO_SEL_SHIFT                                           (4U)
#define CIM_CTRL_LIN_GPIO_SEL(x)                                              (((uint32_t)(((uint32_t)(x)) << CIM_CTRL_LIN_GPIO_SEL_SHIFT)) & CIM_CTRL_LIN_GPIO_SEL_MASK)
#define CIM_CTRL_LOCKUPEN_MASK                                                (0x4U)
#define CIM_CTRL_LOCKUPEN_SHIFT                                               (2U)
#define CIM_CTRL_LOCKUPEN(x)                                                  (((uint32_t)(((uint32_t)(x)) << CIM_CTRL_LOCKUPEN_SHIFT)) & CIM_CTRL_LOCKUPEN_MASK)
#define CIM_CTRL_WICENREQ_MASK                                                (0x1U)
#define CIM_CTRL_WICENREQ_SHIFT                                               (0U)
#define CIM_CTRL_WICENREQ(x)                                                  (((uint32_t)(((uint32_t)(x)) << CIM_CTRL_WICENREQ_SHIFT)) & CIM_CTRL_WICENREQ_MASK)
/* CIM_SWTRIG Register */
#define CIM_SWTRIG_SWTCNT_MASK                                                (0xFEU)
#define CIM_SWTRIG_SWTCNT_SHIFT                                               (1U)
#define CIM_SWTRIG_SWTCNT(x)                                                  (((uint32_t)(((uint32_t)(x)) << CIM_SWTRIG_SWTCNT_SHIFT)) & CIM_SWTRIG_SWTCNT_MASK)
#define CIM_SWTRIG_SWT_MASK                                                   (0x1U)
#define CIM_SWTRIG_SWT_SHIFT                                                  (0U)
#define CIM_SWTRIG_SWT(x)                                                     (((uint32_t)(((uint32_t)(x)) << CIM_SWTRIG_SWT_SHIFT)) & CIM_SWTRIG_SWT_MASK)


/*!
 * @}
 */ /* end of group CIM_Register_Masks */


/*!
 * @}
 */ /* end of group CIM_Peripheral_Access_Layer */


/* -------------------------------------------------------------
   -- SCU Peripheral Access Layer
   ------------------------------------------------------------- */

/*!
 * @addtogroup SCU_Peripheral_Access_Layer SCU Peripheral Access Layer
 * @{
 */


/** SCU - Size of Registers Arrays */

/* SCU Register Layout Typedef */
typedef struct {
    __IO uint32_t CLKS;                                                       /**< Clock Selection Register, offset: 0x0000 */
    __I  uint32_t STS;                                                        /**< Status Register, offset: 0x0004 */
    __IO uint32_t OSC_CTRL;                                                   /**< OSC Control Register, offset: 0x0008 */
    __IO uint32_t PLL_CTRL;                                                   /**< PLL Control Register, offset: 0x000C */
    __IO uint32_t PLL_CFG;                                                    /**< PLL Configuration Register, offset: 0x0010 */
    __IO uint32_t PLL_FM;                                                     /**< PLL Spread Spectrum Mode Register, offset: 0x0014 */
    __IO uint32_t PLL_FD;                                                     /**< PLL Fractional Mode Register, offset: 0x0018 */
    __IO uint32_t RESERVED7[ 5];                                              /**< RESERVED7, offset: 0x001C */
    __IO uint32_t RESERVED8;                                                  /**< RESERVED8, offset: 0x0030 */
    __IO uint32_t RESERVED9;                                                  /**< RESERVED9, offset: 0x0034 */
    __IO uint8_t  RESERVED10[ 4];                                             /**< RESERVED10, offset: 0x0038 */
    __IO uint32_t RESERVED11;                                                 /**< RESERVED11, offset: 0x003C */

} SCU_Type, *SCU_MemMapPtr;

/** Number of instances of the SCU module. */
#define SCU_INSTANCE_COUNT                                                    (1u)

/* SCU base address */
#define SCU_BASE                                                              (0x40064000)
#define SCU                                                                   ((SCU_Type *)(SCU_BASE))

/** Array initializer of SCU peripheral base addresses */
#define SCU_BASE_ADDRS                                                        { SCU_BASE }
/** Array initializer of SCU peripheral base pointers */
#define SCU_BASE_PTRS                                                         { SCU }
/** Number of interrupt vector arrays for the SCU module. */
#define SCU_IRQS_ARR_COUNT                                                    (1u)
/** Number of interrupt channels for the SCU. */
#define SCU_IRQS_CH_COUNT                                                     (1u)
/** Interrupt vectors for the SCU peripheral type */
#define SCU_IRQS                                                              { SCU_IRQn }

/* SCU Register Mask */
/* SCU_CLKS Register */
#define SCU_CLKS_CLKSRCSEL_MASK                                               (0x1U)
#define SCU_CLKS_CLKSRCSEL_SHIFT                                              (0U)
#define SCU_CLKS_CLKSRCSEL(x)                                                 (((uint32_t)(((uint32_t)(x)) << SCU_CLKS_CLKSRCSEL_SHIFT)) & SCU_CLKS_CLKSRCSEL_MASK)
/* SCU_STS Register */
#define SCU_STS_PLL_LOCK_MASK                                                 (0x20U)
#define SCU_STS_PLL_LOCK_SHIFT                                                (5U)
#define SCU_STS_PLL_LOCK(x)                                                   (((uint32_t)(((uint32_t)(x)) << SCU_STS_PLL_LOCK_SHIFT)) & SCU_STS_PLL_LOCK_MASK)
#define SCU_STS_CLKST_MASK                                                    (0x1U)
#define SCU_STS_CLKST_SHIFT                                                   (0U)
#define SCU_STS_CLKST(x)                                                      (((uint32_t)(((uint32_t)(x)) << SCU_STS_CLKST_SHIFT)) & SCU_STS_CLKST_MASK)
/* SCU_OSC_CTRL Register */
#define SCU_OSC_CTRL_OSC_DS_EN_MASK                                           (0x2U)
#define SCU_OSC_CTRL_OSC_DS_EN_SHIFT                                          (1U)
#define SCU_OSC_CTRL_OSC_DS_EN(x)                                             (((uint32_t)(((uint32_t)(x)) << SCU_OSC_CTRL_OSC_DS_EN_SHIFT)) & SCU_OSC_CTRL_OSC_DS_EN_MASK)
/* SCU_PLL_CTRL Register */
#define SCU_PLL_CTRL_PLL_NDIV_MASK                                            (0x3F00U)
#define SCU_PLL_CTRL_PLL_NDIV_SHIFT                                           (8U)
#define SCU_PLL_CTRL_PLL_NDIV(x)                                              (((uint32_t)(((uint32_t)(x)) << SCU_PLL_CTRL_PLL_NDIV_SHIFT)) & SCU_PLL_CTRL_PLL_NDIV_MASK)
#define SCU_PLL_CTRL_UNLOCK_RE_MASK                                           (0x20U)
#define SCU_PLL_CTRL_UNLOCK_RE_SHIFT                                          (5U)
#define SCU_PLL_CTRL_UNLOCK_RE(x)                                             (((uint32_t)(((uint32_t)(x)) << SCU_PLL_CTRL_UNLOCK_RE_SHIFT)) & SCU_PLL_CTRL_UNLOCK_RE_MASK)
#define SCU_PLL_CTRL_PLL_OVRD_EN_MASK                                         (0x4U)
#define SCU_PLL_CTRL_PLL_OVRD_EN_SHIFT                                        (2U)
#define SCU_PLL_CTRL_PLL_OVRD_EN(x)                                           (((uint32_t)(((uint32_t)(x)) << SCU_PLL_CTRL_PLL_OVRD_EN_SHIFT)) & SCU_PLL_CTRL_PLL_OVRD_EN_MASK)
#define SCU_PLL_CTRL_PLL_DS_EN_MASK                                           (0x2U)
#define SCU_PLL_CTRL_PLL_DS_EN_SHIFT                                          (1U)
#define SCU_PLL_CTRL_PLL_DS_EN(x)                                             (((uint32_t)(((uint32_t)(x)) << SCU_PLL_CTRL_PLL_DS_EN_SHIFT)) & SCU_PLL_CTRL_PLL_DS_EN_MASK)
#define SCU_PLL_CTRL_PLL_EN_MASK                                              (0x1U)
#define SCU_PLL_CTRL_PLL_EN_SHIFT                                             (0U)
#define SCU_PLL_CTRL_PLL_EN(x)                                                (((uint32_t)(((uint32_t)(x)) << SCU_PLL_CTRL_PLL_EN_SHIFT)) & SCU_PLL_CTRL_PLL_EN_MASK)
/* SCU_PLL_CFG Register */
#define SCU_PLL_CFG_DITHER_GAIN_MASK                                          (0x300U)
#define SCU_PLL_CFG_DITHER_GAIN_SHIFT                                         (8U)
#define SCU_PLL_CFG_DITHER_GAIN(x)                                            (((uint32_t)(((uint32_t)(x)) << SCU_PLL_CFG_DITHER_GAIN_SHIFT)) & SCU_PLL_CFG_DITHER_GAIN_MASK)
#define SCU_PLL_CFG_DITHER_EN_MASK                                            (0x4U)
#define SCU_PLL_CFG_DITHER_EN_SHIFT                                           (2U)
#define SCU_PLL_CFG_DITHER_EN(x)                                              (((uint32_t)(((uint32_t)(x)) << SCU_PLL_CFG_DITHER_EN_SHIFT)) & SCU_PLL_CFG_DITHER_EN_MASK)
#define SCU_PLL_CFG_FM_EN_MASK                                                (0x2U)
#define SCU_PLL_CFG_FM_EN_SHIFT                                               (1U)
#define SCU_PLL_CFG_FM_EN(x)                                                  (((uint32_t)(((uint32_t)(x)) << SCU_PLL_CFG_FM_EN_SHIFT)) & SCU_PLL_CFG_FM_EN_MASK)
#define SCU_PLL_CFG_FD_EN_MASK                                                (0x1U)
#define SCU_PLL_CFG_FD_EN_SHIFT                                               (0U)
#define SCU_PLL_CFG_FD_EN(x)                                                  (((uint32_t)(((uint32_t)(x)) << SCU_PLL_CFG_FD_EN_SHIFT)) & SCU_PLL_CFG_FD_EN_MASK)
/* SCU_PLL_FM Register */
#define SCU_PLL_FM_FM_STEP_MASK                                               (0x7FF0000U)
#define SCU_PLL_FM_FM_STEP_SHIFT                                              (16U)
#define SCU_PLL_FM_FM_STEP(x)                                                 (((uint32_t)(((uint32_t)(x)) << SCU_PLL_FM_FM_STEP_SHIFT)) & SCU_PLL_FM_FM_STEP_MASK)
/* SCU_PLL_FD Register */
#define SCU_PLL_FD_FD_FRAC_MASK                                               (0x7FFFU)
#define SCU_PLL_FD_FD_FRAC_SHIFT                                              (0U)
#define SCU_PLL_FD_FD_FRAC(x)                                                 (((uint32_t)(((uint32_t)(x)) << SCU_PLL_FD_FD_FRAC_SHIFT)) & SCU_PLL_FD_FD_FRAC_MASK)


/*!
 * @}
 */ /* end of group SCU_Register_Masks */


/*!
 * @}
 */ /* end of group SCU_Peripheral_Access_Layer */


/* -------------------------------------------------------------
   -- IPC Peripheral Access Layer
   ------------------------------------------------------------- */

/*!
 * @addtogroup IPC_Peripheral_Access_Layer IPC Peripheral Access Layer
 * @{
 */


/** IPC - Size of Registers Arrays */
#define IPC_CTRL_COUNT                                                        (128)

/* IPC Register Layout Typedef */
typedef struct {
    __IO uint32_t CTRL[128];                                                  /**< Control Register, offset: 0x0000 */

} IPC_Type, *IPC_MemMapPtr;

/** Number of instances of the IPC module. */
#define IPC_INSTANCE_COUNT                                                    (1u)

/* IPC base address */
#define IPC_BASE                                                              (0x40065000)
#define IPC                                                                   ((IPC_Type *)(IPC_BASE))

/** Array initializer of IPC peripheral base addresses */
#define IPC_BASE_ADDRS                                                        { IPC_BASE }
/** Array initializer of IPC peripheral base pointers */
#define IPC_BASE_PTRS                                                         { IPC }

/* IPC Register Mask */
/* IPC_CTRL Register */
#define IPC_CTRL_SRCSEL_MASK                                                  (0x300U)
#define IPC_CTRL_SRCSEL_SHIFT                                                 (8U)
#define IPC_CTRL_SRCSEL(x)                                                    (((uint32_t)(((uint32_t)(x)) << IPC_CTRL_SRCSEL_SHIFT)) & IPC_CTRL_SRCSEL_MASK)
#define IPC_CTRL_SWREN_MASK                                                   (0x2U)
#define IPC_CTRL_SWREN_SHIFT                                                  (1U)
#define IPC_CTRL_SWREN(x)                                                     (((uint32_t)(((uint32_t)(x)) << IPC_CTRL_SWREN_SHIFT)) & IPC_CTRL_SWREN_MASK)
#define IPC_CTRL_CLKEN_MASK                                                   (0x1U)
#define IPC_CTRL_CLKEN_SHIFT                                                  (0U)
#define IPC_CTRL_CLKEN(x)                                                     (((uint32_t)(((uint32_t)(x)) << IPC_CTRL_CLKEN_SHIFT)) & IPC_CTRL_CLKEN_MASK)


/*!
 * @}
 */ /* end of group IPC_Register_Masks */


/*!
 * @}
 */ /* end of group IPC_Peripheral_Access_Layer */


/* -------------------------------------------------------------
   -- PCU Peripheral Access Layer
   ------------------------------------------------------------- */

/*!
 * @addtogroup PCU_Peripheral_Access_Layer PCU Peripheral Access Layer
 * @{
 */


/** PCU - Size of Registers Arrays */

/* PCU Register Layout Typedef */
typedef struct {
    __IO uint32_t CR;                                                         /**< Configure Register, offset: 0x0000 */
    __IO uint32_t SR;                                                         /**< Status Register, offset: 0x0004 */
    __IO uint32_t IE;                                                         /**< Interrupt Enable Register, offset: 0x0008 */
    __IO uint32_t LDOCR;                                                      /**< LDO Configure Register, offset: 0x000C */
    __IO uint32_t VSMCR;                                                      /**< VSM Configure Register, offset: 0x0010 */
    __IO uint32_t OVTCR;                                                      /**< Over Temperature Detect Configure Register, offset: 0x0014 */
    __IO uint32_t OCCR;                                                       /**< Driver Over Current Detect Configure Register, offset: 0x0018 */
    __IO uint8_t  RESERVED7[ 8];                                              /**< RESERVED7, offset: 0x001C */
    __IO uint32_t RESERVED8;                                                  /**< RESERVED8, offset: 0x0024 */
    __IO uint32_t RESERVED9;                                                  /**< RESERVED9, offset: 0x0028 */
    __IO uint32_t VDDLDOCR;                                                   /**< LDO Control Register, offset: 0x002C */
    __IO uint32_t RESERVED11;                                                 /**< RESERVED11, offset: 0x0030 */
    __IO uint32_t RESERVED12;                                                 /**< RESERVED12, offset: 0x0034 */
    __IO uint32_t RESERVED13;                                                 /**< RESERVED13, offset: 0x0038 */

} PCU_Type, *PCU_MemMapPtr;

/** Number of instances of the PCU module. */
#define PCU_INSTANCE_COUNT                                                    (1u)

/* PCU base address */
#define PCU_BASE                                                              (0x4007D000)
#define PCU                                                                   ((PCU_Type *)(PCU_BASE))

/** Array initializer of PCU peripheral base addresses */
#define PCU_BASE_ADDRS                                                        { PCU_BASE }
/** Array initializer of PCU peripheral base pointers */
#define PCU_BASE_PTRS                                                         { PCU }

/* PCU Register Mask */
/* PCU_CR Register */
#define PCU_CR_INT_WU_SET_MASK                                                (0xCU)
#define PCU_CR_INT_WU_SET_SHIFT                                               (2U)
#define PCU_CR_INT_WU_SET(x)                                                  (((uint32_t)(((uint32_t)(x)) << PCU_CR_INT_WU_SET_SHIFT)) & PCU_CR_INT_WU_SET_MASK)
#define PCU_CR_EN_WU_TIM_MASK                                                 (0x2U)
#define PCU_CR_EN_WU_TIM_SHIFT                                                (1U)
#define PCU_CR_EN_WU_TIM(x)                                                   (((uint32_t)(((uint32_t)(x)) << PCU_CR_EN_WU_TIM_SHIFT)) & PCU_CR_EN_WU_TIM_MASK)
#define PCU_CR_DPD_EN_MASK                                                    (0x1U)
#define PCU_CR_DPD_EN_SHIFT                                                   (0U)
#define PCU_CR_DPD_EN(x)                                                      (((uint32_t)(((uint32_t)(x)) << PCU_CR_DPD_EN_SHIFT)) & PCU_CR_DPD_EN_MASK)
/* PCU_SR Register */
#define PCU_SR_VDDA_UV_STS_MASK                                               (0x40000000U)
#define PCU_SR_VDDA_UV_STS_SHIFT                                              (30U)
#define PCU_SR_VDDA_UV_STS(x)                                                 (((uint32_t)(((uint32_t)(x)) << PCU_SR_VDDA_UV_STS_SHIFT)) & PCU_SR_VDDA_UV_STS_MASK)
#define PCU_SR_VDDA_UV_MASK                                                   (0x20000000U)
#define PCU_SR_VDDA_UV_SHIFT                                                  (29U)
#define PCU_SR_VDDA_UV(x)                                                     (((uint32_t)(((uint32_t)(x)) << PCU_SR_VDDA_UV_SHIFT)) & PCU_SR_VDDA_UV_MASK)
#define PCU_SR_VDDA_OV_MASK                                                   (0x10000000U)
#define PCU_SR_VDDA_OV_SHIFT                                                  (28U)
#define PCU_SR_VDDA_OV(x)                                                     (((uint32_t)(((uint32_t)(x)) << PCU_SR_VDDA_OV_SHIFT)) & PCU_SR_VDDA_OV_MASK)
#define PCU_SR_OC_DRV_MASK                                                    (0x1000000U)
#define PCU_SR_OC_DRV_SHIFT                                                   (24U)
#define PCU_SR_OC_DRV(x)                                                      (((uint32_t)(((uint32_t)(x)) << PCU_SR_OC_DRV_SHIFT)) & PCU_SR_OC_DRV_MASK)
#define PCU_SR_OVT_DEB_MASK                                                   (0x100000U)
#define PCU_SR_OVT_DEB_SHIFT                                                  (20U)
#define PCU_SR_OVT_DEB(x)                                                     (((uint32_t)(((uint32_t)(x)) << PCU_SR_OVT_DEB_SHIFT)) & PCU_SR_OVT_DEB_MASK)
#define PCU_SR_VSM_UV_MASK                                                    (0x20000U)
#define PCU_SR_VSM_UV_SHIFT                                                   (17U)
#define PCU_SR_VSM_UV(x)                                                      (((uint32_t)(((uint32_t)(x)) << PCU_SR_VSM_UV_SHIFT)) & PCU_SR_VSM_UV_MASK)
#define PCU_SR_VSM_OV_MASK                                                    (0x10000U)
#define PCU_SR_VSM_OV_SHIFT                                                   (16U)
#define PCU_SR_VSM_OV(x)                                                      (((uint32_t)(((uint32_t)(x)) << PCU_SR_VSM_OV_SHIFT)) & PCU_SR_VSM_OV_MASK)
#define PCU_SR_WU_FLAG_TIM_MASK                                               (0x20U)
#define PCU_SR_WU_FLAG_TIM_SHIFT                                              (5U)
#define PCU_SR_WU_FLAG_TIM(x)                                                 (((uint32_t)(((uint32_t)(x)) << PCU_SR_WU_FLAG_TIM_SHIFT)) & PCU_SR_WU_FLAG_TIM_MASK)
#define PCU_SR_WU_FLAG_LIN_MASK                                               (0x10U)
#define PCU_SR_WU_FLAG_LIN_SHIFT                                              (4U)
#define PCU_SR_WU_FLAG_LIN(x)                                                 (((uint32_t)(((uint32_t)(x)) << PCU_SR_WU_FLAG_LIN_SHIFT)) & PCU_SR_WU_FLAG_LIN_MASK)
#define PCU_SR_POR_MASK                                                       (0x2U)
#define PCU_SR_POR_SHIFT                                                      (1U)
#define PCU_SR_POR(x)                                                         (((uint32_t)(((uint32_t)(x)) << PCU_SR_POR_SHIFT)) & PCU_SR_POR_MASK)
/* PCU_IE Register */
#define PCU_IE_VDDA_UV_IE_MASK                                                (0x20000000U)
#define PCU_IE_VDDA_UV_IE_SHIFT                                               (29U)
#define PCU_IE_VDDA_UV_IE(x)                                                  (((uint32_t)(((uint32_t)(x)) << PCU_IE_VDDA_UV_IE_SHIFT)) & PCU_IE_VDDA_UV_IE_MASK)
#define PCU_IE_VDDA_OV_IE_MASK                                                (0x10000000U)
#define PCU_IE_VDDA_OV_IE_SHIFT                                               (28U)
#define PCU_IE_VDDA_OV_IE(x)                                                  (((uint32_t)(((uint32_t)(x)) << PCU_IE_VDDA_OV_IE_SHIFT)) & PCU_IE_VDDA_OV_IE_MASK)
#define PCU_IE_OC_DRV_IE_MASK                                                 (0x1000000U)
#define PCU_IE_OC_DRV_IE_SHIFT                                                (24U)
#define PCU_IE_OC_DRV_IE(x)                                                   (((uint32_t)(((uint32_t)(x)) << PCU_IE_OC_DRV_IE_SHIFT)) & PCU_IE_OC_DRV_IE_MASK)
#define PCU_IE_OVT_DEB_IE_MASK                                                (0x100000U)
#define PCU_IE_OVT_DEB_IE_SHIFT                                               (20U)
#define PCU_IE_OVT_DEB_IE(x)                                                  (((uint32_t)(((uint32_t)(x)) << PCU_IE_OVT_DEB_IE_SHIFT)) & PCU_IE_OVT_DEB_IE_MASK)
#define PCU_IE_VSM_UV_IE_MASK                                                 (0x20000U)
#define PCU_IE_VSM_UV_IE_SHIFT                                                (17U)
#define PCU_IE_VSM_UV_IE(x)                                                   (((uint32_t)(((uint32_t)(x)) << PCU_IE_VSM_UV_IE_SHIFT)) & PCU_IE_VSM_UV_IE_MASK)
#define PCU_IE_VSM_OV_IE_MASK                                                 (0x10000U)
#define PCU_IE_VSM_OV_IE_SHIFT                                                (16U)
#define PCU_IE_VSM_OV_IE(x)                                                   (((uint32_t)(((uint32_t)(x)) << PCU_IE_VSM_OV_IE_SHIFT)) & PCU_IE_VSM_OV_IE_MASK)
/* PCU_LDOCR Register */
#define PCU_LDOCR_VDDD_UV_EN_MASK                                             (0x100U)
#define PCU_LDOCR_VDDD_UV_EN_SHIFT                                            (8U)
#define PCU_LDOCR_VDDD_UV_EN(x)                                               (((uint32_t)(((uint32_t)(x)) << PCU_LDOCR_VDDD_UV_EN_SHIFT)) & PCU_LDOCR_VDDD_UV_EN_MASK)
#define PCU_LDOCR_VDDA_BOR_EN_MASK                                            (0x10U)
#define PCU_LDOCR_VDDA_BOR_EN_SHIFT                                           (4U)
#define PCU_LDOCR_VDDA_BOR_EN(x)                                              (((uint32_t)(((uint32_t)(x)) << PCU_LDOCR_VDDA_BOR_EN_SHIFT)) & PCU_LDOCR_VDDA_BOR_EN_MASK)
#define PCU_LDOCR_VDDA_OV_DEBEN_MASK                                          (0x8U)
#define PCU_LDOCR_VDDA_OV_DEBEN_SHIFT                                         (3U)
#define PCU_LDOCR_VDDA_OV_DEBEN(x)                                            (((uint32_t)(((uint32_t)(x)) << PCU_LDOCR_VDDA_OV_DEBEN_SHIFT)) & PCU_LDOCR_VDDA_OV_DEBEN_MASK)
#define PCU_LDOCR_VDDA_OV_EN_MASK                                             (0x4U)
#define PCU_LDOCR_VDDA_OV_EN_SHIFT                                            (2U)
#define PCU_LDOCR_VDDA_OV_EN(x)                                               (((uint32_t)(((uint32_t)(x)) << PCU_LDOCR_VDDA_OV_EN_SHIFT)) & PCU_LDOCR_VDDA_OV_EN_MASK)
#define PCU_LDOCR_VDDA_UV_DEBEN_MASK                                          (0x2U)
#define PCU_LDOCR_VDDA_UV_DEBEN_SHIFT                                         (1U)
#define PCU_LDOCR_VDDA_UV_DEBEN(x)                                            (((uint32_t)(((uint32_t)(x)) << PCU_LDOCR_VDDA_UV_DEBEN_SHIFT)) & PCU_LDOCR_VDDA_UV_DEBEN_MASK)
#define PCU_LDOCR_VDDA_UV_EN_MASK                                             (0x1U)
#define PCU_LDOCR_VDDA_UV_EN_SHIFT                                            (0U)
#define PCU_LDOCR_VDDA_UV_EN(x)                                               (((uint32_t)(((uint32_t)(x)) << PCU_LDOCR_VDDA_UV_EN_SHIFT)) & PCU_LDOCR_VDDA_UV_EN_MASK)
/* PCU_VSMCR Register */
#define PCU_VSMCR_VSM_UV_CFG_MASK                                             (0x700000U)
#define PCU_VSMCR_VSM_UV_CFG_SHIFT                                            (20U)
#define PCU_VSMCR_VSM_UV_CFG(x)                                               (((uint32_t)(((uint32_t)(x)) << PCU_VSMCR_VSM_UV_CFG_SHIFT)) & PCU_VSMCR_VSM_UV_CFG_MASK)
#define PCU_VSMCR_VSM_UV_DEBEN_MASK                                           (0x20000U)
#define PCU_VSMCR_VSM_UV_DEBEN_SHIFT                                          (17U)
#define PCU_VSMCR_VSM_UV_DEBEN(x)                                             (((uint32_t)(((uint32_t)(x)) << PCU_VSMCR_VSM_UV_DEBEN_SHIFT)) & PCU_VSMCR_VSM_UV_DEBEN_MASK)
#define PCU_VSMCR_VSM_UV_DETEN_MASK                                           (0x10000U)
#define PCU_VSMCR_VSM_UV_DETEN_SHIFT                                          (16U)
#define PCU_VSMCR_VSM_UV_DETEN(x)                                             (((uint32_t)(((uint32_t)(x)) << PCU_VSMCR_VSM_UV_DETEN_SHIFT)) & PCU_VSMCR_VSM_UV_DETEN_MASK)
#define PCU_VSMCR_VSM_OV_CFG_MASK                                             (0x30U)
#define PCU_VSMCR_VSM_OV_CFG_SHIFT                                            (4U)
#define PCU_VSMCR_VSM_OV_CFG(x)                                               (((uint32_t)(((uint32_t)(x)) << PCU_VSMCR_VSM_OV_CFG_SHIFT)) & PCU_VSMCR_VSM_OV_CFG_MASK)
#define PCU_VSMCR_VSM_OV_DEBEN_MASK                                           (0x2U)
#define PCU_VSMCR_VSM_OV_DEBEN_SHIFT                                          (1U)
#define PCU_VSMCR_VSM_OV_DEBEN(x)                                             (((uint32_t)(((uint32_t)(x)) << PCU_VSMCR_VSM_OV_DEBEN_SHIFT)) & PCU_VSMCR_VSM_OV_DEBEN_MASK)
#define PCU_VSMCR_VSM_OV_DETEN_MASK                                           (0x1U)
#define PCU_VSMCR_VSM_OV_DETEN_SHIFT                                          (0U)
#define PCU_VSMCR_VSM_OV_DETEN(x)                                             (((uint32_t)(((uint32_t)(x)) << PCU_VSMCR_VSM_OV_DETEN_SHIFT)) & PCU_VSMCR_VSM_OV_DETEN_MASK)
/* PCU_OVTCR Register */
#define PCU_OVTCR_OVT_EN_MASK                                                 (0x4U)
#define PCU_OVTCR_OVT_EN_SHIFT                                                (2U)
#define PCU_OVTCR_OVT_EN(x)                                                   (((uint32_t)(((uint32_t)(x)) << PCU_OVTCR_OVT_EN_SHIFT)) & PCU_OVTCR_OVT_EN_MASK)
#define PCU_OVTCR_OVTCMP_OEN_MASK                                             (0x2U)
#define PCU_OVTCR_OVTCMP_OEN_SHIFT                                            (1U)
#define PCU_OVTCR_OVTCMP_OEN(x)                                               (((uint32_t)(((uint32_t)(x)) << PCU_OVTCR_OVTCMP_OEN_SHIFT)) & PCU_OVTCR_OVTCMP_OEN_MASK)
#define PCU_OVTCR_SEL_OVT_MASK                                                (0x1U)
#define PCU_OVTCR_SEL_OVT_SHIFT                                               (0U)
#define PCU_OVTCR_SEL_OVT(x)                                                  (((uint32_t)(((uint32_t)(x)) << PCU_OVTCR_SEL_OVT_SHIFT)) & PCU_OVTCR_SEL_OVT_MASK)
/* PCU_OCCR Register */
#define PCU_OCCR_OC_DEB_CFG_MASK                                              (0x30U)
#define PCU_OCCR_OC_DEB_CFG_SHIFT                                             (4U)
#define PCU_OCCR_OC_DEB_CFG(x)                                                (((uint32_t)(((uint32_t)(x)) << PCU_OCCR_OC_DEB_CFG_SHIFT)) & PCU_OCCR_OC_DEB_CFG_MASK)
#define PCU_OCCR_OC_DEBEN_MASK                                                (0x4U)
#define PCU_OCCR_OC_DEBEN_SHIFT                                               (2U)
#define PCU_OCCR_OC_DEBEN(x)                                                  (((uint32_t)(((uint32_t)(x)) << PCU_OCCR_OC_DEBEN_SHIFT)) & PCU_OCCR_OC_DEBEN_MASK)
/* PCU_VDDLDOCR Register */
#define PCU_VDDLDOCR_LDO_VDDA33_SEL_MASK                                      (0x40U)
#define PCU_VDDLDOCR_LDO_VDDA33_SEL_SHIFT                                     (6U)
#define PCU_VDDLDOCR_LDO_VDDA33_SEL(x)                                        (((uint32_t)(((uint32_t)(x)) << PCU_VDDLDOCR_LDO_VDDA33_SEL_SHIFT)) & PCU_VDDLDOCR_LDO_VDDA33_SEL_MASK)
#define PCU_VDDLDOCR_LDO_VDDA_EN_MASK                                         (0x20U)
#define PCU_VDDLDOCR_LDO_VDDA_EN_SHIFT                                        (5U)
#define PCU_VDDLDOCR_LDO_VDDA_EN(x)                                           (((uint32_t)(((uint32_t)(x)) << PCU_VDDLDOCR_LDO_VDDA_EN_SHIFT)) & PCU_VDDLDOCR_LDO_VDDA_EN_MASK)
#define PCU_VDDLDOCR_LDO_VDDD_EN_MASK                                         (0x10U)
#define PCU_VDDLDOCR_LDO_VDDD_EN_SHIFT                                        (4U)
#define PCU_VDDLDOCR_LDO_VDDD_EN(x)                                           (((uint32_t)(((uint32_t)(x)) << PCU_VDDLDOCR_LDO_VDDD_EN_SHIFT)) & PCU_VDDLDOCR_LDO_VDDD_EN_MASK)
#define PCU_VDDLDOCR_UNLOCK_MASK                                              (0x1U)
#define PCU_VDDLDOCR_UNLOCK_SHIFT                                             (0U)
#define PCU_VDDLDOCR_UNLOCK(x)                                                (((uint32_t)(((uint32_t)(x)) << PCU_VDDLDOCR_UNLOCK_SHIFT)) & PCU_VDDLDOCR_UNLOCK_MASK)


/*!
 * @}
 */ /* end of group PCU_Register_Masks */


/*!
 * @}
 */ /* end of group PCU_Peripheral_Access_Layer */


/* -------------------------------------------------------------
   -- RCU Peripheral Access Layer
   ------------------------------------------------------------- */

/*!
 * @addtogroup RCU_Peripheral_Access_Layer RCU Peripheral Access Layer
 * @{
 */


/** RCU - Size of Registers Arrays */

/* RCU Register Layout Typedef */
typedef struct {
    __IO uint32_t RSSR;                                                       /**< Reset Source Status Register, offset: 0x0000 */
    __IO uint32_t RPFR;                                                       /**< Reset Pin Filter Register, offset: 0x0004 */

} RCU_Type, *RCU_MemMapPtr;

/** Number of instances of the RCU module. */
#define RCU_INSTANCE_COUNT                                                    (1u)

/* RCU base address */
#define RCU_BASE                                                              (0x4007F000)
#define RCU                                                                   ((RCU_Type *)(RCU_BASE))

/** Array initializer of RCU peripheral base addresses */
#define RCU_BASE_ADDRS                                                        { RCU_BASE }
/** Array initializer of RCU peripheral base pointers */
#define RCU_BASE_PTRS                                                         { RCU }

/* RCU Register Mask */
/* RCU_RSSR Register */
#define RCU_RSSR_LPACK_MASK                                                   (0x80000U)
#define RCU_RSSR_LPACK_SHIFT                                                  (19U)
#define RCU_RSSR_LPACK(x)                                                     (((uint32_t)(((uint32_t)(x)) << RCU_RSSR_LPACK_SHIFT)) & RCU_RSSR_LPACK_MASK)
#define RCU_RSSR_PLL_UNLOCK_MASK                                              (0x40000U)
#define RCU_RSSR_PLL_UNLOCK_SHIFT                                             (18U)
#define RCU_RSSR_PLL_UNLOCK(x)                                                (((uint32_t)(((uint32_t)(x)) << RCU_RSSR_PLL_UNLOCK_SHIFT)) & RCU_RSSR_PLL_UNLOCK_MASK)
#define RCU_RSSR_WDG_MASK                                                     (0x20000U)
#define RCU_RSSR_WDG_SHIFT                                                    (17U)
#define RCU_RSSR_WDG(x)                                                       (((uint32_t)(((uint32_t)(x)) << RCU_RSSR_WDG_SHIFT)) & RCU_RSSR_WDG_MASK)
#define RCU_RSSR_LOCKUP_MASK                                                  (0x10000U)
#define RCU_RSSR_LOCKUP_SHIFT                                                 (16U)
#define RCU_RSSR_LOCKUP(x)                                                    (((uint32_t)(((uint32_t)(x)) << RCU_RSSR_LOCKUP_SHIFT)) & RCU_RSSR_LOCKUP_MASK)
#define RCU_RSSR_SW_MASK                                                      (0x100U)
#define RCU_RSSR_SW_SHIFT                                                     (8U)
#define RCU_RSSR_SW(x)                                                        (((uint32_t)(((uint32_t)(x)) << RCU_RSSR_SW_SHIFT)) & RCU_RSSR_SW_MASK)
#define RCU_RSSR_PIN_MASK                                                     (0x2U)
#define RCU_RSSR_PIN_SHIFT                                                    (1U)
#define RCU_RSSR_PIN(x)                                                       (((uint32_t)(((uint32_t)(x)) << RCU_RSSR_PIN_SHIFT)) & RCU_RSSR_PIN_MASK)
#define RCU_RSSR_POR_BOR_MASK                                                 (0x1U)
#define RCU_RSSR_POR_BOR_SHIFT                                                (0U)
#define RCU_RSSR_POR_BOR(x)                                                   (((uint32_t)(((uint32_t)(x)) << RCU_RSSR_POR_BOR_SHIFT)) & RCU_RSSR_POR_BOR_MASK)
/* RCU_RPFR Register */
#define RCU_RPFR_RSTPIN_OBEN_MASK                                             (0x10000U)
#define RCU_RPFR_RSTPIN_OBEN_SHIFT                                            (16U)
#define RCU_RPFR_RSTPIN_OBEN(x)                                               (((uint32_t)(((uint32_t)(x)) << RCU_RPFR_RSTPIN_OBEN_SHIFT)) & RCU_RPFR_RSTPIN_OBEN_MASK)
#define RCU_RPFR_FLTVAL_MASK                                                  (0xFF00U)
#define RCU_RPFR_FLTVAL_SHIFT                                                 (8U)
#define RCU_RPFR_FLTVAL(x)                                                    (((uint32_t)(((uint32_t)(x)) << RCU_RPFR_FLTVAL_SHIFT)) & RCU_RPFR_FLTVAL_MASK)
#define RCU_RPFR_FLTCLKSEL_MASK                                               (0x4U)
#define RCU_RPFR_FLTCLKSEL_SHIFT                                              (2U)
#define RCU_RPFR_FLTCLKSEL(x)                                                 (((uint32_t)(((uint32_t)(x)) << RCU_RPFR_FLTCLKSEL_SHIFT)) & RCU_RPFR_FLTCLKSEL_MASK)
#define RCU_RPFR_FLTSTEN_MASK                                                 (0x2U)
#define RCU_RPFR_FLTSTEN_SHIFT                                                (1U)
#define RCU_RPFR_FLTSTEN(x)                                                   (((uint32_t)(((uint32_t)(x)) << RCU_RPFR_FLTSTEN_SHIFT)) & RCU_RPFR_FLTSTEN_MASK)
#define RCU_RPFR_FLTEN_MASK                                                   (0x1U)
#define RCU_RPFR_FLTEN_SHIFT                                                  (0U)
#define RCU_RPFR_FLTEN(x)                                                     (((uint32_t)(((uint32_t)(x)) << RCU_RPFR_FLTEN_SHIFT)) & RCU_RPFR_FLTEN_MASK)


/*!
 * @}
 */ /* end of group RCU_Register_Masks */


/*!
 * @}
 */ /* end of group RCU_Peripheral_Access_Layer */


/* -------------------------------------------------------------
   -- SPI Peripheral Access Layer
   ------------------------------------------------------------- */

/*!
 * @addtogroup SPI_Peripheral_Access_Layer SPI Peripheral Access Layer
 * @{
 */


/** SPI - Size of Registers Arrays */

/* SPI Register Layout Typedef */
typedef struct {
    __IO uint32_t CTRL;                                                       /**< Control Register, offset: 0x0000 */
    __IO uint32_t TXCFG;                                                      /**< Transmit Configuration Register, offset: 0x0004 */
    __IO uint32_t CLK;                                                        /**< Clock Configuration Register, offset: 0x0008 */
    __IO uint32_t STS;                                                        /**< Status Register, offset: 0x000C */
    __IO uint32_t INTE;                                                       /**< Interrupt Enable Register, offset: 0x0010 */
    __IO uint8_t  RESERVED5[16];                                              /**< RESERVED5, offset: 0x0014 */
    __IO uint32_t DATA;                                                       /**< Data Register, offset: 0x0024 */

} SPI_Type, *SPI_MemMapPtr;

/** Number of instances of the SPI module. */
#define SPI_INSTANCE_COUNT                                                    (1u)

/* SPI0  base address */
#define SPI0_BASE                                                             (0x4002C000)
#define SPI0                                                                  ((SPI_Type *)(SPI0_BASE))

/** Array initializer of SPI peripheral base addresses */
#define SPI_BASE_ADDRS                                                        { SPI0_BASE }
/** Array initializer of SPI peripheral base pointers */
#define SPI_BASE_PTRS                                                         { SPI0 }
/** Number of interrupt vector arrays for the SPI module. */
#define SPI_IRQS_ARR_COUNT                                                    (1u)
/** Number of interrupt channels for the SPI. */
#define SPI_IRQS_CH_COUNT                                                     (1u)
/** Interrupt vectors for the SPI peripheral type */
#define SPI_IRQS                                                              { SPI0_IRQn }

/* SPI Register Mask */
/* SPI_CTRL Register */
#define SPI_CTRL_AUTOPCS_MASK                                                 (0x2000000U)
#define SPI_CTRL_AUTOPCS_SHIFT                                                (25U)
#define SPI_CTRL_AUTOPCS(x)                                                   (((uint32_t)(((uint32_t)(x)) << SPI_CTRL_AUTOPCS_SHIFT)) & SPI_CTRL_AUTOPCS_MASK)
#define SPI_CTRL_SPDEN_MASK                                                   (0x1000000U)
#define SPI_CTRL_SPDEN_SHIFT                                                  (24U)
#define SPI_CTRL_SPDEN(x)                                                     (((uint32_t)(((uint32_t)(x)) << SPI_CTRL_SPDEN_SHIFT)) & SPI_CTRL_SPDEN_MASK)
#define SPI_CTRL_MANPCSEN_MASK                                                (0x400000U)
#define SPI_CTRL_MANPCSEN_SHIFT                                               (22U)
#define SPI_CTRL_MANPCSEN(x)                                                  (((uint32_t)(((uint32_t)(x)) << SPI_CTRL_MANPCSEN_SHIFT)) & SPI_CTRL_MANPCSEN_MASK)
#define SPI_CTRL_MANPCSDAT_MASK                                               (0x200000U)
#define SPI_CTRL_MANPCSDAT_SHIFT                                              (21U)
#define SPI_CTRL_MANPCSDAT(x)                                                 (((uint32_t)(((uint32_t)(x)) << SPI_CTRL_MANPCSDAT_SHIFT)) & SPI_CTRL_MANPCSDAT_MASK)
#define SPI_CTRL_CSPOL_MASK                                                   (0x300U)
#define SPI_CTRL_CSPOL_SHIFT                                                  (8U)
#define SPI_CTRL_CSPOL(x)                                                     (((uint32_t)(((uint32_t)(x)) << SPI_CTRL_CSPOL_SHIFT)) & SPI_CTRL_CSPOL_MASK)
#define SPI_CTRL_DHZEN_MASK                                                   (0x40U)
#define SPI_CTRL_DHZEN_SHIFT                                                  (6U)
#define SPI_CTRL_DHZEN(x)                                                     (((uint32_t)(((uint32_t)(x)) << SPI_CTRL_DHZEN_SHIFT)) & SPI_CTRL_DHZEN_MASK)
#define SPI_CTRL_PINCFG_MASK                                                  (0x30U)
#define SPI_CTRL_PINCFG_SHIFT                                                 (4U)
#define SPI_CTRL_PINCFG(x)                                                    (((uint32_t)(((uint32_t)(x)) << SPI_CTRL_PINCFG_SHIFT)) & SPI_CTRL_PINCFG_MASK)
#define SPI_CTRL_MODE_MASK                                                    (0x2U)
#define SPI_CTRL_MODE_SHIFT                                                   (1U)
#define SPI_CTRL_MODE(x)                                                      (((uint32_t)(((uint32_t)(x)) << SPI_CTRL_MODE_SHIFT)) & SPI_CTRL_MODE_MASK)
#define SPI_CTRL_EN_MASK                                                      (0x1U)
#define SPI_CTRL_EN_SHIFT                                                     (0U)
#define SPI_CTRL_EN(x)                                                        (((uint32_t)(((uint32_t)(x)) << SPI_CTRL_EN_SHIFT)) & SPI_CTRL_EN_MASK)
/* SPI_TXCFG Register */
#define SPI_TXCFG_CPOL_MASK                                                   (0x40U)
#define SPI_TXCFG_CPOL_SHIFT                                                  (6U)
#define SPI_TXCFG_CPOL(x)                                                     (((uint32_t)(((uint32_t)(x)) << SPI_TXCFG_CPOL_SHIFT)) & SPI_TXCFG_CPOL_MASK)
#define SPI_TXCFG_CPHA_MASK                                                   (0x20U)
#define SPI_TXCFG_CPHA_SHIFT                                                  (5U)
#define SPI_TXCFG_CPHA(x)                                                     (((uint32_t)(((uint32_t)(x)) << SPI_TXCFG_CPHA_SHIFT)) & SPI_TXCFG_CPHA_MASK)
#define SPI_TXCFG_PCS_MASK                                                    (0x10U)
#define SPI_TXCFG_PCS_SHIFT                                                   (4U)
#define SPI_TXCFG_PCS(x)                                                      (((uint32_t)(((uint32_t)(x)) << SPI_TXCFG_PCS_SHIFT)) & SPI_TXCFG_PCS_MASK)
#define SPI_TXCFG_LSBF_MASK                                                   (0x8U)
#define SPI_TXCFG_LSBF_SHIFT                                                  (3U)
#define SPI_TXCFG_LSBF(x)                                                     (((uint32_t)(((uint32_t)(x)) << SPI_TXCFG_LSBF_SHIFT)) & SPI_TXCFG_LSBF_MASK)
#define SPI_TXCFG_CONT_MASK                                                   (0x4U)
#define SPI_TXCFG_CONT_SHIFT                                                  (2U)
#define SPI_TXCFG_CONT(x)                                                     (((uint32_t)(((uint32_t)(x)) << SPI_TXCFG_CONT_SHIFT)) & SPI_TXCFG_CONT_MASK)
#define SPI_TXCFG_MSKRX_MASK                                                  (0x2U)
#define SPI_TXCFG_MSKRX_SHIFT                                                 (1U)
#define SPI_TXCFG_MSKRX(x)                                                    (((uint32_t)(((uint32_t)(x)) << SPI_TXCFG_MSKRX_SHIFT)) & SPI_TXCFG_MSKRX_MASK)
#define SPI_TXCFG_MSKTX_MASK                                                  (0x1U)
#define SPI_TXCFG_MSKTX_SHIFT                                                 (0U)
#define SPI_TXCFG_MSKTX(x)                                                    (((uint32_t)(((uint32_t)(x)) << SPI_TXCFG_MSKTX_SHIFT)) & SPI_TXCFG_MSKTX_MASK)
/* SPI_CLK Register */
#define SPI_CLK_SCKPCS_WIDTH                                                  (3)
#define SPI_CLK_SCKPCS_MASK                                                   (0x7000000U)
#define SPI_CLK_SCKPCS_SHIFT                                                  (24U)
#define SPI_CLK_SCKPCS(x)                                                     (((uint32_t)(((uint32_t)(x)) << SPI_CLK_SCKPCS_SHIFT)) & SPI_CLK_SCKPCS_MASK)
#define SPI_CLK_PCSSCK_WIDTH                                                  (3)
#define SPI_CLK_PCSSCK_MASK                                                   (0x70000U)
#define SPI_CLK_PCSSCK_SHIFT                                                  (16U)
#define SPI_CLK_PCSSCK(x)                                                     (((uint32_t)(((uint32_t)(x)) << SPI_CLK_PCSSCK_SHIFT)) & SPI_CLK_PCSSCK_MASK)
#define SPI_CLK_FMDLY_WIDTH                                                   (3)
#define SPI_CLK_FMDLY_MASK                                                    (0x700U)
#define SPI_CLK_FMDLY_SHIFT                                                   (8U)
#define SPI_CLK_FMDLY(x)                                                      (((uint32_t)(((uint32_t)(x)) << SPI_CLK_FMDLY_SHIFT)) & SPI_CLK_FMDLY_MASK)
#define SPI_CLK_DIV_MASK                                                      (0xFFU)
#define SPI_CLK_DIV_SHIFT                                                     (0U)
#define SPI_CLK_DIV(x)                                                        (((uint32_t)(((uint32_t)(x)) << SPI_CLK_DIV_SHIFT)) & SPI_CLK_DIV_MASK)
/* SPI_STS Register */
#define SPI_STS_BUSY_MASK                                                     (0x40000U)
#define SPI_STS_BUSY_SHIFT                                                    (18U)
#define SPI_STS_BUSY(x)                                                       (((uint32_t)(((uint32_t)(x)) << SPI_STS_BUSY_SHIFT)) & SPI_STS_BUSY_MASK)
#define SPI_STS_RXEPT_MASK                                                    (0x20000U)
#define SPI_STS_RXEPT_SHIFT                                                   (17U)
#define SPI_STS_RXEPT(x)                                                      (((uint32_t)(((uint32_t)(x)) << SPI_STS_RXEPT_SHIFT)) & SPI_STS_RXEPT_MASK)
#define SPI_STS_SOF_MASK                                                      (0x10000U)
#define SPI_STS_SOF_SHIFT                                                     (16U)
#define SPI_STS_SOF(x)                                                        (((uint32_t)(((uint32_t)(x)) << SPI_STS_SOF_SHIFT)) & SPI_STS_SOF_MASK)
#define SPI_STS_PCSPOSIF_MASK                                                 (0x200U)
#define SPI_STS_PCSPOSIF_SHIFT                                                (9U)
#define SPI_STS_PCSPOSIF(x)                                                   (((uint32_t)(((uint32_t)(x)) << SPI_STS_PCSPOSIF_SHIFT)) & SPI_STS_PCSPOSIF_MASK)
#define SPI_STS_PCSNEGIF_MASK                                                 (0x100U)
#define SPI_STS_PCSNEGIF_SHIFT                                                (8U)
#define SPI_STS_PCSNEGIF(x)                                                   (((uint32_t)(((uint32_t)(x)) << SPI_STS_PCSNEGIF_SHIFT)) & SPI_STS_PCSNEGIF_MASK)
#define SPI_STS_RXOVIF_MASK                                                   (0x40U)
#define SPI_STS_RXOVIF_SHIFT                                                  (6U)
#define SPI_STS_RXOVIF(x)                                                     (((uint32_t)(((uint32_t)(x)) << SPI_STS_RXOVIF_SHIFT)) & SPI_STS_RXOVIF_MASK)
#define SPI_STS_TXUNIF_MASK                                                   (0x20U)
#define SPI_STS_TXUNIF_SHIFT                                                  (5U)
#define SPI_STS_TXUNIF(x)                                                     (((uint32_t)(((uint32_t)(x)) << SPI_STS_TXUNIF_SHIFT)) & SPI_STS_TXUNIF_MASK)
#define SPI_STS_TCIF_MASK                                                     (0x10U)
#define SPI_STS_TCIF_SHIFT                                                    (4U)
#define SPI_STS_TCIF(x)                                                       (((uint32_t)(((uint32_t)(x)) << SPI_STS_TCIF_SHIFT)) & SPI_STS_TCIF_MASK)
#define SPI_STS_FCIF_MASK                                                     (0x8U)
#define SPI_STS_FCIF_SHIFT                                                    (3U)
#define SPI_STS_FCIF(x)                                                       (((uint32_t)(((uint32_t)(x)) << SPI_STS_FCIF_SHIFT)) & SPI_STS_FCIF_MASK)
#define SPI_STS_WCIF_MASK                                                     (0x4U)
#define SPI_STS_WCIF_SHIFT                                                    (2U)
#define SPI_STS_WCIF(x)                                                       (((uint32_t)(((uint32_t)(x)) << SPI_STS_WCIF_SHIFT)) & SPI_STS_WCIF_MASK)
/* SPI_INTE Register */
#define SPI_INTE_PCSPOSIE_MASK                                                (0x200U)
#define SPI_INTE_PCSPOSIE_SHIFT                                               (9U)
#define SPI_INTE_PCSPOSIE(x)                                                  (((uint32_t)(((uint32_t)(x)) << SPI_INTE_PCSPOSIE_SHIFT)) & SPI_INTE_PCSPOSIE_MASK)
#define SPI_INTE_PCSNEGIE_MASK                                                (0x100U)
#define SPI_INTE_PCSNEGIE_SHIFT                                               (8U)
#define SPI_INTE_PCSNEGIE(x)                                                  (((uint32_t)(((uint32_t)(x)) << SPI_INTE_PCSNEGIE_SHIFT)) & SPI_INTE_PCSNEGIE_MASK)
#define SPI_INTE_RXOVIE_MASK                                                  (0x40U)
#define SPI_INTE_RXOVIE_SHIFT                                                 (6U)
#define SPI_INTE_RXOVIE(x)                                                    (((uint32_t)(((uint32_t)(x)) << SPI_INTE_RXOVIE_SHIFT)) & SPI_INTE_RXOVIE_MASK)
#define SPI_INTE_TXUNIE_MASK                                                  (0x20U)
#define SPI_INTE_TXUNIE_SHIFT                                                 (5U)
#define SPI_INTE_TXUNIE(x)                                                    (((uint32_t)(((uint32_t)(x)) << SPI_INTE_TXUNIE_SHIFT)) & SPI_INTE_TXUNIE_MASK)
#define SPI_INTE_TCIE_MASK                                                    (0x10U)
#define SPI_INTE_TCIE_SHIFT                                                   (4U)
#define SPI_INTE_TCIE(x)                                                      (((uint32_t)(((uint32_t)(x)) << SPI_INTE_TCIE_SHIFT)) & SPI_INTE_TCIE_MASK)
#define SPI_INTE_FCIE_MASK                                                    (0x8U)
#define SPI_INTE_FCIE_SHIFT                                                   (3U)
#define SPI_INTE_FCIE(x)                                                      (((uint32_t)(((uint32_t)(x)) << SPI_INTE_FCIE_SHIFT)) & SPI_INTE_FCIE_MASK)
#define SPI_INTE_WCIE_MASK                                                    (0x4U)
#define SPI_INTE_WCIE_SHIFT                                                   (2U)
#define SPI_INTE_WCIE(x)                                                      (((uint32_t)(((uint32_t)(x)) << SPI_INTE_WCIE_SHIFT)) & SPI_INTE_WCIE_MASK)
#define SPI_INTE_TXIE_MASK                                                    (0x2U)
#define SPI_INTE_TXIE_SHIFT                                                   (1U)
#define SPI_INTE_TXIE(x)                                                      (((uint32_t)(((uint32_t)(x)) << SPI_INTE_TXIE_SHIFT)) & SPI_INTE_TXIE_MASK)
#define SPI_INTE_RXIE_MASK                                                    (0x1U)
#define SPI_INTE_RXIE_SHIFT                                                   (0U)
#define SPI_INTE_RXIE(x)                                                      (((uint32_t)(((uint32_t)(x)) << SPI_INTE_RXIE_SHIFT)) & SPI_INTE_RXIE_MASK)
/* SPI_DATA Register */
#define SPI_DATA_DATA_MASK                                                    (0xFFFFFFFFU)
#define SPI_DATA_DATA_SHIFT                                                   (0U)
#define SPI_DATA_DATA(x)                                                      (((uint32_t)(((uint32_t)(x)) << SPI_DATA_DATA_SHIFT)) & SPI_DATA_DATA_MASK)


/*!
 * @}
 */ /* end of group SPI_Register_Masks */


/*!
 * @}
 */ /* end of group SPI_Peripheral_Access_Layer */


/* -------------------------------------------------------------
   -- UART Peripheral Access Layer
   ------------------------------------------------------------- */

/*!
 * @addtogroup UART_Peripheral_Access_Layer UART Peripheral Access Layer
 * @{
 */


/** UART - Size of Registers Arrays */

/* UART Register Layout Typedef */
typedef struct {
    __IO uint32_t BAUD;                                                       /**< Baud Rate Configuration Register, offset: 0x0000 */
    __IO uint32_t CTRL0;                                                      /**< Control 0 Register, offset: 0x0004 */
    __IO uint32_t CTRL1;                                                      /**< Control 1 Register, offset: 0x0008 */
    __IO uint32_t INTF;                                                       /**< Interrupt Flag Register, offset: 0x000C */
    __IO uint32_t INTE;                                                       /**< Interrupt Enable Register, offset: 0x0010 */
    __IO uint32_t MATCH;                                                      /**< Match Configuration Register, offset: 0x0014 */
    __IO uint32_t DATA;                                                       /**< Data Buffer Register, offset: 0x0018 */
    __IO uint32_t RESERVED7;                                                  /**< RESERVED7, offset: 0x001C */
    __IO uint32_t RESERVED8;                                                  /**< RESERVED8, offset: 0x0020 */

} UART_Type, *UART_MemMapPtr;

/** Number of instances of the UART module. */
#define UART_INSTANCE_COUNT                                                   (1u)

/* UART0  base address */
#define UART0_BASE                                                            (0x4006A000)
#define UART0                                                                 ((UART_Type *)(UART0_BASE))

/** Array initializer of UART peripheral base addresses */
#define UART_BASE_ADDRS                                                       { UART0_BASE }
/** Array initializer of UART peripheral base pointers */
#define UART_BASE_PTRS                                                        { UART0 }
/** Number of interrupt vector arrays for the UART module. */
#define UART_IRQS_ARR_COUNT                                                   (1u)
/** Number of interrupt channels for the RX_TX type of UART. */
#define UART_RX_TX_IRQS_CH_COUNT                                              (1u)
#define UART_RX_TX_IRQS                                                       { UART0_IRQn }

/* UART Register Mask */
/* UART_BAUD Register */
#define UART_BAUD_OSRVAL_MASK                                                 (0x1F0000U)
#define UART_BAUD_OSRVAL_SHIFT                                                (16U)
#define UART_BAUD_OSRVAL(x)                                                   (((uint32_t)(((uint32_t)(x)) << UART_BAUD_OSRVAL_SHIFT)) & UART_BAUD_OSRVAL_MASK)
#define UART_BAUD_DIV_MASK                                                    (0x1FFFU)
#define UART_BAUD_DIV_SHIFT                                                   (0U)
#define UART_BAUD_DIV(x)                                                      (((uint32_t)(((uint32_t)(x)) << UART_BAUD_DIV_SHIFT)) & UART_BAUD_DIV_MASK)
/* UART_CTRL0 Register */
#define UART_CTRL0_MATEN_MASK                                                 (0x200000U)
#define UART_CTRL0_MATEN_SHIFT                                                (21U)
#define UART_CTRL0_MATEN(x)                                                   (((uint32_t)(((uint32_t)(x)) << UART_CTRL0_MATEN_SHIFT)) & UART_CTRL0_MATEN_MASK)
#define UART_CTRL0_MATMOD_MASK                                                (0x30000U)
#define UART_CTRL0_MATMOD_SHIFT                                               (16U)
#define UART_CTRL0_MATMOD(x)                                                  (((uint32_t)(((uint32_t)(x)) << UART_CTRL0_MATMOD_SHIFT)) & UART_CTRL0_MATMOD_MASK)
#define UART_CTRL0_TXEN_MASK                                                  (0x2U)
#define UART_CTRL0_TXEN_SHIFT                                                 (1U)
#define UART_CTRL0_TXEN(x)                                                    (((uint32_t)(((uint32_t)(x)) << UART_CTRL0_TXEN_SHIFT)) & UART_CTRL0_TXEN_MASK)
#define UART_CTRL0_RXEN_MASK                                                  (0x1U)
#define UART_CTRL0_RXEN_SHIFT                                                 (0U)
#define UART_CTRL0_RXEN(x)                                                    (((uint32_t)(((uint32_t)(x)) << UART_CTRL0_RXEN_SHIFT)) & UART_CTRL0_RXEN_MASK)
/* UART_CTRL1 Register */
#define UART_CTRL1_FRAMESZ_MASK                                               (0xC0000000U)
#define UART_CTRL1_FRAMESZ_SHIFT                                              (30U)
#define UART_CTRL1_FRAMESZ(x)                                                 (((uint32_t)(((uint32_t)(x)) << UART_CTRL1_FRAMESZ_SHIFT)) & UART_CTRL1_FRAMESZ_MASK)
#define UART_CTRL1_STOPSZ_MASK                                                (0x8000000U)
#define UART_CTRL1_STOPSZ_SHIFT                                               (27U)
#define UART_CTRL1_STOPSZ(x)                                                  (((uint32_t)(((uint32_t)(x)) << UART_CTRL1_STOPSZ_SHIFT)) & UART_CTRL1_STOPSZ_MASK)
#define UART_CTRL1_TXINV_MASK                                                 (0x1000000U)
#define UART_CTRL1_TXINV_SHIFT                                                (24U)
#define UART_CTRL1_TXINV(x)                                                   (((uint32_t)(((uint32_t)(x)) << UART_CTRL1_TXINV_SHIFT)) & UART_CTRL1_TXINV_MASK)
#define UART_CTRL1_RXINV_MASK                                                 (0x800000U)
#define UART_CTRL1_RXINV_SHIFT                                                (23U)
#define UART_CTRL1_RXINV(x)                                                   (((uint32_t)(((uint32_t)(x)) << UART_CTRL1_RXINV_SHIFT)) & UART_CTRL1_RXINV_MASK)
#define UART_CTRL1_SWAP_MASK                                                  (0x200000U)
#define UART_CTRL1_SWAP_SHIFT                                                 (21U)
#define UART_CTRL1_SWAP(x)                                                    (((uint32_t)(((uint32_t)(x)) << UART_CTRL1_SWAP_SHIFT)) & UART_CTRL1_SWAP_MASK)
#define UART_CTRL1_IDLESZ_MASK                                                (0x70000U)
#define UART_CTRL1_IDLESZ_SHIFT                                               (16U)
#define UART_CTRL1_IDLESZ(x)                                                  (((uint32_t)(((uint32_t)(x)) << UART_CTRL1_IDLESZ_SHIFT)) & UART_CTRL1_IDLESZ_MASK)
#define UART_CTRL1_BOTHEDGE_MASK                                              (0x8000U)
#define UART_CTRL1_BOTHEDGE_SHIFT                                             (15U)
#define UART_CTRL1_BOTHEDGE(x)                                                (((uint32_t)(((uint32_t)(x)) << UART_CTRL1_BOTHEDGE_SHIFT)) & UART_CTRL1_BOTHEDGE_MASK)
#define UART_CTRL1_LBKDEN_MASK                                                (0x400U)
#define UART_CTRL1_LBKDEN_SHIFT                                               (10U)
#define UART_CTRL1_LBKDEN(x)                                                  (((uint32_t)(((uint32_t)(x)) << UART_CTRL1_LBKDEN_SHIFT)) & UART_CTRL1_LBKDEN_MASK)
#define UART_CTRL1_LBKSZ_MASK                                                 (0x200U)
#define UART_CTRL1_LBKSZ_SHIFT                                                (9U)
#define UART_CTRL1_LBKSZ(x)                                                   (((uint32_t)(((uint32_t)(x)) << UART_CTRL1_LBKSZ_SHIFT)) & UART_CTRL1_LBKSZ_MASK)
#define UART_CTRL1_MSBF_MASK                                                  (0x100U)
#define UART_CTRL1_MSBF_SHIFT                                                 (8U)
#define UART_CTRL1_MSBF(x)                                                    (((uint32_t)(((uint32_t)(x)) << UART_CTRL1_MSBF_SHIFT)) & UART_CTRL1_MSBF_MASK)
#define UART_CTRL1_SBK_MASK                                                   (0x40U)
#define UART_CTRL1_SBK_SHIFT                                                  (6U)
#define UART_CTRL1_SBK(x)                                                     (((uint32_t)(((uint32_t)(x)) << UART_CTRL1_SBK_SHIFT)) & UART_CTRL1_SBK_MASK)
#define UART_CTRL1_PAREN_MASK                                                 (0x2U)
#define UART_CTRL1_PAREN_SHIFT                                                (1U)
#define UART_CTRL1_PAREN(x)                                                   (((uint32_t)(((uint32_t)(x)) << UART_CTRL1_PAREN_SHIFT)) & UART_CTRL1_PAREN_MASK)
#define UART_CTRL1_PARSEL_MASK                                                (0x1U)
#define UART_CTRL1_PARSEL_SHIFT                                               (0U)
#define UART_CTRL1_PARSEL(x)                                                  (((uint32_t)(((uint32_t)(x)) << UART_CTRL1_PARSEL_SHIFT)) & UART_CTRL1_PARSEL_MASK)
/* UART_INTF Register */
#define UART_INTF_IDLEWAKE_MASK                                               (0x20000U)
#define UART_INTF_IDLEWAKE_SHIFT                                              (17U)
#define UART_INTF_IDLEWAKE(x)                                                 (((uint32_t)(((uint32_t)(x)) << UART_INTF_IDLEWAKE_SHIFT)) & UART_INTF_IDLEWAKE_MASK)
#define UART_INTF_RXBUSY_MASK                                                 (0x10000U)
#define UART_INTF_RXBUSY_SHIFT                                                (16U)
#define UART_INTF_RXBUSY(x)                                                   (((uint32_t)(((uint32_t)(x)) << UART_INTF_RXBUSY_SHIFT)) & UART_INTF_RXBUSY_MASK)
#define UART_INTF_LBKDIF_MASK                                                 (0x800U)
#define UART_INTF_LBKDIF_SHIFT                                                (11U)
#define UART_INTF_LBKDIF(x)                                                   (((uint32_t)(((uint32_t)(x)) << UART_INTF_LBKDIF_SHIFT)) & UART_INTF_LBKDIF_MASK)
#define UART_INTF_RXEDGEIF_MASK                                               (0x400U)
#define UART_INTF_RXEDGEIF_SHIFT                                              (10U)
#define UART_INTF_RXEDGEIF(x)                                                 (((uint32_t)(((uint32_t)(x)) << UART_INTF_RXEDGEIF_SHIFT)) & UART_INTF_RXEDGEIF_MASK)
#define UART_INTF_MATIF_MASK                                                  (0x200U)
#define UART_INTF_MATIF_SHIFT                                                 (9U)
#define UART_INTF_MATIF(x)                                                    (((uint32_t)(((uint32_t)(x)) << UART_INTF_MATIF_SHIFT)) & UART_INTF_MATIF_MASK)
#define UART_INTF_NOZIF_MASK                                                  (0x80U)
#define UART_INTF_NOZIF_SHIFT                                                 (7U)
#define UART_INTF_NOZIF(x)                                                    (((uint32_t)(((uint32_t)(x)) << UART_INTF_NOZIF_SHIFT)) & UART_INTF_NOZIF_MASK)
#define UART_INTF_OVRIF_MASK                                                  (0x40U)
#define UART_INTF_OVRIF_SHIFT                                                 (6U)
#define UART_INTF_OVRIF(x)                                                    (((uint32_t)(((uint32_t)(x)) << UART_INTF_OVRIF_SHIFT)) & UART_INTF_OVRIF_MASK)
#define UART_INTF_FEIF_MASK                                                   (0x20U)
#define UART_INTF_FEIF_SHIFT                                                  (5U)
#define UART_INTF_FEIF(x)                                                     (((uint32_t)(((uint32_t)(x)) << UART_INTF_FEIF_SHIFT)) & UART_INTF_FEIF_MASK)
#define UART_INTF_IDLEIF_MASK                                                 (0x10U)
#define UART_INTF_IDLEIF_SHIFT                                                (4U)
#define UART_INTF_IDLEIF(x)                                                   (((uint32_t)(((uint32_t)(x)) << UART_INTF_IDLEIF_SHIFT)) & UART_INTF_IDLEIF_MASK)
#define UART_INTF_PARIF_MASK                                                  (0x8U)
#define UART_INTF_PARIF_SHIFT                                                 (3U)
#define UART_INTF_PARIF(x)                                                    (((uint32_t)(((uint32_t)(x)) << UART_INTF_PARIF_SHIFT)) & UART_INTF_PARIF_MASK)
#define UART_INTF_TCIF_MASK                                                   (0x4U)
#define UART_INTF_TCIF_SHIFT                                                  (2U)
#define UART_INTF_TCIF(x)                                                     (((uint32_t)(((uint32_t)(x)) << UART_INTF_TCIF_SHIFT)) & UART_INTF_TCIF_MASK)
#define UART_INTF_TXIF_MASK                                                   (0x2U)
#define UART_INTF_TXIF_SHIFT                                                  (1U)
#define UART_INTF_TXIF(x)                                                     (((uint32_t)(((uint32_t)(x)) << UART_INTF_TXIF_SHIFT)) & UART_INTF_TXIF_MASK)
#define UART_INTF_RXIF_MASK                                                   (0x1U)
#define UART_INTF_RXIF_SHIFT                                                  (0U)
#define UART_INTF_RXIF(x)                                                     (((uint32_t)(((uint32_t)(x)) << UART_INTF_RXIF_SHIFT)) & UART_INTF_RXIF_MASK)
/* UART_INTE Register */
#define UART_INTE_LBKDIE_MASK                                                 (0x800U)
#define UART_INTE_LBKDIE_SHIFT                                                (11U)
#define UART_INTE_LBKDIE(x)                                                   (((uint32_t)(((uint32_t)(x)) << UART_INTE_LBKDIE_SHIFT)) & UART_INTE_LBKDIE_MASK)
#define UART_INTE_RXEDGEIE_MASK                                               (0x400U)
#define UART_INTE_RXEDGEIE_SHIFT                                              (10U)
#define UART_INTE_RXEDGEIE(x)                                                 (((uint32_t)(((uint32_t)(x)) << UART_INTE_RXEDGEIE_SHIFT)) & UART_INTE_RXEDGEIE_MASK)
#define UART_INTE_MATIE_MASK                                                  (0x200U)
#define UART_INTE_MATIE_SHIFT                                                 (9U)
#define UART_INTE_MATIE(x)                                                    (((uint32_t)(((uint32_t)(x)) << UART_INTE_MATIE_SHIFT)) & UART_INTE_MATIE_MASK)
#define UART_INTE_NOZIE_MASK                                                  (0x80U)
#define UART_INTE_NOZIE_SHIFT                                                 (7U)
#define UART_INTE_NOZIE(x)                                                    (((uint32_t)(((uint32_t)(x)) << UART_INTE_NOZIE_SHIFT)) & UART_INTE_NOZIE_MASK)
#define UART_INTE_OVRIE_MASK                                                  (0x40U)
#define UART_INTE_OVRIE_SHIFT                                                 (6U)
#define UART_INTE_OVRIE(x)                                                    (((uint32_t)(((uint32_t)(x)) << UART_INTE_OVRIE_SHIFT)) & UART_INTE_OVRIE_MASK)
#define UART_INTE_FEIE_MASK                                                   (0x20U)
#define UART_INTE_FEIE_SHIFT                                                  (5U)
#define UART_INTE_FEIE(x)                                                     (((uint32_t)(((uint32_t)(x)) << UART_INTE_FEIE_SHIFT)) & UART_INTE_FEIE_MASK)
#define UART_INTE_IDLEIE_MASK                                                 (0x10U)
#define UART_INTE_IDLEIE_SHIFT                                                (4U)
#define UART_INTE_IDLEIE(x)                                                   (((uint32_t)(((uint32_t)(x)) << UART_INTE_IDLEIE_SHIFT)) & UART_INTE_IDLEIE_MASK)
#define UART_INTE_PARIE_MASK                                                  (0x8U)
#define UART_INTE_PARIE_SHIFT                                                 (3U)
#define UART_INTE_PARIE(x)                                                    (((uint32_t)(((uint32_t)(x)) << UART_INTE_PARIE_SHIFT)) & UART_INTE_PARIE_MASK)
#define UART_INTE_TCIE_MASK                                                   (0x4U)
#define UART_INTE_TCIE_SHIFT                                                  (2U)
#define UART_INTE_TCIE(x)                                                     (((uint32_t)(((uint32_t)(x)) << UART_INTE_TCIE_SHIFT)) & UART_INTE_TCIE_MASK)
#define UART_INTE_TXIE_MASK                                                   (0x2U)
#define UART_INTE_TXIE_SHIFT                                                  (1U)
#define UART_INTE_TXIE(x)                                                     (((uint32_t)(((uint32_t)(x)) << UART_INTE_TXIE_SHIFT)) & UART_INTE_TXIE_MASK)
#define UART_INTE_RXIE_MASK                                                   (0x1U)
#define UART_INTE_RXIE_SHIFT                                                  (0U)
#define UART_INTE_RXIE(x)                                                     (((uint32_t)(((uint32_t)(x)) << UART_INTE_RXIE_SHIFT)) & UART_INTE_RXIE_MASK)
/* UART_MATCH Register */
#define UART_MATCH_DATAB_MASK                                                 (0x3FF0000U)
#define UART_MATCH_DATAB_SHIFT                                                (16U)
#define UART_MATCH_DATAB(x)                                                   (((uint32_t)(((uint32_t)(x)) << UART_MATCH_DATAB_SHIFT)) & UART_MATCH_DATAB_MASK)
#define UART_MATCH_DATAA_MASK                                                 (0x3FFU)
#define UART_MATCH_DATAA_SHIFT                                                (0U)
#define UART_MATCH_DATAA(x)                                                   (((uint32_t)(((uint32_t)(x)) << UART_MATCH_DATAA_SHIFT)) & UART_MATCH_DATAA_MASK)
/* UART_DATA Register */
#define UART_DATA_FERR_MASK                                                   (0x100000U)
#define UART_DATA_FERR_SHIFT                                                  (20U)
#define UART_DATA_FERR(x)                                                     (((uint32_t)(((uint32_t)(x)) << UART_DATA_FERR_SHIFT)) & UART_DATA_FERR_MASK)
#define UART_DATA_NOISE_MASK                                                  (0x80000U)
#define UART_DATA_NOISE_SHIFT                                                 (19U)
#define UART_DATA_NOISE(x)                                                    (((uint32_t)(((uint32_t)(x)) << UART_DATA_NOISE_SHIFT)) & UART_DATA_NOISE_MASK)
#define UART_DATA_PARERR_MASK                                                 (0x40000U)
#define UART_DATA_PARERR_SHIFT                                                (18U)
#define UART_DATA_PARERR(x)                                                   (((uint32_t)(((uint32_t)(x)) << UART_DATA_PARERR_SHIFT)) & UART_DATA_PARERR_MASK)
#define UART_DATA_IDLE_MASK                                                   (0x20000U)
#define UART_DATA_IDLE_SHIFT                                                  (17U)
#define UART_DATA_IDLE(x)                                                     (((uint32_t)(((uint32_t)(x)) << UART_DATA_IDLE_SHIFT)) & UART_DATA_IDLE_MASK)
#define UART_DATA_RXEMPT_MASK                                                 (0x10000U)
#define UART_DATA_RXEMPT_SHIFT                                                (16U)
#define UART_DATA_RXEMPT(x)                                                   (((uint32_t)(((uint32_t)(x)) << UART_DATA_RXEMPT_SHIFT)) & UART_DATA_RXEMPT_MASK)
#define UART_DATA_LINBRK_MASK                                                 (0x8000U)
#define UART_DATA_LINBRK_SHIFT                                                (15U)
#define UART_DATA_LINBRK(x)                                                   (((uint32_t)(((uint32_t)(x)) << UART_DATA_LINBRK_SHIFT)) & UART_DATA_LINBRK_MASK)
#define UART_DATA_DATA_MASK                                                   (0x3FFU)
#define UART_DATA_DATA_SHIFT                                                  (0U)
#define UART_DATA_DATA(x)                                                     (((uint32_t)(((uint32_t)(x)) << UART_DATA_DATA_SHIFT)) & UART_DATA_DATA_MASK)


/*!
 * @}
 */ /* end of group UART_Register_Masks */


/*!
 * @}
 */ /* end of group UART_Peripheral_Access_Layer */


/* -------------------------------------------------------------
   -- LINPHY Peripheral Access Layer
   ------------------------------------------------------------- */

/*!
 * @addtogroup LINPHY_Peripheral_Access_Layer LINPHY Peripheral Access Layer
 * @{
 */


/** LINPHY - Size of Registers Arrays */

/* LINPHY Register Layout Typedef */
typedef struct {
    __IO uint32_t SR;                                                         /**< Status Register, offset: 0x0000 */
    __IO uint32_t CR;                                                         /**< Control Register, offset: 0x0004 */
    __IO uint32_t PCR;                                                        /**< Power Control Register, offset: 0x0008 */
    __IO uint32_t BSMR;                                                       /**< BSM Register, offset: 0x000C */
    __IO uint32_t IER;                                                        /**< Interrupt Enable Register, offset: 0x0010 */
    __IO uint32_t SRC1R;                                                      /**< Source 1 Register, offset: 0x0014 */
    __IO uint32_t SRC2R;                                                      /**< Source 2 Register, offset: 0x0018 */
    __IO uint32_t DIFFR;                                                      /**< Diff Register, offset: 0x001C */
    __IO uint32_t TRIMR;                                                      /**< Trim Register, offset: 0x0020 */
    __IO uint32_t RESERVED9;                                                  /**< RESERVED9, offset: 0x0024 */

} LINPHY_Type, *LINPHY_MemMapPtr;

/** Number of instances of the LINPHY module. */
#define LINPHY_INSTANCE_COUNT                                                 (1u)

/* LINPHY base address */
#define LINPHY_BASE                                                           (0x40070000)
#define LINPHY                                                                ((LINPHY_Type *)(LINPHY_BASE))

/** Array initializer of LINPHY peripheral base addresses */
#define LINPHY_BASE_ADDRS                                                     { LINPHY_BASE }
/** Array initializer of LINPHY peripheral base pointers */
#define LINPHY_BASE_PTRS                                                      { LINPHY }

/* LINPHY Register Mask */
/* LINPHY_SR Register */
#define LINPHY_SR_BSMSTS_MASK                                                 (0x7000U)
#define LINPHY_SR_BSMSTS_SHIFT                                                (12U)
#define LINPHY_SR_BSMSTS(x)                                                   (((uint32_t)(((uint32_t)(x)) << LINPHY_SR_BSMSTS_SHIFT)) & LINPHY_SR_BSMSTS_MASK)
#define LINPHY_SR_PUENHSTS_MASK                                               (0x800U)
#define LINPHY_SR_PUENHSTS_SHIFT                                              (11U)
#define LINPHY_SR_PUENHSTS(x)                                                 (((uint32_t)(((uint32_t)(x)) << LINPHY_SR_PUENHSTS_SHIFT)) & LINPHY_SR_PUENHSTS_MASK)
#define LINPHY_SR_CDOUTENSTS_MASK                                             (0x400U)
#define LINPHY_SR_CDOUTENSTS_SHIFT                                            (10U)
#define LINPHY_SR_CDOUTENSTS(x)                                               (((uint32_t)(((uint32_t)(x)) << LINPHY_SR_CDOUTENSTS_SHIFT)) & LINPHY_SR_CDOUTENSTS_MASK)
#define LINPHY_SR_CDENSTS_MASK                                                (0x200U)
#define LINPHY_SR_CDENSTS_SHIFT                                               (9U)
#define LINPHY_SR_CDENSTS(x)                                                  (((uint32_t)(((uint32_t)(x)) << LINPHY_SR_CDENSTS_SHIFT)) & LINPHY_SR_CDENSTS_MASK)
#define LINPHY_SR_RXDSTS_MASK                                                 (0x100U)
#define LINPHY_SR_RXDSTS_SHIFT                                                (8U)
#define LINPHY_SR_RXDSTS(x)                                                   (((uint32_t)(((uint32_t)(x)) << LINPHY_SR_RXDSTS_SHIFT)) & LINPHY_SR_RXDSTS_MASK)
#define LINPHY_SR_LINIRQ_MASK                                                 (0x8U)
#define LINPHY_SR_LINIRQ_SHIFT                                                (3U)
#define LINPHY_SR_LINIRQ(x)                                                   (((uint32_t)(((uint32_t)(x)) << LINPHY_SR_LINIRQ_SHIFT)) & LINPHY_SR_LINIRQ_MASK)
#define LINPHY_SR_ADCERR_MASK                                                 (0x4U)
#define LINPHY_SR_ADCERR_SHIFT                                                (2U)
#define LINPHY_SR_ADCERR(x)                                                   (((uint32_t)(((uint32_t)(x)) << LINPHY_SR_ADCERR_SHIFT)) & LINPHY_SR_ADCERR_MASK)
#define LINPHY_SR_BSMSUCCESS_MASK                                             (0x2U)
#define LINPHY_SR_BSMSUCCESS_SHIFT                                            (1U)
#define LINPHY_SR_BSMSUCCESS(x)                                               (((uint32_t)(((uint32_t)(x)) << LINPHY_SR_BSMSUCCESS_SHIFT)) & LINPHY_SR_BSMSUCCESS_MASK)
#define LINPHY_SR_BSMDONE_MASK                                                (0x1U)
#define LINPHY_SR_BSMDONE_SHIFT                                               (0U)
#define LINPHY_SR_BSMDONE(x)                                                  (((uint32_t)(((uint32_t)(x)) << LINPHY_SR_BSMDONE_SHIFT)) & LINPHY_SR_BSMDONE_MASK)
/* LINPHY_CR Register */
#define LINPHY_CR_LINEN_MASK                                                  (0x1U)
#define LINPHY_CR_LINEN_SHIFT                                                 (0U)
#define LINPHY_CR_LINEN(x)                                                    (((uint32_t)(((uint32_t)(x)) << LINPHY_CR_LINEN_SHIFT)) & LINPHY_CR_LINEN_MASK)
/* LINPHY_PCR Register */
#define LINPHY_PCR_SRCFG_MASK                                                 (0x70U)
#define LINPHY_PCR_SRCFG_SHIFT                                                (4U)
#define LINPHY_PCR_SRCFG(x)                                                   (((uint32_t)(((uint32_t)(x)) << LINPHY_PCR_SRCFG_SHIFT)) & LINPHY_PCR_SRCFG_MASK)
#define LINPHY_PCR_PUENH_MASK                                                 (0x4U)
#define LINPHY_PCR_PUENH_SHIFT                                                (2U)
#define LINPHY_PCR_PUENH(x)                                                   (((uint32_t)(((uint32_t)(x)) << LINPHY_PCR_PUENH_SHIFT)) & LINPHY_PCR_PUENH_MASK)
#define LINPHY_PCR_PUEN_MASK                                                  (0x1U)
#define LINPHY_PCR_PUEN_SHIFT                                                 (0U)
#define LINPHY_PCR_PUEN(x)                                                    (((uint32_t)(((uint32_t)(x)) << LINPHY_PCR_PUEN_SHIFT)) & LINPHY_PCR_PUEN_MASK)
/* LINPHY_BSMR Register */
#define LINPHY_BSMR_CDOUTEN_MASK                                              (0x20000U)
#define LINPHY_BSMR_CDOUTEN_SHIFT                                             (17U)
#define LINPHY_BSMR_CDOUTEN(x)                                                (((uint32_t)(((uint32_t)(x)) << LINPHY_BSMR_CDOUTEN_SHIFT)) & LINPHY_BSMR_CDOUTEN_MASK)
#define LINPHY_BSMR_CDEN_MASK                                                 (0x10000U)
#define LINPHY_BSMR_CDEN_SHIFT                                                (16U)
#define LINPHY_BSMR_CDEN(x)                                                   (((uint32_t)(((uint32_t)(x)) << LINPHY_BSMR_CDEN_SHIFT)) & LINPHY_BSMR_CDEN_MASK)
#define LINPHY_BSMR_BSMAUTO_MASK                                              (0x2000U)
#define LINPHY_BSMR_BSMAUTO_SHIFT                                             (13U)
#define LINPHY_BSMR_BSMAUTO(x)                                                (((uint32_t)(((uint32_t)(x)) << LINPHY_BSMR_BSMAUTO_SHIFT)) & LINPHY_BSMR_BSMAUTO_MASK)
#define LINPHY_BSMR_DACEN_MASK                                                (0x1000U)
#define LINPHY_BSMR_DACEN_SHIFT                                               (12U)
#define LINPHY_BSMR_DACEN(x)                                                  (((uint32_t)(((uint32_t)(x)) << LINPHY_BSMR_DACEN_SHIFT)) & LINPHY_BSMR_DACEN_MASK)
#define LINPHY_BSMR_SCSH_MASK                                                 (0x300U)
#define LINPHY_BSMR_SCSH_SHIFT                                                (8U)
#define LINPHY_BSMR_SCSH(x)                                                   (((uint32_t)(((uint32_t)(x)) << LINPHY_BSMR_SCSH_SHIFT)) & LINPHY_BSMR_SCSH_MASK)
#define LINPHY_BSMR_SCGMSEL_MASK                                              (0xF0U)
#define LINPHY_BSMR_SCGMSEL_SHIFT                                             (4U)
#define LINPHY_BSMR_SCGMSEL(x)                                                (((uint32_t)(((uint32_t)(x)) << LINPHY_BSMR_SCGMSEL_SHIFT)) & LINPHY_BSMR_SCGMSEL_MASK)
#define LINPHY_BSMR_SCVCMSEL_MASK                                             (0x4U)
#define LINPHY_BSMR_SCVCMSEL_SHIFT                                            (2U)
#define LINPHY_BSMR_SCVCMSEL(x)                                               (((uint32_t)(((uint32_t)(x)) << LINPHY_BSMR_SCVCMSEL_SHIFT)) & LINPHY_BSMR_SCVCMSEL_MASK)
#define LINPHY_BSMR_SCAMPEN_MASK                                              (0x2U)
#define LINPHY_BSMR_SCAMPEN_SHIFT                                             (1U)
#define LINPHY_BSMR_SCAMPEN(x)                                                (((uint32_t)(((uint32_t)(x)) << LINPHY_BSMR_SCAMPEN_SHIFT)) & LINPHY_BSMR_SCAMPEN_MASK)
#define LINPHY_BSMR_BSMEN_MASK                                                (0x1U)
#define LINPHY_BSMR_BSMEN_SHIFT                                               (0U)
#define LINPHY_BSMR_BSMEN(x)                                                  (((uint32_t)(((uint32_t)(x)) << LINPHY_BSMR_BSMEN_SHIFT)) & LINPHY_BSMR_BSMEN_MASK)
/* LINPHY_IER Register */
#define LINPHY_IER_BSMIE_MASK                                                 (0x100U)
#define LINPHY_IER_BSMIE_SHIFT                                                (8U)
#define LINPHY_IER_BSMIE(x)                                                   (((uint32_t)(((uint32_t)(x)) << LINPHY_IER_BSMIE_SHIFT)) & LINPHY_IER_BSMIE_MASK)
#define LINPHY_IER_LINIE_MASK                                                 (0x1U)
#define LINPHY_IER_LINIE_SHIFT                                                (0U)
#define LINPHY_IER_LINIE(x)                                                   (((uint32_t)(((uint32_t)(x)) << LINPHY_IER_LINIE_SHIFT)) & LINPHY_IER_LINIE_MASK)
/* LINPHY_SRC1R Register */
#define LINPHY_SRC1R_SRC1DATA_MASK                                            (0xFFU)
#define LINPHY_SRC1R_SRC1DATA_SHIFT                                           (0U)
#define LINPHY_SRC1R_SRC1DATA(x)                                              (((uint32_t)(((uint32_t)(x)) << LINPHY_SRC1R_SRC1DATA_SHIFT)) & LINPHY_SRC1R_SRC1DATA_MASK)
/* LINPHY_SRC2R Register */
#define LINPHY_SRC2R_SRC2DATA_MASK                                            (0xFFU)
#define LINPHY_SRC2R_SRC2DATA_SHIFT                                           (0U)
#define LINPHY_SRC2R_SRC2DATA(x)                                              (((uint32_t)(((uint32_t)(x)) << LINPHY_SRC2R_SRC2DATA_SHIFT)) & LINPHY_SRC2R_SRC2DATA_MASK)
/* LINPHY_DIFFR Register */
#define LINPHY_DIFFR_DIFFVAL_MASK                                             (0xFFFU)
#define LINPHY_DIFFR_DIFFVAL_SHIFT                                            (0U)
#define LINPHY_DIFFR_DIFFVAL(x)                                               (((uint32_t)(((uint32_t)(x)) << LINPHY_DIFFR_DIFFVAL_SHIFT)) & LINPHY_DIFFR_DIFFVAL_MASK)
/* LINPHY_TRIMR Register */
#define LINPHY_TRIMR_DACTRIM_MASK                                             (0x1F00U)
#define LINPHY_TRIMR_DACTRIM_SHIFT                                            (8U)
#define LINPHY_TRIMR_DACTRIM(x)                                               (((uint32_t)(((uint32_t)(x)) << LINPHY_TRIMR_DACTRIM_SHIFT)) & LINPHY_TRIMR_DACTRIM_MASK)
#define LINPHY_TRIMR_ADCTRIM_MASK                                             (0x1FU)
#define LINPHY_TRIMR_ADCTRIM_SHIFT                                            (0U)
#define LINPHY_TRIMR_ADCTRIM(x)                                               (((uint32_t)(((uint32_t)(x)) << LINPHY_TRIMR_ADCTRIM_SHIFT)) & LINPHY_TRIMR_ADCTRIM_MASK)


/*!
 * @}
 */ /* end of group LINPHY_Register_Masks */


/*!
 * @}
 */ /* end of group LINPHY_Peripheral_Access_Layer */


/* -------------------------------------------------------------
   -- CCTMR Peripheral Access Layer
   ------------------------------------------------------------- */

/*!
 * @addtogroup CCTMR_Peripheral_Access_Layer CCTMR Peripheral Access Layer
 * @{
 */


/** CCTMR - Size of Registers Arrays */
#define CCTMR_CH_COUNT                                                        (4)

/* CCTMR Register Layout Typedef */
typedef struct {
    __IO uint32_t STS;                                                        /**< Status Register, offset: 0x0000 */
    __IO uint32_t INTE;                                                       /**< Interrupt Enable Register, offset: 0x0004 */
    __IO uint32_t CTRL;                                                       /**< Control Register, offset: 0x0008 */
    __IO uint32_t SYNC;                                                       /**< Synchronous Register, offset: 0x000C */
    __IO uint8_t  RESERVED4[16];                                              /**< RESERVED4, offset: 0x0010 */
    __IO uint32_t FILT;                                                       /**< Filter Register, offset: 0x0020 */
    __IO uint8_t  RESERVED6[12];                                              /**< RESERVED6, offset: 0x0024 */
    __IO uint32_t RESERVED7;                                                  /**< RESERVED7, offset: 0x0030 */
    __IO uint8_t  RESERVED8[12];                                              /**< RESERVED8, offset: 0x0034 */
    struct {
        __IO uint32_t CFG;                                                    /**< Channel Configuration Register, offset: 0x0040 */
        __IO uint32_t VAL0;                                                   /**< Value 0 Register, offset: 0x0044 */
        __IO uint32_t VAL1;                                                   /**< Value 1 Register, offset: 0x0048 */
        __IO uint32_t RESERVEDS93[ 5];                                        /**< RESERVEDS93, offset: 0x004C */

    } CH[4];

} CCTMR_Type, *CCTMR_MemMapPtr;

/** Number of instances of the CCTMR module. */
#define CCTMR_INSTANCE_COUNT                                                  (1u)

/* CCTMR0  base address */
#define CCTMR0_BASE                                                           (0x40039000)
#define CCTMR0                                                                ((CCTMR_Type *)(CCTMR0_BASE))

/** Array initializer of CCTMR peripheral base addresses */
#define CCTMR_BASE_ADDRS                                                      { CCTMR0_BASE }
/** Array initializer of CCTMR peripheral base pointers */
#define CCTMR_BASE_PTRS                                                       { CCTMR0 }

/* CCTMR Register Mask */
/* CCTMR_STS Register */
#define CCTMR_STS_CH3IO_MASK                                                  (0x80000U)
#define CCTMR_STS_CH3IO_SHIFT                                                 (19U)
#define CCTMR_STS_CH3IO(x)                                                    (((uint32_t)(((uint32_t)(x)) << CCTMR_STS_CH3IO_SHIFT)) & CCTMR_STS_CH3IO_MASK)
#define CCTMR_STS_CH2IO_MASK                                                  (0x40000U)
#define CCTMR_STS_CH2IO_SHIFT                                                 (18U)
#define CCTMR_STS_CH2IO(x)                                                    (((uint32_t)(((uint32_t)(x)) << CCTMR_STS_CH2IO_SHIFT)) & CCTMR_STS_CH2IO_MASK)
#define CCTMR_STS_CH1IO_MASK                                                  (0x20000U)
#define CCTMR_STS_CH1IO_SHIFT                                                 (17U)
#define CCTMR_STS_CH1IO(x)                                                    (((uint32_t)(((uint32_t)(x)) << CCTMR_STS_CH1IO_SHIFT)) & CCTMR_STS_CH1IO_MASK)
#define CCTMR_STS_CH0IO_MASK                                                  (0x10000U)
#define CCTMR_STS_CH0IO_SHIFT                                                 (16U)
#define CCTMR_STS_CH0IO(x)                                                    (((uint32_t)(((uint32_t)(x)) << CCTMR_STS_CH0IO_SHIFT)) & CCTMR_STS_CH0IO_MASK)
#define CCTMR_STS_CH3TOF_MASK                                                 (0x800U)
#define CCTMR_STS_CH3TOF_SHIFT                                                (11U)
#define CCTMR_STS_CH3TOF(x)                                                   (((uint32_t)(((uint32_t)(x)) << CCTMR_STS_CH3TOF_SHIFT)) & CCTMR_STS_CH3TOF_MASK)
#define CCTMR_STS_CH2TOF_MASK                                                 (0x400U)
#define CCTMR_STS_CH2TOF_SHIFT                                                (10U)
#define CCTMR_STS_CH2TOF(x)                                                   (((uint32_t)(((uint32_t)(x)) << CCTMR_STS_CH2TOF_SHIFT)) & CCTMR_STS_CH2TOF_MASK)
#define CCTMR_STS_CH1TOF_MASK                                                 (0x200U)
#define CCTMR_STS_CH1TOF_SHIFT                                                (9U)
#define CCTMR_STS_CH1TOF(x)                                                   (((uint32_t)(((uint32_t)(x)) << CCTMR_STS_CH1TOF_SHIFT)) & CCTMR_STS_CH1TOF_MASK)
#define CCTMR_STS_CH0TOF_MASK                                                 (0x100U)
#define CCTMR_STS_CH0TOF_SHIFT                                                (8U)
#define CCTMR_STS_CH0TOF(x)                                                   (((uint32_t)(((uint32_t)(x)) << CCTMR_STS_CH0TOF_SHIFT)) & CCTMR_STS_CH0TOF_MASK)
#define CCTMR_STS_CH3F1_MASK                                                  (0x80U)
#define CCTMR_STS_CH3F1_SHIFT                                                 (7U)
#define CCTMR_STS_CH3F1(x)                                                    (((uint32_t)(((uint32_t)(x)) << CCTMR_STS_CH3F1_SHIFT)) & CCTMR_STS_CH3F1_MASK)
#define CCTMR_STS_CH2F1_MASK                                                  (0x40U)
#define CCTMR_STS_CH2F1_SHIFT                                                 (6U)
#define CCTMR_STS_CH2F1(x)                                                    (((uint32_t)(((uint32_t)(x)) << CCTMR_STS_CH2F1_SHIFT)) & CCTMR_STS_CH2F1_MASK)
#define CCTMR_STS_CH1F1_MASK                                                  (0x20U)
#define CCTMR_STS_CH1F1_SHIFT                                                 (5U)
#define CCTMR_STS_CH1F1(x)                                                    (((uint32_t)(((uint32_t)(x)) << CCTMR_STS_CH1F1_SHIFT)) & CCTMR_STS_CH1F1_MASK)
#define CCTMR_STS_CH0F1_MASK                                                  (0x10U)
#define CCTMR_STS_CH0F1_SHIFT                                                 (4U)
#define CCTMR_STS_CH0F1(x)                                                    (((uint32_t)(((uint32_t)(x)) << CCTMR_STS_CH0F1_SHIFT)) & CCTMR_STS_CH0F1_MASK)
#define CCTMR_STS_CH3F0_MASK                                                  (0x8U)
#define CCTMR_STS_CH3F0_SHIFT                                                 (3U)
#define CCTMR_STS_CH3F0(x)                                                    (((uint32_t)(((uint32_t)(x)) << CCTMR_STS_CH3F0_SHIFT)) & CCTMR_STS_CH3F0_MASK)
#define CCTMR_STS_CH2F0_MASK                                                  (0x4U)
#define CCTMR_STS_CH2F0_SHIFT                                                 (2U)
#define CCTMR_STS_CH2F0(x)                                                    (((uint32_t)(((uint32_t)(x)) << CCTMR_STS_CH2F0_SHIFT)) & CCTMR_STS_CH2F0_MASK)
#define CCTMR_STS_CH1F0_MASK                                                  (0x2U)
#define CCTMR_STS_CH1F0_SHIFT                                                 (1U)
#define CCTMR_STS_CH1F0(x)                                                    (((uint32_t)(((uint32_t)(x)) << CCTMR_STS_CH1F0_SHIFT)) & CCTMR_STS_CH1F0_MASK)
#define CCTMR_STS_CH0F0_MASK                                                  (0x1U)
#define CCTMR_STS_CH0F0_SHIFT                                                 (0U)
#define CCTMR_STS_CH0F0(x)                                                    (((uint32_t)(((uint32_t)(x)) << CCTMR_STS_CH0F0_SHIFT)) & CCTMR_STS_CH0F0_MASK)
/* CCTMR_INTE Register */
#define CCTMR_INTE_TO3IE_MASK                                                 (0x800U)
#define CCTMR_INTE_TO3IE_SHIFT                                                (11U)
#define CCTMR_INTE_TO3IE(x)                                                   (((uint32_t)(((uint32_t)(x)) << CCTMR_INTE_TO3IE_SHIFT)) & CCTMR_INTE_TO3IE_MASK)
#define CCTMR_INTE_TO2IE_MASK                                                 (0x400U)
#define CCTMR_INTE_TO2IE_SHIFT                                                (10U)
#define CCTMR_INTE_TO2IE(x)                                                   (((uint32_t)(((uint32_t)(x)) << CCTMR_INTE_TO2IE_SHIFT)) & CCTMR_INTE_TO2IE_MASK)
#define CCTMR_INTE_TO1IE_MASK                                                 (0x200U)
#define CCTMR_INTE_TO1IE_SHIFT                                                (9U)
#define CCTMR_INTE_TO1IE(x)                                                   (((uint32_t)(((uint32_t)(x)) << CCTMR_INTE_TO1IE_SHIFT)) & CCTMR_INTE_TO1IE_MASK)
#define CCTMR_INTE_TO0IE_MASK                                                 (0x100U)
#define CCTMR_INTE_TO0IE_SHIFT                                                (8U)
#define CCTMR_INTE_TO0IE(x)                                                   (((uint32_t)(((uint32_t)(x)) << CCTMR_INTE_TO0IE_SHIFT)) & CCTMR_INTE_TO0IE_MASK)
#define CCTMR_INTE_CH3IE_MASK                                                 (0x8U)
#define CCTMR_INTE_CH3IE_SHIFT                                                (3U)
#define CCTMR_INTE_CH3IE(x)                                                   (((uint32_t)(((uint32_t)(x)) << CCTMR_INTE_CH3IE_SHIFT)) & CCTMR_INTE_CH3IE_MASK)
#define CCTMR_INTE_CH2IE_MASK                                                 (0x4U)
#define CCTMR_INTE_CH2IE_SHIFT                                                (2U)
#define CCTMR_INTE_CH2IE(x)                                                   (((uint32_t)(((uint32_t)(x)) << CCTMR_INTE_CH2IE_SHIFT)) & CCTMR_INTE_CH2IE_MASK)
#define CCTMR_INTE_CH1IE_MASK                                                 (0x2U)
#define CCTMR_INTE_CH1IE_SHIFT                                                (1U)
#define CCTMR_INTE_CH1IE(x)                                                   (((uint32_t)(((uint32_t)(x)) << CCTMR_INTE_CH1IE_SHIFT)) & CCTMR_INTE_CH1IE_MASK)
#define CCTMR_INTE_CH0IE_MASK                                                 (0x1U)
#define CCTMR_INTE_CH0IE_SHIFT                                                (0U)
#define CCTMR_INTE_CH0IE(x)                                                   (((uint32_t)(((uint32_t)(x)) << CCTMR_INTE_CH0IE_SHIFT)) & CCTMR_INTE_CH0IE_MASK)
/* CCTMR_CTRL Register */
#define CCTMR_CTRL_DBGDIS_MASK                                                (0x80000000U)
#define CCTMR_CTRL_DBGDIS_SHIFT                                               (31U)
#define CCTMR_CTRL_DBGDIS(x)                                                  (((uint32_t)(((uint32_t)(x)) << CCTMR_CTRL_DBGDIS_SHIFT)) & CCTMR_CTRL_DBGDIS_MASK)
#define CCTMR_CTRL_CLDOK_MASK                                                 (0x20000000U)
#define CCTMR_CTRL_CLDOK_SHIFT                                                (29U)
#define CCTMR_CTRL_CLDOK(x)                                                   (((uint32_t)(((uint32_t)(x)) << CCTMR_CTRL_CLDOK_SHIFT)) & CCTMR_CTRL_CLDOK_MASK)
#define CCTMR_CTRL_LDOK_MASK                                                  (0x10000000U)
#define CCTMR_CTRL_LDOK_SHIFT                                                 (28U)
#define CCTMR_CTRL_LDOK(x)                                                    (((uint32_t)(((uint32_t)(x)) << CCTMR_CTRL_LDOK_SHIFT)) & CCTMR_CTRL_LDOK_MASK)
#define CCTMR_CTRL_COMB23_MASK                                                (0x2000000U)
#define CCTMR_CTRL_COMB23_SHIFT                                               (25U)
#define CCTMR_CTRL_COMB23(x)                                                  (((uint32_t)(((uint32_t)(x)) << CCTMR_CTRL_COMB23_SHIFT)) & CCTMR_CTRL_COMB23_MASK)
#define CCTMR_CTRL_COMB01_MASK                                                (0x1000000U)
#define CCTMR_CTRL_COMB01_SHIFT                                               (24U)
#define CCTMR_CTRL_COMB01(x)                                                  (((uint32_t)(((uint32_t)(x)) << CCTMR_CTRL_COMB01_SHIFT)) & CCTMR_CTRL_COMB01_MASK)
#define CCTMR_CTRL_CH3EN_MASK                                                 (0x8U)
#define CCTMR_CTRL_CH3EN_SHIFT                                                (3U)
#define CCTMR_CTRL_CH3EN(x)                                                   (((uint32_t)(((uint32_t)(x)) << CCTMR_CTRL_CH3EN_SHIFT)) & CCTMR_CTRL_CH3EN_MASK)
#define CCTMR_CTRL_CH2EN_MASK                                                 (0x4U)
#define CCTMR_CTRL_CH2EN_SHIFT                                                (2U)
#define CCTMR_CTRL_CH2EN(x)                                                   (((uint32_t)(((uint32_t)(x)) << CCTMR_CTRL_CH2EN_SHIFT)) & CCTMR_CTRL_CH2EN_MASK)
#define CCTMR_CTRL_CH1EN_MASK                                                 (0x2U)
#define CCTMR_CTRL_CH1EN_SHIFT                                                (1U)
#define CCTMR_CTRL_CH1EN(x)                                                   (((uint32_t)(((uint32_t)(x)) << CCTMR_CTRL_CH1EN_SHIFT)) & CCTMR_CTRL_CH1EN_MASK)
#define CCTMR_CTRL_CH0EN_MASK                                                 (0x1U)
#define CCTMR_CTRL_CH0EN_SHIFT                                                (0U)
#define CCTMR_CTRL_CH0EN(x)                                                   (((uint32_t)(((uint32_t)(x)) << CCTMR_CTRL_CH0EN_SHIFT)) & CCTMR_CTRL_CH0EN_MASK)
/* CCTMR_SYNC Register */
#define CCTMR_SYNC_CH3TSRC_MASK                                               (0xC00000U)
#define CCTMR_SYNC_CH3TSRC_SHIFT                                              (22U)
#define CCTMR_SYNC_CH3TSRC(x)                                                 (((uint32_t)(((uint32_t)(x)) << CCTMR_SYNC_CH3TSRC_SHIFT)) & CCTMR_SYNC_CH3TSRC_MASK)
#define CCTMR_SYNC_CH2TSRC_MASK                                               (0x300000U)
#define CCTMR_SYNC_CH2TSRC_SHIFT                                              (20U)
#define CCTMR_SYNC_CH2TSRC(x)                                                 (((uint32_t)(((uint32_t)(x)) << CCTMR_SYNC_CH2TSRC_SHIFT)) & CCTMR_SYNC_CH2TSRC_MASK)
#define CCTMR_SYNC_CH1TSRC_MASK                                               (0xC0000U)
#define CCTMR_SYNC_CH1TSRC_SHIFT                                              (18U)
#define CCTMR_SYNC_CH1TSRC(x)                                                 (((uint32_t)(((uint32_t)(x)) << CCTMR_SYNC_CH1TSRC_SHIFT)) & CCTMR_SYNC_CH1TSRC_MASK)
#define CCTMR_SYNC_CH0TSRC_MASK                                               (0x30000U)
#define CCTMR_SYNC_CH0TSRC_SHIFT                                              (16U)
#define CCTMR_SYNC_CH0TSRC(x)                                                 (((uint32_t)(((uint32_t)(x)) << CCTMR_SYNC_CH0TSRC_SHIFT)) & CCTMR_SYNC_CH0TSRC_MASK)
#define CCTMR_SYNC_CH3VALSYNC_MASK                                            (0x8U)
#define CCTMR_SYNC_CH3VALSYNC_SHIFT                                           (3U)
#define CCTMR_SYNC_CH3VALSYNC(x)                                              (((uint32_t)(((uint32_t)(x)) << CCTMR_SYNC_CH3VALSYNC_SHIFT)) & CCTMR_SYNC_CH3VALSYNC_MASK)
#define CCTMR_SYNC_CH2VALSYNC_MASK                                            (0x4U)
#define CCTMR_SYNC_CH2VALSYNC_SHIFT                                           (2U)
#define CCTMR_SYNC_CH2VALSYNC(x)                                              (((uint32_t)(((uint32_t)(x)) << CCTMR_SYNC_CH2VALSYNC_SHIFT)) & CCTMR_SYNC_CH2VALSYNC_MASK)
#define CCTMR_SYNC_CH1VALSYNC_MASK                                            (0x2U)
#define CCTMR_SYNC_CH1VALSYNC_SHIFT                                           (1U)
#define CCTMR_SYNC_CH1VALSYNC(x)                                              (((uint32_t)(((uint32_t)(x)) << CCTMR_SYNC_CH1VALSYNC_SHIFT)) & CCTMR_SYNC_CH1VALSYNC_MASK)
#define CCTMR_SYNC_CH0VALSYNC_MASK                                            (0x1U)
#define CCTMR_SYNC_CH0VALSYNC_SHIFT                                           (0U)
#define CCTMR_SYNC_CH0VALSYNC(x)                                              (((uint32_t)(((uint32_t)(x)) << CCTMR_SYNC_CH0VALSYNC_SHIFT)) & CCTMR_SYNC_CH0VALSYNC_MASK)
/* CCTMR_FILT Register */
#define CCTMR_FILT_CNT1_WIDTH                                                 (4)
#define CCTMR_FILT_CNT1_MASK                                                  (0xF0000000U)
#define CCTMR_FILT_CNT1_SHIFT                                                 (28U)
#define CCTMR_FILT_CNT1(x)                                                    (((uint32_t)(((uint32_t)(x)) << CCTMR_FILT_CNT1_SHIFT)) & CCTMR_FILT_CNT1_MASK)
#define CCTMR_FILT_PER1_WIDTH                                                 (4)
#define CCTMR_FILT_PER1_MASK                                                  (0xF000000U)
#define CCTMR_FILT_PER1_SHIFT                                                 (24U)
#define CCTMR_FILT_PER1(x)                                                    (((uint32_t)(((uint32_t)(x)) << CCTMR_FILT_PER1_SHIFT)) & CCTMR_FILT_PER1_MASK)
#define CCTMR_FILT_CNT0_WIDTH                                                 (4)
#define CCTMR_FILT_CNT0_MASK                                                  (0xF000U)
#define CCTMR_FILT_CNT0_SHIFT                                                 (12U)
#define CCTMR_FILT_CNT0(x)                                                    (((uint32_t)(((uint32_t)(x)) << CCTMR_FILT_CNT0_SHIFT)) & CCTMR_FILT_CNT0_MASK)
#define CCTMR_FILT_PER0_WIDTH                                                 (4)
#define CCTMR_FILT_PER0_MASK                                                  (0xF00U)
#define CCTMR_FILT_PER0_SHIFT                                                 (8U)
#define CCTMR_FILT_PER0(x)                                                    (((uint32_t)(((uint32_t)(x)) << CCTMR_FILT_PER0_SHIFT)) & CCTMR_FILT_PER0_MASK)
#define CCTMR_FILT_SEL_WIDTH                                                  (3)
#define CCTMR_FILT_SEL_MASK                                                   (0x7U)
#define CCTMR_FILT_SEL_SHIFT                                                  (0U)
#define CCTMR_FILT_SEL(x)                                                     (((uint32_t)(((uint32_t)(x)) << CCTMR_FILT_SEL_SHIFT)) & CCTMR_FILT_SEL_MASK)
/* CCTMR_CH_CFG Register */
#define CCTMR_CH_CFG_CAPMD_WIDTH                                              (2)
#define CCTMR_CH_CFG_CAPMD_MASK                                               (0xC0000U)
#define CCTMR_CH_CFG_CAPMD_SHIFT                                              (18U)
#define CCTMR_CH_CFG_CAPMD(x)                                                 (((uint32_t)(((uint32_t)(x)) << CCTMR_CH_CFG_CAPMD_SHIFT)) & CCTMR_CH_CFG_CAPMD_MASK)
#define CCTMR_CH_CFG_CAPEDGE_WIDTH                                            (2)
#define CCTMR_CH_CFG_CAPEDGE_MASK                                             (0x30000U)
#define CCTMR_CH_CFG_CAPEDGE_SHIFT                                            (16U)
#define CCTMR_CH_CFG_CAPEDGE(x)                                               (((uint32_t)(((uint32_t)(x)) << CCTMR_CH_CFG_CAPEDGE_SHIFT)) & CCTMR_CH_CFG_CAPEDGE_MASK)
#define CCTMR_CH_CFG_VAL1CMP_WIDTH                                            (2)
#define CCTMR_CH_CFG_VAL1CMP_MASK                                             (0xC00U)
#define CCTMR_CH_CFG_VAL1CMP_SHIFT                                            (10U)
#define CCTMR_CH_CFG_VAL1CMP(x)                                               (((uint32_t)(((uint32_t)(x)) << CCTMR_CH_CFG_VAL1CMP_SHIFT)) & CCTMR_CH_CFG_VAL1CMP_MASK)
#define CCTMR_CH_CFG_VAL0CMP_WIDTH                                            (2)
#define CCTMR_CH_CFG_VAL0CMP_MASK                                             (0x300U)
#define CCTMR_CH_CFG_VAL0CMP_SHIFT                                            (8U)
#define CCTMR_CH_CFG_VAL0CMP(x)                                               (((uint32_t)(((uint32_t)(x)) << CCTMR_CH_CFG_VAL0CMP_SHIFT)) & CCTMR_CH_CFG_VAL0CMP_MASK)
#define CCTMR_CH_CFG_POL_MASK                                                 (0x40U)
#define CCTMR_CH_CFG_POL_SHIFT                                                (6U)
#define CCTMR_CH_CFG_POL(x)                                                   (((uint32_t)(((uint32_t)(x)) << CCTMR_CH_CFG_POL_SHIFT)) & CCTMR_CH_CFG_POL_MASK)
#define CCTMR_CH_CFG_MODE_WIDTH                                               (2)
#define CCTMR_CH_CFG_MODE_MASK                                                (0x3U)
#define CCTMR_CH_CFG_MODE_SHIFT                                               (0U)
#define CCTMR_CH_CFG_MODE(x)                                                  (((uint32_t)(((uint32_t)(x)) << CCTMR_CH_CFG_MODE_SHIFT)) & CCTMR_CH_CFG_MODE_MASK)
/* CCTMR_CH_VAL0 Register */
#define CCTMR_CH_VAL0_VAL0_MASK                                               (0xFFFFFFFFU)
#define CCTMR_CH_VAL0_VAL0_SHIFT                                              (0U)
#define CCTMR_CH_VAL0_VAL0(x)                                                 (((uint32_t)(((uint32_t)(x)) << CCTMR_CH_VAL0_VAL0_SHIFT)) & CCTMR_CH_VAL0_VAL0_MASK)
/* CCTMR_CH_VAL1 Register */
#define CCTMR_CH_VAL1_VAL1_MASK                                               (0xFFFFFFFFU)
#define CCTMR_CH_VAL1_VAL1_SHIFT                                              (0U)
#define CCTMR_CH_VAL1_VAL1(x)                                                 (((uint32_t)(((uint32_t)(x)) << CCTMR_CH_VAL1_VAL1_SHIFT)) & CCTMR_CH_VAL1_VAL1_MASK)



/*!
 * @}
 */ /* end of group CCTMR_Register_Masks */


/*!
 * @}
 */ /* end of group CCTMR_Peripheral_Access_Layer */


/* -------------------------------------------------------------
   -- MSPWM Peripheral Access Layer
   ------------------------------------------------------------- */

/*!
 * @addtogroup MSPWM_Peripheral_Access_Layer MSPWM Peripheral Access Layer
 * @{
 */


/** MSPWM - Size of Registers Arrays */
#define MSPWM_CH_COUNT                                                        (6)

/* MSPWM Register Layout Typedef */
typedef struct {
    __IO uint32_t STS;                                                        /**< Status Register, offset: 0x0000 */
    __I  uint32_t CNT;                                                        /**< Counter Value Register, offset: 0x0004 */
    __IO uint32_t CTRL;                                                       /**< Control Register, offset: 0x0008 */
    __IO uint32_t SYNC;                                                       /**< Synchronous Register, offset: 0x000C */
    __IO uint32_t OTRIG;                                                      /**< Output Trigger Register, offset: 0x0010 */
    __IO uint32_t OWIN;                                                       /**< Output Window, offset: 0x0014 */
    __IO uint32_t INTE;                                                       /**< Interrupt Enable Register, offset: 0x0018 */
    __IO uint32_t CHINIT;                                                     /**< Channel Output Initial Value Register, offset: 0x001C */
    __IO uint32_t CHSWV;                                                      /**< Channel Software Value Register, offset: 0x0020 */
    __IO uint32_t CHMASK;                                                     /**< Channel Output Mask Value Register, offset: 0x0024 */
    __IO uint32_t MOD;                                                        /**< Counter Modulus Value Register, offset: 0x0028 */
    __IO uint32_t RESERVED11[ 1];                                             /**< RESERVED11, offset: 0x002C */
    __IO uint32_t RESERVED12[ 1];                                             /**< RESERVED12, offset: 0x0030 */
    __IO uint32_t RESERVED13[ 3];                                             /**< RESERVED13, offset: 0x0034 */
    struct {
        __IO uint32_t CTRL;                                                   /**< Channel Control Register, offset: 0x0080 */
        __IO uint32_t VAL0;                                                   /**< Value 0 Register, offset: 0x0084 */
        __IO uint32_t VAL1;                                                   /**< Value 1 Register, offset: 0x0088 */
        __IO uint32_t RESERVEDS143[ 5];                                       /**< RESERVEDS143, offset: 0x008C */

    } CH[6];

} MSPWM_Type, *MSPWM_MemMapPtr;

/** Number of instances of the MSPWM module. */
#define MSPWM_INSTANCE_COUNT                                                  (1u)

/* MSPWM base address */
#define MSPWM_BASE                                                            (0x40038000)
#define MSPWM                                                                 ((MSPWM_Type *)(MSPWM_BASE))

/** Array initializer of MSPWM peripheral base addresses */
#define MSPWM_BASE_ADDRS                                                      { MSPWM_BASE }
/** Array initializer of MSPWM peripheral base pointers */
#define MSPWM_BASE_PTRS                                                       { MSPWM }

/* MSPWM Register Mask */
/* MSPWM_STS Register */
#define MSPWM_STS_TOF_MASK                                                    (0x2000U)
#define MSPWM_STS_TOF_SHIFT                                                   (13U)
#define MSPWM_STS_TOF(x)                                                      (((uint32_t)(((uint32_t)(x)) << MSPWM_STS_TOF_SHIFT)) & MSPWM_STS_TOF_MASK)
#define MSPWM_STS_RF_MASK                                                     (0x1000U)
#define MSPWM_STS_RF_SHIFT                                                    (12U)
#define MSPWM_STS_RF(x)                                                       (((uint32_t)(((uint32_t)(x)) << MSPWM_STS_RF_SHIFT)) & MSPWM_STS_RF_MASK)
#define MSPWM_STS_CH5F_MASK                                                   (0x20U)
#define MSPWM_STS_CH5F_SHIFT                                                  (5U)
#define MSPWM_STS_CH5F(x)                                                     (((uint32_t)(((uint32_t)(x)) << MSPWM_STS_CH5F_SHIFT)) & MSPWM_STS_CH5F_MASK)
#define MSPWM_STS_CH4F_MASK                                                   (0x10U)
#define MSPWM_STS_CH4F_SHIFT                                                  (4U)
#define MSPWM_STS_CH4F(x)                                                     (((uint32_t)(((uint32_t)(x)) << MSPWM_STS_CH4F_SHIFT)) & MSPWM_STS_CH4F_MASK)
#define MSPWM_STS_CH3F_MASK                                                   (0x8U)
#define MSPWM_STS_CH3F_SHIFT                                                  (3U)
#define MSPWM_STS_CH3F(x)                                                     (((uint32_t)(((uint32_t)(x)) << MSPWM_STS_CH3F_SHIFT)) & MSPWM_STS_CH3F_MASK)
#define MSPWM_STS_CH2F_MASK                                                   (0x4U)
#define MSPWM_STS_CH2F_SHIFT                                                  (2U)
#define MSPWM_STS_CH2F(x)                                                     (((uint32_t)(((uint32_t)(x)) << MSPWM_STS_CH2F_SHIFT)) & MSPWM_STS_CH2F_MASK)
#define MSPWM_STS_CH1F_MASK                                                   (0x2U)
#define MSPWM_STS_CH1F_SHIFT                                                  (1U)
#define MSPWM_STS_CH1F(x)                                                     (((uint32_t)(((uint32_t)(x)) << MSPWM_STS_CH1F_SHIFT)) & MSPWM_STS_CH1F_MASK)
#define MSPWM_STS_CH0F_MASK                                                   (0x1U)
#define MSPWM_STS_CH0F_SHIFT                                                  (0U)
#define MSPWM_STS_CH0F(x)                                                     (((uint32_t)(((uint32_t)(x)) << MSPWM_STS_CH0F_SHIFT)) & MSPWM_STS_CH0F_MASK)
/* MSPWM_CNT Register */
#define MSPWM_CNT_CNT_MASK                                                    (0xFFFFU)
#define MSPWM_CNT_CNT_SHIFT                                                   (0U)
#define MSPWM_CNT_CNT(x)                                                      (((uint32_t)(((uint32_t)(x)) << MSPWM_CNT_CNT_SHIFT)) & MSPWM_CNT_CNT_MASK)
/* MSPWM_CTRL Register */
#define MSPWM_CTRL_DBGDIS_MASK                                                (0x8000U)
#define MSPWM_CTRL_DBGDIS_SHIFT                                               (15U)
#define MSPWM_CTRL_DBGDIS(x)                                                  (((uint32_t)(((uint32_t)(x)) << MSPWM_CTRL_DBGDIS_SHIFT)) & MSPWM_CTRL_DBGDIS_MASK)
#define MSPWM_CTRL_CLKPRS_WIDTH                                               (7)
#define MSPWM_CTRL_CLKPRS_MASK                                                (0x7F00U)
#define MSPWM_CTRL_CLKPRS_SHIFT                                               (8U)
#define MSPWM_CTRL_CLKPRS(x)                                                  (((uint32_t)(((uint32_t)(x)) << MSPWM_CTRL_CLKPRS_SHIFT)) & MSPWM_CTRL_CLKPRS_MASK)
#define MSPWM_CTRL_EN_MASK                                                    (0x1U)
#define MSPWM_CTRL_EN_SHIFT                                                   (0U)
#define MSPWM_CTRL_EN(x)                                                      (((uint32_t)(((uint32_t)(x)) << MSPWM_CTRL_EN_SHIFT)) & MSPWM_CTRL_EN_MASK)
/* MSPWM_SYNC Register */
#define MSPWM_SYNC_LDFRQ_WIDTH                                                (3)
#define MSPWM_SYNC_LDFRQ_MASK                                                 (0x7000000U)
#define MSPWM_SYNC_LDFRQ_SHIFT                                                (24U)
#define MSPWM_SYNC_LDFRQ(x)                                                   (((uint32_t)(((uint32_t)(x)) << MSPWM_SYNC_LDFRQ_SHIFT)) & MSPWM_SYNC_LDFRQ_MASK)
#define MSPWM_SYNC_TRIG1EN_MASK                                               (0x40000U)
#define MSPWM_SYNC_TRIG1EN_SHIFT                                              (18U)
#define MSPWM_SYNC_TRIG1EN(x)                                                 (((uint32_t)(((uint32_t)(x)) << MSPWM_SYNC_TRIG1EN_SHIFT)) & MSPWM_SYNC_TRIG1EN_MASK)
#define MSPWM_SYNC_TRIG0EN_MASK                                               (0x20000U)
#define MSPWM_SYNC_TRIG0EN_SHIFT                                              (17U)
#define MSPWM_SYNC_TRIG0EN(x)                                                 (((uint32_t)(((uint32_t)(x)) << MSPWM_SYNC_TRIG0EN_SHIFT)) & MSPWM_SYNC_TRIG0EN_MASK)
#define MSPWM_SYNC_SWTRIG_MASK                                                (0x10000U)
#define MSPWM_SYNC_SWTRIG_SHIFT                                               (16U)
#define MSPWM_SYNC_SWTRIG(x)                                                  (((uint32_t)(((uint32_t)(x)) << MSPWM_SYNC_SWTRIG_SHIFT)) & MSPWM_SYNC_SWTRIG_MASK)
#define MSPWM_SYNC_MASKLD_WIDTH                                               (2)
#define MSPWM_SYNC_MASKLD_MASK                                                (0xC000U)
#define MSPWM_SYNC_MASKLD_SHIFT                                               (14U)
#define MSPWM_SYNC_MASKLD(x)                                                  (((uint32_t)(((uint32_t)(x)) << MSPWM_SYNC_MASKLD_SHIFT)) & MSPWM_SYNC_MASKLD_MASK)
#define MSPWM_SYNC_MASKTRIG_WIDTH                                             (2)
#define MSPWM_SYNC_MASKTRIG_MASK                                              (0x3000U)
#define MSPWM_SYNC_MASKTRIG_SHIFT                                             (12U)
#define MSPWM_SYNC_MASKTRIG(x)                                                (((uint32_t)(((uint32_t)(x)) << MSPWM_SYNC_MASKTRIG_SHIFT)) & MSPWM_SYNC_MASKTRIG_MASK)
#define MSPWM_SYNC_CNTLD_WIDTH                                                (2)
#define MSPWM_SYNC_CNTLD_MASK                                                 (0xC00U)
#define MSPWM_SYNC_CNTLD_SHIFT                                                (10U)
#define MSPWM_SYNC_CNTLD(x)                                                   (((uint32_t)(((uint32_t)(x)) << MSPWM_SYNC_CNTLD_SHIFT)) & MSPWM_SYNC_CNTLD_MASK)
#define MSPWM_SYNC_CNTTRIG_WIDTH                                              (2)
#define MSPWM_SYNC_CNTTRIG_MASK                                               (0x300U)
#define MSPWM_SYNC_CNTTRIG_SHIFT                                              (8U)
#define MSPWM_SYNC_CNTTRIG(x)                                                 (((uint32_t)(((uint32_t)(x)) << MSPWM_SYNC_CNTTRIG_SHIFT)) & MSPWM_SYNC_CNTTRIG_MASK)
#define MSPWM_SYNC_REGLD_WIDTH                                                (2)
#define MSPWM_SYNC_REGLD_MASK                                                 (0xC0U)
#define MSPWM_SYNC_REGLD_SHIFT                                                (6U)
#define MSPWM_SYNC_REGLD(x)                                                   (((uint32_t)(((uint32_t)(x)) << MSPWM_SYNC_REGLD_SHIFT)) & MSPWM_SYNC_REGLD_MASK)
#define MSPWM_SYNC_REGTRIG_WIDTH                                              (2)
#define MSPWM_SYNC_REGTRIG_MASK                                               (0x30U)
#define MSPWM_SYNC_REGTRIG_SHIFT                                              (4U)
#define MSPWM_SYNC_REGTRIG(x)                                                 (((uint32_t)(((uint32_t)(x)) << MSPWM_SYNC_REGTRIG_SHIFT)) & MSPWM_SYNC_REGTRIG_MASK)
#define MSPWM_SYNC_CLDOK_MASK                                                 (0x2U)
#define MSPWM_SYNC_CLDOK_SHIFT                                                (1U)
#define MSPWM_SYNC_CLDOK(x)                                                   (((uint32_t)(((uint32_t)(x)) << MSPWM_SYNC_CLDOK_SHIFT)) & MSPWM_SYNC_CLDOK_MASK)
#define MSPWM_SYNC_LDOK_MASK                                                  (0x1U)
#define MSPWM_SYNC_LDOK_SHIFT                                                 (0U)
#define MSPWM_SYNC_LDOK(x)                                                    (((uint32_t)(((uint32_t)(x)) << MSPWM_SYNC_LDOK_SHIFT)) & MSPWM_SYNC_LDOK_MASK)
/* MSPWM_OTRIG Register */
#define MSPWM_OTRIG_T1FRQ_WIDTH                                               (5)
#define MSPWM_OTRIG_T1FRQ_MASK                                                (0x1F000000U)
#define MSPWM_OTRIG_T1FRQ_SHIFT                                               (24U)
#define MSPWM_OTRIG_T1FRQ(x)                                                  (((uint32_t)(((uint32_t)(x)) << MSPWM_OTRIG_T1FRQ_SHIFT)) & MSPWM_OTRIG_T1FRQ_MASK)
#define MSPWM_OTRIG_INITT1EN_MASK                                             (0x20000U)
#define MSPWM_OTRIG_INITT1EN_SHIFT                                            (17U)
#define MSPWM_OTRIG_INITT1EN(x)                                               (((uint32_t)(((uint32_t)(x)) << MSPWM_OTRIG_INITT1EN_SHIFT)) & MSPWM_OTRIG_INITT1EN_MASK)
#define MSPWM_OTRIG_MODT1EN_MASK                                              (0x10000U)
#define MSPWM_OTRIG_MODT1EN_SHIFT                                             (16U)
#define MSPWM_OTRIG_MODT1EN(x)                                                (((uint32_t)(((uint32_t)(x)) << MSPWM_OTRIG_MODT1EN_SHIFT)) & MSPWM_OTRIG_MODT1EN_MASK)
#define MSPWM_OTRIG_T0FRQ_WIDTH                                               (5)
#define MSPWM_OTRIG_T0FRQ_MASK                                                (0x1F00U)
#define MSPWM_OTRIG_T0FRQ_SHIFT                                               (8U)
#define MSPWM_OTRIG_T0FRQ(x)                                                  (((uint32_t)(((uint32_t)(x)) << MSPWM_OTRIG_T0FRQ_SHIFT)) & MSPWM_OTRIG_T0FRQ_MASK)
#define MSPWM_OTRIG_INITT0EN_MASK                                             (0x2U)
#define MSPWM_OTRIG_INITT0EN_SHIFT                                            (1U)
#define MSPWM_OTRIG_INITT0EN(x)                                               (((uint32_t)(((uint32_t)(x)) << MSPWM_OTRIG_INITT0EN_SHIFT)) & MSPWM_OTRIG_INITT0EN_MASK)
#define MSPWM_OTRIG_MODT0EN_MASK                                              (0x1U)
#define MSPWM_OTRIG_MODT0EN_SHIFT                                             (0U)
#define MSPWM_OTRIG_MODT0EN(x)                                                (((uint32_t)(((uint32_t)(x)) << MSPWM_OTRIG_MODT0EN_SHIFT)) & MSPWM_OTRIG_MODT0EN_MASK)
/* MSPWM_OWIN Register */
#define MSPWM_OWIN_WINSEL_WIDTH                                               (4)
#define MSPWM_OWIN_WINSEL_MASK                                                (0xF00U)
#define MSPWM_OWIN_WINSEL_SHIFT                                               (8U)
#define MSPWM_OWIN_WINSEL(x)                                                  (((uint32_t)(((uint32_t)(x)) << MSPWM_OWIN_WINSEL_SHIFT)) & MSPWM_OWIN_WINSEL_MASK)
#define MSPWM_OWIN_WINEN_MASK                                                 (0x1U)
#define MSPWM_OWIN_WINEN_SHIFT                                                (0U)
#define MSPWM_OWIN_WINEN(x)                                                   (((uint32_t)(((uint32_t)(x)) << MSPWM_OWIN_WINEN_SHIFT)) & MSPWM_OWIN_WINEN_MASK)
/* MSPWM_INTE Register */
#define MSPWM_INTE_TOIE_MASK                                                  (0x2000U)
#define MSPWM_INTE_TOIE_SHIFT                                                 (13U)
#define MSPWM_INTE_TOIE(x)                                                    (((uint32_t)(((uint32_t)(x)) << MSPWM_INTE_TOIE_SHIFT)) & MSPWM_INTE_TOIE_MASK)
#define MSPWM_INTE_CH5IE_MASK                                                 (0x20U)
#define MSPWM_INTE_CH5IE_SHIFT                                                (5U)
#define MSPWM_INTE_CH5IE(x)                                                   (((uint32_t)(((uint32_t)(x)) << MSPWM_INTE_CH5IE_SHIFT)) & MSPWM_INTE_CH5IE_MASK)
#define MSPWM_INTE_CH4IE_MASK                                                 (0x10U)
#define MSPWM_INTE_CH4IE_SHIFT                                                (4U)
#define MSPWM_INTE_CH4IE(x)                                                   (((uint32_t)(((uint32_t)(x)) << MSPWM_INTE_CH4IE_SHIFT)) & MSPWM_INTE_CH4IE_MASK)
#define MSPWM_INTE_CH3IE_MASK                                                 (0x8U)
#define MSPWM_INTE_CH3IE_SHIFT                                                (3U)
#define MSPWM_INTE_CH3IE(x)                                                   (((uint32_t)(((uint32_t)(x)) << MSPWM_INTE_CH3IE_SHIFT)) & MSPWM_INTE_CH3IE_MASK)
#define MSPWM_INTE_CH2IE_MASK                                                 (0x4U)
#define MSPWM_INTE_CH2IE_SHIFT                                                (2U)
#define MSPWM_INTE_CH2IE(x)                                                   (((uint32_t)(((uint32_t)(x)) << MSPWM_INTE_CH2IE_SHIFT)) & MSPWM_INTE_CH2IE_MASK)
#define MSPWM_INTE_CH1IE_MASK                                                 (0x2U)
#define MSPWM_INTE_CH1IE_SHIFT                                                (1U)
#define MSPWM_INTE_CH1IE(x)                                                   (((uint32_t)(((uint32_t)(x)) << MSPWM_INTE_CH1IE_SHIFT)) & MSPWM_INTE_CH1IE_MASK)
#define MSPWM_INTE_CH0IE_MASK                                                 (0x1U)
#define MSPWM_INTE_CH0IE_SHIFT                                                (0U)
#define MSPWM_INTE_CH0IE(x)                                                   (((uint32_t)(((uint32_t)(x)) << MSPWM_INTE_CH0IE_SHIFT)) & MSPWM_INTE_CH0IE_MASK)
/* MSPWM_CHINIT Register */
#define MSPWM_CHINIT_CH5INITV_MASK                                            (0x200000U)
#define MSPWM_CHINIT_CH5INITV_SHIFT                                           (21U)
#define MSPWM_CHINIT_CH5INITV(x)                                              (((uint32_t)(((uint32_t)(x)) << MSPWM_CHINIT_CH5INITV_SHIFT)) & MSPWM_CHINIT_CH5INITV_MASK)
#define MSPWM_CHINIT_CH4INITV_MASK                                            (0x100000U)
#define MSPWM_CHINIT_CH4INITV_SHIFT                                           (20U)
#define MSPWM_CHINIT_CH4INITV(x)                                              (((uint32_t)(((uint32_t)(x)) << MSPWM_CHINIT_CH4INITV_SHIFT)) & MSPWM_CHINIT_CH4INITV_MASK)
#define MSPWM_CHINIT_CH3INITV_MASK                                            (0x80000U)
#define MSPWM_CHINIT_CH3INITV_SHIFT                                           (19U)
#define MSPWM_CHINIT_CH3INITV(x)                                              (((uint32_t)(((uint32_t)(x)) << MSPWM_CHINIT_CH3INITV_SHIFT)) & MSPWM_CHINIT_CH3INITV_MASK)
#define MSPWM_CHINIT_CH2INITV_MASK                                            (0x40000U)
#define MSPWM_CHINIT_CH2INITV_SHIFT                                           (18U)
#define MSPWM_CHINIT_CH2INITV(x)                                              (((uint32_t)(((uint32_t)(x)) << MSPWM_CHINIT_CH2INITV_SHIFT)) & MSPWM_CHINIT_CH2INITV_MASK)
#define MSPWM_CHINIT_CH1INITV_MASK                                            (0x20000U)
#define MSPWM_CHINIT_CH1INITV_SHIFT                                           (17U)
#define MSPWM_CHINIT_CH1INITV(x)                                              (((uint32_t)(((uint32_t)(x)) << MSPWM_CHINIT_CH1INITV_SHIFT)) & MSPWM_CHINIT_CH1INITV_MASK)
#define MSPWM_CHINIT_CH0INITV_MASK                                            (0x10000U)
#define MSPWM_CHINIT_CH0INITV_SHIFT                                           (16U)
#define MSPWM_CHINIT_CH0INITV(x)                                              (((uint32_t)(((uint32_t)(x)) << MSPWM_CHINIT_CH0INITV_SHIFT)) & MSPWM_CHINIT_CH0INITV_MASK)
#define MSPWM_CHINIT_CH5INIT_MASK                                             (0x20U)
#define MSPWM_CHINIT_CH5INIT_SHIFT                                            (5U)
#define MSPWM_CHINIT_CH5INIT(x)                                               (((uint32_t)(((uint32_t)(x)) << MSPWM_CHINIT_CH5INIT_SHIFT)) & MSPWM_CHINIT_CH5INIT_MASK)
#define MSPWM_CHINIT_CH4INIT_MASK                                             (0x10U)
#define MSPWM_CHINIT_CH4INIT_SHIFT                                            (4U)
#define MSPWM_CHINIT_CH4INIT(x)                                               (((uint32_t)(((uint32_t)(x)) << MSPWM_CHINIT_CH4INIT_SHIFT)) & MSPWM_CHINIT_CH4INIT_MASK)
#define MSPWM_CHINIT_CH3INIT_MASK                                             (0x8U)
#define MSPWM_CHINIT_CH3INIT_SHIFT                                            (3U)
#define MSPWM_CHINIT_CH3INIT(x)                                               (((uint32_t)(((uint32_t)(x)) << MSPWM_CHINIT_CH3INIT_SHIFT)) & MSPWM_CHINIT_CH3INIT_MASK)
#define MSPWM_CHINIT_CH2INIT_MASK                                             (0x4U)
#define MSPWM_CHINIT_CH2INIT_SHIFT                                            (2U)
#define MSPWM_CHINIT_CH2INIT(x)                                               (((uint32_t)(((uint32_t)(x)) << MSPWM_CHINIT_CH2INIT_SHIFT)) & MSPWM_CHINIT_CH2INIT_MASK)
#define MSPWM_CHINIT_CH1INIT_MASK                                             (0x2U)
#define MSPWM_CHINIT_CH1INIT_SHIFT                                            (1U)
#define MSPWM_CHINIT_CH1INIT(x)                                               (((uint32_t)(((uint32_t)(x)) << MSPWM_CHINIT_CH1INIT_SHIFT)) & MSPWM_CHINIT_CH1INIT_MASK)
#define MSPWM_CHINIT_CH0INIT_MASK                                             (0x1U)
#define MSPWM_CHINIT_CH0INIT_SHIFT                                            (0U)
#define MSPWM_CHINIT_CH0INIT(x)                                               (((uint32_t)(((uint32_t)(x)) << MSPWM_CHINIT_CH0INIT_SHIFT)) & MSPWM_CHINIT_CH0INIT_MASK)
/* MSPWM_CHSWV Register */
#define MSPWM_CHSWV_CH5SWV_MASK                                               (0x20U)
#define MSPWM_CHSWV_CH5SWV_SHIFT                                              (5U)
#define MSPWM_CHSWV_CH5SWV(x)                                                 (((uint32_t)(((uint32_t)(x)) << MSPWM_CHSWV_CH5SWV_SHIFT)) & MSPWM_CHSWV_CH5SWV_MASK)
#define MSPWM_CHSWV_CH4SWV_MASK                                               (0x10U)
#define MSPWM_CHSWV_CH4SWV_SHIFT                                              (4U)
#define MSPWM_CHSWV_CH4SWV(x)                                                 (((uint32_t)(((uint32_t)(x)) << MSPWM_CHSWV_CH4SWV_SHIFT)) & MSPWM_CHSWV_CH4SWV_MASK)
#define MSPWM_CHSWV_CH3SWV_MASK                                               (0x8U)
#define MSPWM_CHSWV_CH3SWV_SHIFT                                              (3U)
#define MSPWM_CHSWV_CH3SWV(x)                                                 (((uint32_t)(((uint32_t)(x)) << MSPWM_CHSWV_CH3SWV_SHIFT)) & MSPWM_CHSWV_CH3SWV_MASK)
#define MSPWM_CHSWV_CH2SWV_MASK                                               (0x4U)
#define MSPWM_CHSWV_CH2SWV_SHIFT                                              (2U)
#define MSPWM_CHSWV_CH2SWV(x)                                                 (((uint32_t)(((uint32_t)(x)) << MSPWM_CHSWV_CH2SWV_SHIFT)) & MSPWM_CHSWV_CH2SWV_MASK)
#define MSPWM_CHSWV_CH1SWV_MASK                                               (0x2U)
#define MSPWM_CHSWV_CH1SWV_SHIFT                                              (1U)
#define MSPWM_CHSWV_CH1SWV(x)                                                 (((uint32_t)(((uint32_t)(x)) << MSPWM_CHSWV_CH1SWV_SHIFT)) & MSPWM_CHSWV_CH1SWV_MASK)
#define MSPWM_CHSWV_CH0SWV_MASK                                               (0x1U)
#define MSPWM_CHSWV_CH0SWV_SHIFT                                              (0U)
#define MSPWM_CHSWV_CH0SWV(x)                                                 (((uint32_t)(((uint32_t)(x)) << MSPWM_CHSWV_CH0SWV_SHIFT)) & MSPWM_CHSWV_CH0SWV_MASK)
/* MSPWM_CHMASK Register */
#define MSPWM_CHMASK_CH5MV_WIDTH                                              (1)
#define MSPWM_CHMASK_CH5MV_MASK                                               (0x200000U)
#define MSPWM_CHMASK_CH5MV_SHIFT                                              (21U)
#define MSPWM_CHMASK_CH5MV(x)                                                 (((uint32_t)(((uint32_t)(x)) << MSPWM_CHMASK_CH5MV_SHIFT)) & MSPWM_CHMASK_CH5MV_MASK)
#define MSPWM_CHMASK_CH4MV_WIDTH                                              (1)
#define MSPWM_CHMASK_CH4MV_MASK                                               (0x100000U)
#define MSPWM_CHMASK_CH4MV_SHIFT                                              (20U)
#define MSPWM_CHMASK_CH4MV(x)                                                 (((uint32_t)(((uint32_t)(x)) << MSPWM_CHMASK_CH4MV_SHIFT)) & MSPWM_CHMASK_CH4MV_MASK)
#define MSPWM_CHMASK_CH3MV_WIDTH                                              (1)
#define MSPWM_CHMASK_CH3MV_MASK                                               (0x80000U)
#define MSPWM_CHMASK_CH3MV_SHIFT                                              (19U)
#define MSPWM_CHMASK_CH3MV(x)                                                 (((uint32_t)(((uint32_t)(x)) << MSPWM_CHMASK_CH3MV_SHIFT)) & MSPWM_CHMASK_CH3MV_MASK)
#define MSPWM_CHMASK_CH2MV_WIDTH                                              (1)
#define MSPWM_CHMASK_CH2MV_MASK                                               (0x40000U)
#define MSPWM_CHMASK_CH2MV_SHIFT                                              (18U)
#define MSPWM_CHMASK_CH2MV(x)                                                 (((uint32_t)(((uint32_t)(x)) << MSPWM_CHMASK_CH2MV_SHIFT)) & MSPWM_CHMASK_CH2MV_MASK)
#define MSPWM_CHMASK_CH1MV_WIDTH                                              (1)
#define MSPWM_CHMASK_CH1MV_MASK                                               (0x20000U)
#define MSPWM_CHMASK_CH1MV_SHIFT                                              (17U)
#define MSPWM_CHMASK_CH1MV(x)                                                 (((uint32_t)(((uint32_t)(x)) << MSPWM_CHMASK_CH1MV_SHIFT)) & MSPWM_CHMASK_CH1MV_MASK)
#define MSPWM_CHMASK_CH0MV_WIDTH                                              (1)
#define MSPWM_CHMASK_CH0MV_MASK                                               (0x10000U)
#define MSPWM_CHMASK_CH0MV_SHIFT                                              (16U)
#define MSPWM_CHMASK_CH0MV(x)                                                 (((uint32_t)(((uint32_t)(x)) << MSPWM_CHMASK_CH0MV_SHIFT)) & MSPWM_CHMASK_CH0MV_MASK)
#define MSPWM_CHMASK_CH5M_MASK                                                (0x20U)
#define MSPWM_CHMASK_CH5M_SHIFT                                               (5U)
#define MSPWM_CHMASK_CH5M(x)                                                  (((uint32_t)(((uint32_t)(x)) << MSPWM_CHMASK_CH5M_SHIFT)) & MSPWM_CHMASK_CH5M_MASK)
#define MSPWM_CHMASK_CH4M_MASK                                                (0x10U)
#define MSPWM_CHMASK_CH4M_SHIFT                                               (4U)
#define MSPWM_CHMASK_CH4M(x)                                                  (((uint32_t)(((uint32_t)(x)) << MSPWM_CHMASK_CH4M_SHIFT)) & MSPWM_CHMASK_CH4M_MASK)
#define MSPWM_CHMASK_CH3M_MASK                                                (0x8U)
#define MSPWM_CHMASK_CH3M_SHIFT                                               (3U)
#define MSPWM_CHMASK_CH3M(x)                                                  (((uint32_t)(((uint32_t)(x)) << MSPWM_CHMASK_CH3M_SHIFT)) & MSPWM_CHMASK_CH3M_MASK)
#define MSPWM_CHMASK_CH2M_MASK                                                (0x4U)
#define MSPWM_CHMASK_CH2M_SHIFT                                               (2U)
#define MSPWM_CHMASK_CH2M(x)                                                  (((uint32_t)(((uint32_t)(x)) << MSPWM_CHMASK_CH2M_SHIFT)) & MSPWM_CHMASK_CH2M_MASK)
#define MSPWM_CHMASK_CH1M_MASK                                                (0x2U)
#define MSPWM_CHMASK_CH1M_SHIFT                                               (1U)
#define MSPWM_CHMASK_CH1M(x)                                                  (((uint32_t)(((uint32_t)(x)) << MSPWM_CHMASK_CH1M_SHIFT)) & MSPWM_CHMASK_CH1M_MASK)
#define MSPWM_CHMASK_CH0M_MASK                                                (0x1U)
#define MSPWM_CHMASK_CH0M_SHIFT                                               (0U)
#define MSPWM_CHMASK_CH0M(x)                                                  (((uint32_t)(((uint32_t)(x)) << MSPWM_CHMASK_CH0M_SHIFT)) & MSPWM_CHMASK_CH0M_MASK)
/* MSPWM_MOD Register */
#define MSPWM_MOD_MOD_MASK                                                    (0xFFFFU)
#define MSPWM_MOD_MOD_SHIFT                                                   (0U)
#define MSPWM_MOD_MOD(x)                                                      (((uint32_t)(((uint32_t)(x)) << MSPWM_MOD_MOD_SHIFT)) & MSPWM_MOD_MOD_MASK)
/* MSPWM_CH_CTRL Register */
#define MSPWM_CH_CTRL_LDEN_MASK                                               (0x10000000U)
#define MSPWM_CH_CTRL_LDEN_SHIFT                                              (28U)
#define MSPWM_CH_CTRL_LDEN(x)                                                 (((uint32_t)(((uint32_t)(x)) << MSPWM_CH_CTRL_LDEN_SHIFT)) & MSPWM_CH_CTRL_LDEN_MASK)
#define MSPWM_CH_CTRL_VAL1T1EN_MASK                                           (0x80000U)
#define MSPWM_CH_CTRL_VAL1T1EN_SHIFT                                          (19U)
#define MSPWM_CH_CTRL_VAL1T1EN(x)                                             (((uint32_t)(((uint32_t)(x)) << MSPWM_CH_CTRL_VAL1T1EN_SHIFT)) & MSPWM_CH_CTRL_VAL1T1EN_MASK)
#define MSPWM_CH_CTRL_VAL0T1EN_MASK                                           (0x40000U)
#define MSPWM_CH_CTRL_VAL0T1EN_SHIFT                                          (18U)
#define MSPWM_CH_CTRL_VAL0T1EN(x)                                             (((uint32_t)(((uint32_t)(x)) << MSPWM_CH_CTRL_VAL0T1EN_SHIFT)) & MSPWM_CH_CTRL_VAL0T1EN_MASK)
#define MSPWM_CH_CTRL_VAL1T0EN_MASK                                           (0x20000U)
#define MSPWM_CH_CTRL_VAL1T0EN_SHIFT                                          (17U)
#define MSPWM_CH_CTRL_VAL1T0EN(x)                                             (((uint32_t)(((uint32_t)(x)) << MSPWM_CH_CTRL_VAL1T0EN_SHIFT)) & MSPWM_CH_CTRL_VAL1T0EN_MASK)
#define MSPWM_CH_CTRL_VAL0T0EN_MASK                                           (0x10000U)
#define MSPWM_CH_CTRL_VAL0T0EN_SHIFT                                          (16U)
#define MSPWM_CH_CTRL_VAL0T0EN(x)                                             (((uint32_t)(((uint32_t)(x)) << MSPWM_CH_CTRL_VAL0T0EN_SHIFT)) & MSPWM_CH_CTRL_VAL0T0EN_MASK)
#define MSPWM_CH_CTRL_PWMSRC_MASK                                             (0x300U)
#define MSPWM_CH_CTRL_PWMSRC_SHIFT                                            (8U)
#define MSPWM_CH_CTRL_PWMSRC(x)                                               (((uint32_t)(((uint32_t)(x)) << MSPWM_CH_CTRL_PWMSRC_SHIFT)) & MSPWM_CH_CTRL_PWMSRC_MASK)
#define MSPWM_CH_CTRL_CHEN_MASK                                               (0x1U)
#define MSPWM_CH_CTRL_CHEN_SHIFT                                              (0U)
#define MSPWM_CH_CTRL_CHEN(x)                                                 (((uint32_t)(((uint32_t)(x)) << MSPWM_CH_CTRL_CHEN_SHIFT)) & MSPWM_CH_CTRL_CHEN_MASK)
/* MSPWM_CH_VAL0 Register */
#define MSPWM_CH_VAL0_VAL0_MASK                                               (0xFFFFU)
#define MSPWM_CH_VAL0_VAL0_SHIFT                                              (0U)
#define MSPWM_CH_VAL0_VAL0(x)                                                 (((uint32_t)(((uint32_t)(x)) << MSPWM_CH_VAL0_VAL0_SHIFT)) & MSPWM_CH_VAL0_VAL0_MASK)
/* MSPWM_CH_VAL1 Register */
#define MSPWM_CH_VAL1_VAL1_MASK                                               (0xFFFFU)
#define MSPWM_CH_VAL1_VAL1_SHIFT                                              (0U)
#define MSPWM_CH_VAL1_VAL1(x)                                                 (((uint32_t)(((uint32_t)(x)) << MSPWM_CH_VAL1_VAL1_SHIFT)) & MSPWM_CH_VAL1_VAL1_MASK)



/*!
 * @}
 */ /* end of group MSPWM_Register_Masks */


/*!
 * @}
 */ /* end of group MSPWM_Peripheral_Access_Layer */


/* -------------------------------------------------------------
   -- pTMR Peripheral Access Layer
   ------------------------------------------------------------- */

/*!
 * @addtogroup pTMR_Peripheral_Access_Layer pTMR Peripheral Access Layer
 * @{
 */


/** pTMR - Size of Registers Arrays */
#define pTMR_CH_COUNT                                                         (4)

/* pTMR Register Layout Typedef */
typedef struct {
    __IO uint32_t MCR;                                                        /**< Module Control Register, offset: 0x0000 */
    __IO uint8_t  RESERVED1[252];                                             /**< RESERVED1, offset: 0x0004 */
    struct {
        __IO uint32_t TSV;                                                    /**< Timer Start Value Register, offset: 0x0100 */
        __IO uint32_t TCV;                                                    /**< Timer Current Value Register, offset: 0x0104 */
        __IO uint32_t TCR;                                                    /**< Timer Control Register, offset: 0x0108 */
        __IO uint32_t TFR;                                                    /**< Timer Flag Register, offset: 0x010C */

    } CH[4];

} pTMR_Type, *pTMR_MemMapPtr;

/** Number of instances of the pTMR module. */
#define pTMR_INSTANCE_COUNT                                                   (1u)

/* pTMR base address */
#define pTMR_BASE                                                             (0x40037000)
#define pTMR                                                                  ((pTMR_Type *)(pTMR_BASE))

/** Array initializer of pTMR peripheral base addresses */
#define pTMR_BASE_ADDRS                                                       { pTMR_BASE }
/** Array initializer of pTMR peripheral base pointers */
#define pTMR_BASE_PTRS                                                        { pTMR }
/** Number of interrupt vector arrays for the pTMR module. */
#define pTMR_IRQS_ARR_COUNT                                                   (1u)
/** Number of interrupt channels for the pTMR. */
#define pTMR_IRQS_CH_COUNT                                                    (4u)
/** Interrupt vectors for the pTMR peripheral type */
#define pTMR_IRQS                                                             { {pTMR_Ch0_IRQn, pTMR_Ch1_IRQn, pTMR_Ch2_IRQn, pTMR_Ch3_IRQn} }

/* pTMR Register Mask */
/* pTMR_MCR Register */
#define pTMR_MCR_CLK_SEL_MASK                                                 (0x4U)
#define pTMR_MCR_CLK_SEL_SHIFT                                                (2U)
#define pTMR_MCR_CLK_SEL(x)                                                   (((uint32_t)(((uint32_t)(x)) << pTMR_MCR_CLK_SEL_SHIFT)) & pTMR_MCR_CLK_SEL_MASK)
#define pTMR_MCR_FRZ_MASK                                                     (0x2U)
#define pTMR_MCR_FRZ_SHIFT                                                    (1U)
#define pTMR_MCR_FRZ(x)                                                       (((uint32_t)(((uint32_t)(x)) << pTMR_MCR_FRZ_SHIFT)) & pTMR_MCR_FRZ_MASK)
#define pTMR_MCR_EN_MASK                                                      (0x1U)
#define pTMR_MCR_EN_SHIFT                                                     (0U)
#define pTMR_MCR_EN(x)                                                        (((uint32_t)(((uint32_t)(x)) << pTMR_MCR_EN_SHIFT)) & pTMR_MCR_EN_MASK)
/* pTMR_CH_TSV Register */
#define pTMR_CH_TSV_STVAL_MASK                                                (0xFFFFU)
#define pTMR_CH_TSV_STVAL_SHIFT                                               (0U)
#define pTMR_CH_TSV_STVAL(x)                                                  (((uint32_t)(((uint32_t)(x)) << pTMR_CH_TSV_STVAL_SHIFT)) & pTMR_CH_TSV_STVAL_MASK)
/* pTMR_CH_TCV Register */
#define pTMR_CH_TCV_CVAL_MASK                                                 (0xFFFFU)
#define pTMR_CH_TCV_CVAL_SHIFT                                                (0U)
#define pTMR_CH_TCV_CVAL(x)                                                   (((uint32_t)(((uint32_t)(x)) << pTMR_CH_TCV_CVAL_SHIFT)) & pTMR_CH_TCV_CVAL_MASK)
/* pTMR_CH_TCR Register */
#define pTMR_CH_TCR_CHAIN_MASK                                                (0x4U)
#define pTMR_CH_TCR_CHAIN_SHIFT                                               (2U)
#define pTMR_CH_TCR_CHAIN(x)                                                  (((uint32_t)(((uint32_t)(x)) << pTMR_CH_TCR_CHAIN_SHIFT)) & pTMR_CH_TCR_CHAIN_MASK)
#define pTMR_CH_TCR_TIE_MASK                                                  (0x2U)
#define pTMR_CH_TCR_TIE_SHIFT                                                 (1U)
#define pTMR_CH_TCR_TIE(x)                                                    (((uint32_t)(((uint32_t)(x)) << pTMR_CH_TCR_TIE_SHIFT)) & pTMR_CH_TCR_TIE_MASK)
#define pTMR_CH_TCR_TEN_MASK                                                  (0x1U)
#define pTMR_CH_TCR_TEN_SHIFT                                                 (0U)
#define pTMR_CH_TCR_TEN(x)                                                    (((uint32_t)(((uint32_t)(x)) << pTMR_CH_TCR_TEN_SHIFT)) & pTMR_CH_TCR_TEN_MASK)
/* pTMR_CH_TFR Register */
#define pTMR_CH_TFR_TIF_MASK                                                  (0x1U)
#define pTMR_CH_TFR_TIF_SHIFT                                                 (0U)
#define pTMR_CH_TFR_TIF(x)                                                    (((uint32_t)(((uint32_t)(x)) << pTMR_CH_TFR_TIF_SHIFT)) & pTMR_CH_TFR_TIF_MASK)



/*!
 * @}
 */ /* end of group pTMR_Register_Masks */


/*!
 * @}
 */ /* end of group pTMR_Peripheral_Access_Layer */


/* -------------------------------------------------------------
   -- ADC Peripheral Access Layer
   ------------------------------------------------------------- */

/*!
 * @addtogroup ADC_Peripheral_Access_Layer ADC Peripheral Access Layer
 * @{
 */


/** ADC - Size of Registers Arrays */
#define ADC_CHSEL_COUNT                                                       (16)

/* ADC Register Layout Typedef */
typedef struct {
    __IO uint32_t STS;                                                        /**< Status register, offset: 0x0000 */
    __IO uint32_t INTE;                                                       /**< Interrupt Enable Register, offset: 0x0004 */
    __IO uint32_t CTRL;                                                       /**< Control Register, offset: 0x0008 */
    __IO uint32_t CFG0;                                                       /**< Configuration 0 Register, offset: 0x000C */
    __IO uint32_t CFG1;                                                       /**< Configuration 1 Register, offset: 0x0010 */
    __IO uint32_t SMP;                                                        /**< Sampling Register, offset: 0x0014 */
    __IO uint32_t WDCTRL;                                                     /**< Watchdog Control Register, offset: 0x0018 */
    __IO uint32_t WDTH;                                                       /**< Watchdog Threshold Register, offset: 0x001C */
    __IO uint8_t  RESERVED8[12];                                              /**< RESERVED8, offset: 0x0020 */
    __IO uint8_t  CHSEL[16];                                                  /**< Channel Configure 0 Register, offset: 0x002C */
    __IO uint8_t  RESERVED10[16];                                             /**< RESERVED10, offset: 0x003C */
    __IO uint32_t FIFO;                                                       /**< FIFO Access Register, offset: 0x004C */
    __IO uint32_t RESERVED12;                                                 /**< RESERVED12, offset: 0x0050 */
    __IO uint32_t TRIM;                                                       /**< Trim Register, offset: 0x0054 */
    __IO uint32_t TSC;                                                        /**< Temperature Sensor Coefficient Register, offset: 0x0058 */

} ADC_Type, *ADC_MemMapPtr;

/** Number of instances of the ADC module. */
#define ADC_INSTANCE_COUNT                                                    (1u)

/* ADC0  base address */
#define ADC0_BASE                                                             (0x4003B000)
#define ADC0                                                                  ((ADC_Type *)(ADC0_BASE))

/** Array initializer of ADC peripheral base addresses */
#define ADC_BASE_ADDRS                                                        { ADC0_BASE }
/** Array initializer of ADC peripheral base pointers */
#define ADC_BASE_PTRS                                                         { ADC0 }
/** Number of interrupt vector arrays for the ADC module. */
#define ADC_IRQS_ARR_COUNT                                                    (u)
/** Number of interrupt channels for the ADC. */
#define ADC_IRQS_CH_COUNT                                                     (1u)
/** Interrupt vectors for the ADC peripheral type */
#define ADC_IRQS                                                              { ADC0_IRQn }

/* ADC Register Mask */
/* ADC_STS Register */
#define ADC_STS_FIFOCNT_WIDTH                                                 (5)
#define ADC_STS_FIFOCNT_MASK                                                  (0xF80000U)
#define ADC_STS_FIFOCNT_SHIFT                                                 (19U)
#define ADC_STS_FIFOCNT(x)                                                    (((uint32_t)(((uint32_t)(x)) << ADC_STS_FIFOCNT_SHIFT)) & ADC_STS_FIFOCNT_MASK)
#define ADC_STS_EMPTY_MASK                                                    (0x20000U)
#define ADC_STS_EMPTY_SHIFT                                                   (17U)
#define ADC_STS_EMPTY(x)                                                      (((uint32_t)(((uint32_t)(x)) << ADC_STS_EMPTY_SHIFT)) & ADC_STS_EMPTY_MASK)
#define ADC_STS_FULL_MASK                                                     (0x10000U)
#define ADC_STS_FULL_SHIFT                                                    (16U)
#define ADC_STS_FULL(x)                                                       (((uint32_t)(((uint32_t)(x)) << ADC_STS_FULL_SHIFT)) & ADC_STS_FULL_MASK)
#define ADC_STS_SEQERR_MASK                                                   (0x800U)
#define ADC_STS_SEQERR_SHIFT                                                  (11U)
#define ADC_STS_SEQERR(x)                                                     (((uint32_t)(((uint32_t)(x)) << ADC_STS_SEQERR_SHIFT)) & ADC_STS_SEQERR_MASK)
#define ADC_STS_SINGERR_MASK                                                  (0x400U)
#define ADC_STS_SINGERR_SHIFT                                                 (10U)
#define ADC_STS_SINGERR(x)                                                    (((uint32_t)(((uint32_t)(x)) << ADC_STS_SINGERR_SHIFT)) & ADC_STS_SINGERR_MASK)
#define ADC_STS_WD_MASK                                                       (0x200U)
#define ADC_STS_WD_SHIFT                                                      (9U)
#define ADC_STS_WD(x)                                                         (((uint32_t)(((uint32_t)(x)) << ADC_STS_WD_SHIFT)) & ADC_STS_WD_MASK)
#define ADC_STS_OVR_MASK                                                      (0x100U)
#define ADC_STS_OVR_SHIFT                                                     (8U)
#define ADC_STS_OVR(x)                                                        (((uint32_t)(((uint32_t)(x)) << ADC_STS_OVR_SHIFT)) & ADC_STS_OVR_MASK)
#define ADC_STS_EOSEQ3_MASK                                                   (0x40U)
#define ADC_STS_EOSEQ3_SHIFT                                                  (6U)
#define ADC_STS_EOSEQ3(x)                                                     (((uint32_t)(((uint32_t)(x)) << ADC_STS_EOSEQ3_SHIFT)) & ADC_STS_EOSEQ3_MASK)
#define ADC_STS_EOSEQ2_MASK                                                   (0x20U)
#define ADC_STS_EOSEQ2_SHIFT                                                  (5U)
#define ADC_STS_EOSEQ2(x)                                                     (((uint32_t)(((uint32_t)(x)) << ADC_STS_EOSEQ2_SHIFT)) & ADC_STS_EOSEQ2_MASK)
#define ADC_STS_EOSEQ1_MASK                                                   (0x10U)
#define ADC_STS_EOSEQ1_SHIFT                                                  (4U)
#define ADC_STS_EOSEQ1(x)                                                     (((uint32_t)(((uint32_t)(x)) << ADC_STS_EOSEQ1_SHIFT)) & ADC_STS_EOSEQ1_MASK)
#define ADC_STS_EOSEQ0_MASK                                                   (0x8U)
#define ADC_STS_EOSEQ0_SHIFT                                                  (3U)
#define ADC_STS_EOSEQ0(x)                                                     (((uint32_t)(((uint32_t)(x)) << ADC_STS_EOSEQ0_SHIFT)) & ADC_STS_EOSEQ0_MASK)
#define ADC_STS_EOC_MASK                                                      (0x4U)
#define ADC_STS_EOC_SHIFT                                                     (2U)
#define ADC_STS_EOC(x)                                                        (((uint32_t)(((uint32_t)(x)) << ADC_STS_EOC_SHIFT)) & ADC_STS_EOC_MASK)
#define ADC_STS_EOSMP_MASK                                                    (0x2U)
#define ADC_STS_EOSMP_SHIFT                                                   (1U)
#define ADC_STS_EOSMP(x)                                                      (((uint32_t)(((uint32_t)(x)) << ADC_STS_EOSMP_SHIFT)) & ADC_STS_EOSMP_MASK)
#define ADC_STS_ADRDY_MASK                                                    (0x1U)
#define ADC_STS_ADRDY_SHIFT                                                   (0U)
#define ADC_STS_ADRDY(x)                                                      (((uint32_t)(((uint32_t)(x)) << ADC_STS_ADRDY_SHIFT)) & ADC_STS_ADRDY_MASK)
/* ADC_INTE Register */
#define ADC_INTE_SEQERRIE_MASK                                                (0x800U)
#define ADC_INTE_SEQERRIE_SHIFT                                               (11U)
#define ADC_INTE_SEQERRIE(x)                                                  (((uint32_t)(((uint32_t)(x)) << ADC_INTE_SEQERRIE_SHIFT)) & ADC_INTE_SEQERRIE_MASK)
#define ADC_INTE_SINGERRIE_MASK                                               (0x400U)
#define ADC_INTE_SINGERRIE_SHIFT                                              (10U)
#define ADC_INTE_SINGERRIE(x)                                                 (((uint32_t)(((uint32_t)(x)) << ADC_INTE_SINGERRIE_SHIFT)) & ADC_INTE_SINGERRIE_MASK)
#define ADC_INTE_WDIE_MASK                                                    (0x200U)
#define ADC_INTE_WDIE_SHIFT                                                   (9U)
#define ADC_INTE_WDIE(x)                                                      (((uint32_t)(((uint32_t)(x)) << ADC_INTE_WDIE_SHIFT)) & ADC_INTE_WDIE_MASK)
#define ADC_INTE_OVRIE_MASK                                                   (0x100U)
#define ADC_INTE_OVRIE_SHIFT                                                  (8U)
#define ADC_INTE_OVRIE(x)                                                     (((uint32_t)(((uint32_t)(x)) << ADC_INTE_OVRIE_SHIFT)) & ADC_INTE_OVRIE_MASK)
#define ADC_INTE_EOSEQ3IE_MASK                                                (0x40U)
#define ADC_INTE_EOSEQ3IE_SHIFT                                               (6U)
#define ADC_INTE_EOSEQ3IE(x)                                                  (((uint32_t)(((uint32_t)(x)) << ADC_INTE_EOSEQ3IE_SHIFT)) & ADC_INTE_EOSEQ3IE_MASK)
#define ADC_INTE_EOSEQ2IE_MASK                                                (0x20U)
#define ADC_INTE_EOSEQ2IE_SHIFT                                               (5U)
#define ADC_INTE_EOSEQ2IE(x)                                                  (((uint32_t)(((uint32_t)(x)) << ADC_INTE_EOSEQ2IE_SHIFT)) & ADC_INTE_EOSEQ2IE_MASK)
#define ADC_INTE_EOSEQ1IE_MASK                                                (0x10U)
#define ADC_INTE_EOSEQ1IE_SHIFT                                               (4U)
#define ADC_INTE_EOSEQ1IE(x)                                                  (((uint32_t)(((uint32_t)(x)) << ADC_INTE_EOSEQ1IE_SHIFT)) & ADC_INTE_EOSEQ1IE_MASK)
#define ADC_INTE_EOSEQ0IE_MASK                                                (0x8U)
#define ADC_INTE_EOSEQ0IE_SHIFT                                               (3U)
#define ADC_INTE_EOSEQ0IE(x)                                                  (((uint32_t)(((uint32_t)(x)) << ADC_INTE_EOSEQ0IE_SHIFT)) & ADC_INTE_EOSEQ0IE_MASK)
#define ADC_INTE_EOCIE_MASK                                                   (0x4U)
#define ADC_INTE_EOCIE_SHIFT                                                  (2U)
#define ADC_INTE_EOCIE(x)                                                     (((uint32_t)(((uint32_t)(x)) << ADC_INTE_EOCIE_SHIFT)) & ADC_INTE_EOCIE_MASK)
#define ADC_INTE_EOSMPIE_MASK                                                 (0x2U)
#define ADC_INTE_EOSMPIE_SHIFT                                                (1U)
#define ADC_INTE_EOSMPIE(x)                                                   (((uint32_t)(((uint32_t)(x)) << ADC_INTE_EOSMPIE_SHIFT)) & ADC_INTE_EOSMPIE_MASK)
#define ADC_INTE_ADRDYIE_MASK                                                 (0x1U)
#define ADC_INTE_ADRDYIE_SHIFT                                                (0U)
#define ADC_INTE_ADRDYIE(x)                                                   (((uint32_t)(((uint32_t)(x)) << ADC_INTE_ADRDYIE_SHIFT)) & ADC_INTE_ADRDYIE_MASK)
/* ADC_CTRL Register */
#define ADC_CTRL_FIFORST_MASK                                                 (0x10U)
#define ADC_CTRL_FIFORST_SHIFT                                                (4U)
#define ADC_CTRL_FIFORST(x)                                                   (((uint32_t)(((uint32_t)(x)) << ADC_CTRL_FIFORST_SHIFT)) & ADC_CTRL_FIFORST_MASK)
#define ADC_CTRL_ADSTOP_MASK                                                  (0x8U)
#define ADC_CTRL_ADSTOP_SHIFT                                                 (3U)
#define ADC_CTRL_ADSTOP(x)                                                    (((uint32_t)(((uint32_t)(x)) << ADC_CTRL_ADSTOP_SHIFT)) & ADC_CTRL_ADSTOP_MASK)
#define ADC_CTRL_ADSTART_MASK                                                 (0x4U)
#define ADC_CTRL_ADSTART_SHIFT                                                (2U)
#define ADC_CTRL_ADSTART(x)                                                   (((uint32_t)(((uint32_t)(x)) << ADC_CTRL_ADSTART_SHIFT)) & ADC_CTRL_ADSTART_MASK)
#define ADC_CTRL_ADDIS_MASK                                                   (0x2U)
#define ADC_CTRL_ADDIS_SHIFT                                                  (1U)
#define ADC_CTRL_ADDIS(x)                                                     (((uint32_t)(((uint32_t)(x)) << ADC_CTRL_ADDIS_SHIFT)) & ADC_CTRL_ADDIS_MASK)
#define ADC_CTRL_ADEN_MASK                                                    (0x1U)
#define ADC_CTRL_ADEN_SHIFT                                                   (0U)
#define ADC_CTRL_ADEN(x)                                                      (((uint32_t)(((uint32_t)(x)) << ADC_CTRL_ADEN_SHIFT)) & ADC_CTRL_ADEN_MASK)
/* ADC_CFG0 Register */
#define ADC_CFG0_SEQ3LEN_WIDTH                                                (2)
#define ADC_CFG0_SEQ3LEN_MASK                                                 (0xC0000000U)
#define ADC_CFG0_SEQ3LEN_SHIFT                                                (30U)
#define ADC_CFG0_SEQ3LEN(x)                                                   (((uint32_t)(((uint32_t)(x)) << ADC_CFG0_SEQ3LEN_SHIFT)) & ADC_CFG0_SEQ3LEN_MASK)
#define ADC_CFG0_SEQ2LEN_WIDTH                                                (2)
#define ADC_CFG0_SEQ2LEN_MASK                                                 (0x30000000U)
#define ADC_CFG0_SEQ2LEN_SHIFT                                                (28U)
#define ADC_CFG0_SEQ2LEN(x)                                                   (((uint32_t)(((uint32_t)(x)) << ADC_CFG0_SEQ2LEN_SHIFT)) & ADC_CFG0_SEQ2LEN_MASK)
#define ADC_CFG0_SEQ1LEN_WIDTH                                                (2)
#define ADC_CFG0_SEQ1LEN_MASK                                                 (0xC000000U)
#define ADC_CFG0_SEQ1LEN_SHIFT                                                (26U)
#define ADC_CFG0_SEQ1LEN(x)                                                   (((uint32_t)(((uint32_t)(x)) << ADC_CFG0_SEQ1LEN_SHIFT)) & ADC_CFG0_SEQ1LEN_MASK)
#define ADC_CFG0_SEQ0LEN_WIDTH                                                (2)
#define ADC_CFG0_SEQ0LEN_MASK                                                 (0x3000000U)
#define ADC_CFG0_SEQ0LEN_SHIFT                                                (24U)
#define ADC_CFG0_SEQ0LEN(x)                                                   (((uint32_t)(((uint32_t)(x)) << ADC_CFG0_SEQ0LEN_SHIFT)) & ADC_CFG0_SEQ0LEN_MASK)
#define ADC_CFG0_TRIG3EN_WIDTH                                                (1)
#define ADC_CFG0_TRIG3EN_MASK                                                 (0x80000U)
#define ADC_CFG0_TRIG3EN_SHIFT                                                (19U)
#define ADC_CFG0_TRIG3EN(x)                                                   (((uint32_t)(((uint32_t)(x)) << ADC_CFG0_TRIG3EN_SHIFT)) & ADC_CFG0_TRIG3EN_MASK)
#define ADC_CFG0_TRIG2EN_WIDTH                                                (1)
#define ADC_CFG0_TRIG2EN_MASK                                                 (0x40000U)
#define ADC_CFG0_TRIG2EN_SHIFT                                                (18U)
#define ADC_CFG0_TRIG2EN(x)                                                   (((uint32_t)(((uint32_t)(x)) << ADC_CFG0_TRIG2EN_SHIFT)) & ADC_CFG0_TRIG2EN_MASK)
#define ADC_CFG0_TRIG1EN_WIDTH                                                (1)
#define ADC_CFG0_TRIG1EN_MASK                                                 (0x20000U)
#define ADC_CFG0_TRIG1EN_SHIFT                                                (17U)
#define ADC_CFG0_TRIG1EN(x)                                                   (((uint32_t)(((uint32_t)(x)) << ADC_CFG0_TRIG1EN_SHIFT)) & ADC_CFG0_TRIG1EN_MASK)
#define ADC_CFG0_TRIG0EN_WIDTH                                                (1)
#define ADC_CFG0_TRIG0EN_MASK                                                 (0x10000U)
#define ADC_CFG0_TRIG0EN_SHIFT                                                (16U)
#define ADC_CFG0_TRIG0EN(x)                                                   (((uint32_t)(((uint32_t)(x)) << ADC_CFG0_TRIG0EN_SHIFT)) & ADC_CFG0_TRIG0EN_MASK)
#define ADC_CFG0_SEQCFG_MASK                                                  (0xF000U)
#define ADC_CFG0_SEQCFG_SHIFT                                                 (12U)
#define ADC_CFG0_SEQCFG(x)                                                    (((uint32_t)(((uint32_t)(x)) << ADC_CFG0_SEQCFG_SHIFT)) & ADC_CFG0_SEQCFG_MASK)
#define ADC_CFG0_SEQ3MD_WIDTH                                                 (1)
#define ADC_CFG0_SEQ3MD_MASK                                                  (0x800U)
#define ADC_CFG0_SEQ3MD_SHIFT                                                 (11U)
#define ADC_CFG0_SEQ3MD(x)                                                    (((uint32_t)(((uint32_t)(x)) << ADC_CFG0_SEQ3MD_SHIFT)) & ADC_CFG0_SEQ3MD_MASK)
#define ADC_CFG0_SEQ2MD_WIDTH                                                 (1)
#define ADC_CFG0_SEQ2MD_MASK                                                  (0x400U)
#define ADC_CFG0_SEQ2MD_SHIFT                                                 (10U)
#define ADC_CFG0_SEQ2MD(x)                                                    (((uint32_t)(((uint32_t)(x)) << ADC_CFG0_SEQ2MD_SHIFT)) & ADC_CFG0_SEQ2MD_MASK)
#define ADC_CFG0_SEQ1MD_WIDTH                                                 (1)
#define ADC_CFG0_SEQ1MD_MASK                                                  (0x200U)
#define ADC_CFG0_SEQ1MD_SHIFT                                                 (9U)
#define ADC_CFG0_SEQ1MD(x)                                                    (((uint32_t)(((uint32_t)(x)) << ADC_CFG0_SEQ1MD_SHIFT)) & ADC_CFG0_SEQ1MD_MASK)
#define ADC_CFG0_SEQ0MD_WIDTH                                                 (1)
#define ADC_CFG0_SEQ0MD_MASK                                                  (0x100U)
#define ADC_CFG0_SEQ0MD_SHIFT                                                 (8U)
#define ADC_CFG0_SEQ0MD(x)                                                    (((uint32_t)(((uint32_t)(x)) << ADC_CFG0_SEQ0MD_SHIFT)) & ADC_CFG0_SEQ0MD_MASK)
#define ADC_CFG0_ALIGN_MASK                                                   (0x80U)
#define ADC_CFG0_ALIGN_SHIFT                                                  (7U)
#define ADC_CFG0_ALIGN(x)                                                     (((uint32_t)(((uint32_t)(x)) << ADC_CFG0_ALIGN_SHIFT)) & ADC_CFG0_ALIGN_MASK)
#define ADC_CFG0_OVRMD_MASK                                                   (0x40U)
#define ADC_CFG0_OVRMD_SHIFT                                                  (6U)
#define ADC_CFG0_OVRMD(x)                                                     (((uint32_t)(((uint32_t)(x)) << ADC_CFG0_OVRMD_SHIFT)) & ADC_CFG0_OVRMD_MASK)
#define ADC_CFG0_AUTOOFF_MASK                                                 (0x20U)
#define ADC_CFG0_AUTOOFF_SHIFT                                                (5U)
#define ADC_CFG0_AUTOOFF(x)                                                   (((uint32_t)(((uint32_t)(x)) << ADC_CFG0_AUTOOFF_SHIFT)) & ADC_CFG0_AUTOOFF_MASK)
#define ADC_CFG0_WAIT_MASK                                                    (0x10U)
#define ADC_CFG0_WAIT_SHIFT                                                   (4U)
#define ADC_CFG0_WAIT(x)                                                      (((uint32_t)(((uint32_t)(x)) << ADC_CFG0_WAIT_SHIFT)) & ADC_CFG0_WAIT_MASK)
#define ADC_CFG0_TRIGMD_WIDTH                                                 (1)
#define ADC_CFG0_TRIGMD_MASK                                                  (0x8U)
#define ADC_CFG0_TRIGMD_SHIFT                                                 (3U)
#define ADC_CFG0_TRIGMD(x)                                                    (((uint32_t)(((uint32_t)(x)) << ADC_CFG0_TRIGMD_SHIFT)) & ADC_CFG0_TRIGMD_MASK)
#define ADC_CFG0_MODE_WIDTH                                                   (1)
#define ADC_CFG0_MODE_MASK                                                    (0x1U)
#define ADC_CFG0_MODE_SHIFT                                                   (0U)
#define ADC_CFG0_MODE(x)                                                      (((uint32_t)(((uint32_t)(x)) << ADC_CFG0_MODE_SHIFT)) & ADC_CFG0_MODE_MASK)
/* ADC_CFG1 Register */
#define ADC_CFG1_PRS_MASK                                                     (0xF0000U)
#define ADC_CFG1_PRS_SHIFT                                                    (16U)
#define ADC_CFG1_PRS(x)                                                       (((uint32_t)(((uint32_t)(x)) << ADC_CFG1_PRS_SHIFT)) & ADC_CFG1_PRS_MASK)
#define ADC_CFG1_STCNT_MASK                                                   (0xFFU)
#define ADC_CFG1_STCNT_SHIFT                                                  (0U)
#define ADC_CFG1_STCNT(x)                                                     (((uint32_t)(((uint32_t)(x)) << ADC_CFG1_STCNT_SHIFT)) & ADC_CFG1_STCNT_MASK)
/* ADC_SMP Register */
#define ADC_SMP_SMP1_MASK                                                     (0xFF00U)
#define ADC_SMP_SMP1_SHIFT                                                    (8U)
#define ADC_SMP_SMP1(x)                                                       (((uint32_t)(((uint32_t)(x)) << ADC_SMP_SMP1_SHIFT)) & ADC_SMP_SMP1_MASK)
#define ADC_SMP_SMP0_MASK                                                     (0xFFU)
#define ADC_SMP_SMP0_SHIFT                                                    (0U)
#define ADC_SMP_SMP0(x)                                                       (((uint32_t)(((uint32_t)(x)) << ADC_SMP_SMP0_SHIFT)) & ADC_SMP_SMP0_MASK)
/* ADC_WDCTRL Register */
#define ADC_WDCTRL_WDEN_MASK                                                  (0x80U)
#define ADC_WDCTRL_WDEN_SHIFT                                                 (7U)
#define ADC_WDCTRL_WDEN(x)                                                    (((uint32_t)(((uint32_t)(x)) << ADC_WDCTRL_WDEN_SHIFT)) & ADC_WDCTRL_WDEN_MASK)
#define ADC_WDCTRL_WDSGL_MASK                                                 (0x40U)
#define ADC_WDCTRL_WDSGL_SHIFT                                                (6U)
#define ADC_WDCTRL_WDSGL(x)                                                   (((uint32_t)(((uint32_t)(x)) << ADC_WDCTRL_WDSGL_SHIFT)) & ADC_WDCTRL_WDSGL_MASK)
#define ADC_WDCTRL_WDCHSEL_MASK                                               (0x1FU)
#define ADC_WDCTRL_WDCHSEL_SHIFT                                              (0U)
#define ADC_WDCTRL_WDCHSEL(x)                                                 (((uint32_t)(((uint32_t)(x)) << ADC_WDCTRL_WDCHSEL_SHIFT)) & ADC_WDCTRL_WDCHSEL_MASK)
/* ADC_WDTH Register */
#define ADC_WDTH_THMD_MASK                                                    (0x80000000U)
#define ADC_WDTH_THMD_SHIFT                                                   (31U)
#define ADC_WDTH_THMD(x)                                                      (((uint32_t)(((uint32_t)(x)) << ADC_WDTH_THMD_SHIFT)) & ADC_WDTH_THMD_MASK)
#define ADC_WDTH_HIGH_MASK                                                    (0xFFF0000U)
#define ADC_WDTH_HIGH_SHIFT                                                   (16U)
#define ADC_WDTH_HIGH(x)                                                      (((uint32_t)(((uint32_t)(x)) << ADC_WDTH_HIGH_SHIFT)) & ADC_WDTH_HIGH_MASK)
#define ADC_WDTH_LOW_MASK                                                     (0xFFFU)
#define ADC_WDTH_LOW_SHIFT                                                    (0U)
#define ADC_WDTH_LOW(x)                                                       (((uint32_t)(((uint32_t)(x)) << ADC_WDTH_LOW_SHIFT)) & ADC_WDTH_LOW_MASK)
/* ADC_CHSEL Register */
#define ADC_CHSEL_SMPSEL_MASK                                                 (0x80U)
#define ADC_CHSEL_SMPSEL_SHIFT                                                (7U)
#define ADC_CHSEL_SMPSEL(x)                                                   (((uint32_t)(((uint32_t)(x)) << ADC_CHSEL_SMPSEL_SHIFT)) & ADC_CHSEL_SMPSEL_MASK)
#define ADC_CHSEL_CHSEL_MASK                                                  (0x1FU)
#define ADC_CHSEL_CHSEL_SHIFT                                                 (0U)
#define ADC_CHSEL_CHSEL(x)                                                    (((uint32_t)(((uint32_t)(x)) << ADC_CHSEL_CHSEL_SHIFT)) & ADC_CHSEL_CHSEL_MASK)
/* ADC_FIFO Register */
#define ADC_FIFO_CHID_MASK                                                    (0x1F0000U)
#define ADC_FIFO_CHID_SHIFT                                                   (16U)
#define ADC_FIFO_CHID(x)                                                      (((uint32_t)(((uint32_t)(x)) << ADC_FIFO_CHID_SHIFT)) & ADC_FIFO_CHID_MASK)
#define ADC_FIFO_DATA_MASK                                                    (0xFFFFU)
#define ADC_FIFO_DATA_SHIFT                                                   (0U)
#define ADC_FIFO_DATA(x)                                                      (((uint32_t)(((uint32_t)(x)) << ADC_FIFO_DATA_SHIFT)) & ADC_FIFO_DATA_MASK)
/* ADC_TRIM Register */
#define ADC_TRIM_TRIM_WIDTH                                                   (7)
#define ADC_TRIM_TRIM_MASK                                                    (0x7FU)
#define ADC_TRIM_TRIM_SHIFT                                                   (0U)
#define ADC_TRIM_TRIM(x)                                                      (((uint32_t)(((uint32_t)(x)) << ADC_TRIM_TRIM_SHIFT)) & ADC_TRIM_TRIM_MASK)
/* ADC_TSC Register */
#define ADC_TSC_COEFF_WIDTH                                                   (12)
#define ADC_TSC_COEFF_MASK                                                    (0xFFFU)
#define ADC_TSC_COEFF_SHIFT                                                   (0U)
#define ADC_TSC_COEFF(x)                                                      (((uint32_t)(((uint32_t)(x)) << ADC_TSC_COEFF_SHIFT)) & ADC_TSC_COEFF_MASK)


/*!
 * @}
 */ /* end of group ADC_Register_Masks */


/*!
 * @}
 */ /* end of group ADC_Peripheral_Access_Layer */


/* -------------------------------------------------------------
   -- ZDU Peripheral Access Layer
   ------------------------------------------------------------- */

/*!
 * @addtogroup ZDU_Peripheral_Access_Layer ZDU Peripheral Access Layer
 * @{
 */


/** ZDU - Size of Registers Arrays */

/* ZDU Register Layout Typedef */
typedef struct {
    __IO uint32_t STS;                                                        /**< Status Register, offset: 0x0000 */
    __IO uint32_t INTE;                                                       /**< Interrupt Enable Register, offset: 0x0004 */
    __IO uint32_t CTRL;                                                       /**< Control Register, offset: 0x0008 */
    __IO uint32_t MUX;                                                        /**< MUX Register, offset: 0x000C */
    __IO uint32_t FILT;                                                       /**< Synchronous Register, offset: 0x0010 */
    __IO uint32_t MON;                                                        /**< Monitor Register, offset: 0x0014 */
    __IO uint32_t RESERVED6[ 1];                                              /**< RESERVED6, offset: 0x0018 */

} ZDU_Type, *ZDU_MemMapPtr;

/** Number of instances of the ZDU module. */
#define ZDU_INSTANCE_COUNT                                                    (1u)

/* ZDU0  base address */
#define ZDU0_BASE                                                             (0x40073000)
#define ZDU0                                                                  ((ZDU_Type *)(ZDU0_BASE))

/** Array initializer of ZDU peripheral base addresses */
#define ZDU_BASE_ADDRS                                                        { ZDU0_BASE }
/** Array initializer of ZDU peripheral base pointers */
#define ZDU_BASE_PTRS                                                         { ZDU0 }
/** Number of interrupt vector arrays for the ZDU module. */
#define ZDU_IRQS_ARR_COUNT                                                    (1u)
/** Number of interrupt channels for the ZDU. */
#define ZDU_IRQS_CH_COUNT                                                     (1u)
/** Interrupt vectors for the ZDU peripheral type */
#define ZDU_IRQS                                                              { ZDU_IRQn }

/* ZDU Register Mask */
/* ZDU_STS Register */
#define ZDU_STS_NF_MASK                                                       (0x4U)
#define ZDU_STS_NF_SHIFT                                                      (2U)
#define ZDU_STS_NF(x)                                                         (((uint32_t)(((uint32_t)(x)) << ZDU_STS_NF_SHIFT)) & ZDU_STS_NF_MASK)
#define ZDU_STS_PF_MASK                                                       (0x2U)
#define ZDU_STS_PF_SHIFT                                                      (1U)
#define ZDU_STS_PF(x)                                                         (((uint32_t)(((uint32_t)(x)) << ZDU_STS_PF_SHIFT)) & ZDU_STS_PF_MASK)
#define ZDU_STS_OUT_MASK                                                      (0x1U)
#define ZDU_STS_OUT_SHIFT                                                     (0U)
#define ZDU_STS_OUT(x)                                                        (((uint32_t)(((uint32_t)(x)) << ZDU_STS_OUT_SHIFT)) & ZDU_STS_OUT_MASK)
/* ZDU_INTE Register */
#define ZDU_INTE_IE_MASK                                                      (0x1U)
#define ZDU_INTE_IE_SHIFT                                                     (0U)
#define ZDU_INTE_IE(x)                                                        (((uint32_t)(((uint32_t)(x)) << ZDU_INTE_IE_SHIFT)) & ZDU_INTE_IE_MASK)
/* ZDU_CTRL Register */
#define ZDU_CTRL_HYSBP_MASK                                                   (0x10000000U)
#define ZDU_CTRL_HYSBP_SHIFT                                                  (28U)
#define ZDU_CTRL_HYSBP(x)                                                     (((uint32_t)(((uint32_t)(x)) << ZDU_CTRL_HYSBP_SHIFT)) & ZDU_CTRL_HYSBP_MASK)
#define ZDU_CTRL_PHSYNC_WIDTH                                                 (2)
#define ZDU_CTRL_PHSYNC_MASK                                                  (0x3000000U)
#define ZDU_CTRL_PHSYNC_SHIFT                                                 (24U)
#define ZDU_CTRL_PHSYNC(x)                                                    (((uint32_t)(((uint32_t)(x)) << ZDU_CTRL_PHSYNC_SHIFT)) & ZDU_CTRL_PHSYNC_MASK)
#define ZDU_CTRL_OTSEL_WIDTH                                                  (2)
#define ZDU_CTRL_OTSEL_MASK                                                   (0x300000U)
#define ZDU_CTRL_OTSEL_SHIFT                                                  (20U)
#define ZDU_CTRL_OTSEL(x)                                                     (((uint32_t)(((uint32_t)(x)) << ZDU_CTRL_OTSEL_SHIFT)) & ZDU_CTRL_OTSEL_MASK)
#define ZDU_CTRL_OTEN_MASK                                                    (0x10000U)
#define ZDU_CTRL_OTEN_SHIFT                                                   (16U)
#define ZDU_CTRL_OTEN(x)                                                      (((uint32_t)(((uint32_t)(x)) << ZDU_CTRL_OTEN_SHIFT)) & ZDU_CTRL_OTEN_MASK)
#define ZDU_CTRL_EDGESEL_WIDTH                                                (2)
#define ZDU_CTRL_EDGESEL_MASK                                                 (0x3000U)
#define ZDU_CTRL_EDGESEL_SHIFT                                                (12U)
#define ZDU_CTRL_EDGESEL(x)                                                   (((uint32_t)(((uint32_t)(x)) << ZDU_CTRL_EDGESEL_SHIFT)) & ZDU_CTRL_EDGESEL_MASK)
#define ZDU_CTRL_OUTSEL_MASK                                                  (0x100U)
#define ZDU_CTRL_OUTSEL_SHIFT                                                 (8U)
#define ZDU_CTRL_OUTSEL(x)                                                    (((uint32_t)(((uint32_t)(x)) << ZDU_CTRL_OUTSEL_SHIFT)) & ZDU_CTRL_OUTSEL_MASK)
#define ZDU_CTRL_MODE_MASK                                                    (0x10U)
#define ZDU_CTRL_MODE_SHIFT                                                   (4U)
#define ZDU_CTRL_MODE(x)                                                      (((uint32_t)(((uint32_t)(x)) << ZDU_CTRL_MODE_SHIFT)) & ZDU_CTRL_MODE_MASK)
#define ZDU_CTRL_EN_MASK                                                      (0x1U)
#define ZDU_CTRL_EN_SHIFT                                                     (0U)
#define ZDU_CTRL_EN(x)                                                        (((uint32_t)(((uint32_t)(x)) << ZDU_CTRL_EN_SHIFT)) & ZDU_CTRL_EN_MASK)
/* ZDU_MUX Register */
#define ZDU_MUX_PHSEL_WIDTH                                                   (2)
#define ZDU_MUX_PHSEL_MASK                                                    (0x3U)
#define ZDU_MUX_PHSEL_SHIFT                                                   (0U)
#define ZDU_MUX_PHSEL(x)                                                      (((uint32_t)(((uint32_t)(x)) << ZDU_MUX_PHSEL_SHIFT)) & ZDU_MUX_PHSEL_MASK)
/* ZDU_FILT Register */
#define ZDU_FILT_FCNT_WIDTH                                                   (8)
#define ZDU_FILT_FCNT_MASK                                                    (0xFF00U)
#define ZDU_FILT_FCNT_SHIFT                                                   (8U)
#define ZDU_FILT_FCNT(x)                                                      (((uint32_t)(((uint32_t)(x)) << ZDU_FILT_FCNT_SHIFT)) & ZDU_FILT_FCNT_MASK)
#define ZDU_FILT_FPER_WIDTH                                                   (4)
#define ZDU_FILT_FPER_MASK                                                    (0xFU)
#define ZDU_FILT_FPER_SHIFT                                                   (0U)
#define ZDU_FILT_FPER(x)                                                      (((uint32_t)(((uint32_t)(x)) << ZDU_FILT_FPER_SHIFT)) & ZDU_FILT_FPER_MASK)
/* ZDU_MON Register */
#define ZDU_MON_VNSEL_MASK                                                    (0x2U)
#define ZDU_MON_VNSEL_SHIFT                                                   (1U)
#define ZDU_MON_VNSEL(x)                                                      (((uint32_t)(((uint32_t)(x)) << ZDU_MON_VNSEL_SHIFT)) & ZDU_MON_VNSEL_MASK)
#define ZDU_MON_UVWSEL_MASK                                                   (0x1U)
#define ZDU_MON_UVWSEL_SHIFT                                                  (0U)
#define ZDU_MON_UVWSEL(x)                                                     (((uint32_t)(((uint32_t)(x)) << ZDU_MON_UVWSEL_SHIFT)) & ZDU_MON_UVWSEL_MASK)


/*!
 * @}
 */ /* end of group ZDU_Register_Masks */


/*!
 * @}
 */ /* end of group ZDU_Peripheral_Access_Layer */


/* -------------------------------------------------------------
   -- DRIVER Peripheral Access Layer
   ------------------------------------------------------------- */

/*!
 * @addtogroup DRIVER_Peripheral_Access_Layer DRIVER Peripheral Access Layer
 * @{
 */


/** DRIVER - Size of Registers Arrays */

/* DRIVER Register Layout Typedef */
typedef struct {
    __IO uint32_t STS0;                                                       /**< Status 0 Register, offset: 0x0000 */
    __IO uint32_t STS1;                                                       /**< Status 1 Register, offset: 0x0004 */
    __IO uint32_t INTEN;                                                      /**< Interrupt Enable Register, offset: 0x0008 */
    __IO uint32_t FILT;                                                       /**< Filter Register, offset: 0x000C */
    __IO uint32_t PROT;                                                       /**< Protect Register, offset: 0x0010 */
    __IO uint32_t PHASE;                                                      /**< Phase Register, offset: 0x0014 */
    __IO uint32_t CP;                                                         /**< Charge Pump Register, offset: 0x0018 */
    __IO uint32_t ILS;                                                        /**< ILS Register, offset: 0x001C */
    __IO uint32_t CTRL;                                                       /**< Control Register, offset: 0x0020 */
    __IO uint32_t IDAC;                                                       /**< IDAC Register, offset: 0x0024 */
    __IO uint32_t CMP;                                                        /**< Comparator Register, offset: 0x0028 */
    __IO uint32_t STEP;                                                       /**< Stepper Register, offset: 0x002C */
    __IO uint32_t FDCNT;                                                      /**< Fast Decay Counter Register, offset: 0x0030 */
    __IO uint32_t RESERVED13[ 3];                                             /**< RESERVED13, offset: 0x0034 */
    __IO uint32_t TRIM;                                                       /**< Trim Register, offset: 0x0040 */
    __IO uint32_t CAL;                                                        /**< Calibration Register, offset: 0x0044 */
    __IO uint32_t RESERVED16[ 2];                                             /**< RESERVED16, offset: 0x0048 */
    __IO uint32_t RESERVED17;                                                 /**< RESERVED17, offset: 0x0050 */

} DRIVER_Type, *DRIVER_MemMapPtr;

/** Number of instances of the DRIVER module. */
#define DRIVER_INSTANCE_COUNT                                                 (1u)

/* DRIVER base address */
#define DRIVER_BASE                                                           (0x40071000)
#define DRIVER                                                                ((DRIVER_Type *)(DRIVER_BASE))

/** Array initializer of DRIVER peripheral base addresses */
#define DRIVER_BASE_ADDRS                                                     { DRIVER_BASE }
/** Array initializer of DRIVER peripheral base pointers */
#define DRIVER_BASE_PTRS                                                      { DRIVER }

/* DRIVER Register Mask */
/* DRIVER_STS0 Register */
#define DRIVER_STS0_OVERTEMP_MASK                                             (0xF0000000U)
#define DRIVER_STS0_OVERTEMP_SHIFT                                            (28U)
#define DRIVER_STS0_OVERTEMP(x)                                               (((uint32_t)(((uint32_t)(x)) << DRIVER_STS0_OVERTEMP_SHIFT)) & DRIVER_STS0_OVERTEMP_MASK)
#define DRIVER_STS0_MALDET_MASK                                               (0xF000000U)
#define DRIVER_STS0_MALDET_SHIFT                                              (24U)
#define DRIVER_STS0_MALDET(x)                                                 (((uint32_t)(((uint32_t)(x)) << DRIVER_STS0_MALDET_SHIFT)) & DRIVER_STS0_MALDET_MASK)
#define DRIVER_STS0_HSVDSOV_MASK                                              (0xF00000U)
#define DRIVER_STS0_HSVDSOV_SHIFT                                             (20U)
#define DRIVER_STS0_HSVDSOV(x)                                                (((uint32_t)(((uint32_t)(x)) << DRIVER_STS0_HSVDSOV_SHIFT)) & DRIVER_STS0_HSVDSOV_MASK)
#define DRIVER_STS0_LSVDSOV_MASK                                              (0xF0000U)
#define DRIVER_STS0_LSVDSOV_SHIFT                                             (16U)
#define DRIVER_STS0_LSVDSOV(x)                                                (((uint32_t)(((uint32_t)(x)) << DRIVER_STS0_LSVDSOV_SHIFT)) & DRIVER_STS0_LSVDSOV_MASK)
#define DRIVER_STS0_UVFD_MASK                                                 (0x8000U)
#define DRIVER_STS0_UVFD_SHIFT                                                (15U)
#define DRIVER_STS0_UVFD(x)                                                   (((uint32_t)(((uint32_t)(x)) << DRIVER_STS0_UVFD_SHIFT)) & DRIVER_STS0_UVFD_MASK)
#define DRIVER_STS0_WTFD_MASK                                                 (0x4000U)
#define DRIVER_STS0_WTFD_SHIFT                                                (14U)
#define DRIVER_STS0_WTFD(x)                                                   (((uint32_t)(((uint32_t)(x)) << DRIVER_STS0_WTFD_SHIFT)) & DRIVER_STS0_WTFD_MASK)
#define DRIVER_STS0_UVCMPOUT_MASK                                             (0x2000U)
#define DRIVER_STS0_UVCMPOUT_SHIFT                                            (13U)
#define DRIVER_STS0_UVCMPOUT(x)                                               (((uint32_t)(((uint32_t)(x)) << DRIVER_STS0_UVCMPOUT_SHIFT)) & DRIVER_STS0_UVCMPOUT_MASK)
#define DRIVER_STS0_WTCMPOUT_MASK                                             (0x1000U)
#define DRIVER_STS0_WTCMPOUT_SHIFT                                            (12U)
#define DRIVER_STS0_WTCMPOUT(x)                                               (((uint32_t)(((uint32_t)(x)) << DRIVER_STS0_WTCMPOUT_SHIFT)) & DRIVER_STS0_WTCMPOUT_MASK)
#define DRIVER_STS0_VDDPUV_MASK                                               (0x20U)
#define DRIVER_STS0_VDDPUV_SHIFT                                              (5U)
#define DRIVER_STS0_VDDPUV(x)                                                 (((uint32_t)(((uint32_t)(x)) << DRIVER_STS0_VDDPUV_SHIFT)) & DRIVER_STS0_VDDPUV_MASK)
#define DRIVER_STS0_OVC_MASK                                                  (0x10U)
#define DRIVER_STS0_OVC_SHIFT                                                 (4U)
#define DRIVER_STS0_OVC(x)                                                    (((uint32_t)(((uint32_t)(x)) << DRIVER_STS0_OVC_SHIFT)) & DRIVER_STS0_OVC_MASK)
#define DRIVER_STS0_OVT_MASK                                                  (0x8U)
#define DRIVER_STS0_OVT_SHIFT                                                 (3U)
#define DRIVER_STS0_OVT(x)                                                    (((uint32_t)(((uint32_t)(x)) << DRIVER_STS0_OVT_SHIFT)) & DRIVER_STS0_OVT_MASK)
#define DRIVER_STS0_VSOV_MASK                                                 (0x4U)
#define DRIVER_STS0_VSOV_SHIFT                                                (2U)
#define DRIVER_STS0_VSOV(x)                                                   (((uint32_t)(((uint32_t)(x)) << DRIVER_STS0_VSOV_SHIFT)) & DRIVER_STS0_VSOV_MASK)
#define DRIVER_STS0_VSUV_MASK                                                 (0x2U)
#define DRIVER_STS0_VSUV_SHIFT                                                (1U)
#define DRIVER_STS0_VSUV(x)                                                   (((uint32_t)(((uint32_t)(x)) << DRIVER_STS0_VSUV_SHIFT)) & DRIVER_STS0_VSUV_MASK)
#define DRIVER_STS0_VDDAUV_MASK                                               (0x1U)
#define DRIVER_STS0_VDDAUV_SHIFT                                              (0U)
#define DRIVER_STS0_VDDAUV(x)                                                 (((uint32_t)(((uint32_t)(x)) << DRIVER_STS0_VDDAUV_SHIFT)) & DRIVER_STS0_VDDAUV_MASK)
/* DRIVER_STS1 Register */
#define DRIVER_STS1_OVERTEMPF_MASK                                            (0xF0000000U)
#define DRIVER_STS1_OVERTEMPF_SHIFT                                           (28U)
#define DRIVER_STS1_OVERTEMPF(x)                                              (((uint32_t)(((uint32_t)(x)) << DRIVER_STS1_OVERTEMPF_SHIFT)) & DRIVER_STS1_OVERTEMPF_MASK)
#define DRIVER_STS1_MALDETF_MASK                                              (0xF000000U)
#define DRIVER_STS1_MALDETF_SHIFT                                             (24U)
#define DRIVER_STS1_MALDETF(x)                                                (((uint32_t)(((uint32_t)(x)) << DRIVER_STS1_MALDETF_SHIFT)) & DRIVER_STS1_MALDETF_MASK)
#define DRIVER_STS1_HSVDSOVF_MASK                                             (0xF00000U)
#define DRIVER_STS1_HSVDSOVF_SHIFT                                            (20U)
#define DRIVER_STS1_HSVDSOVF(x)                                               (((uint32_t)(((uint32_t)(x)) << DRIVER_STS1_HSVDSOVF_SHIFT)) & DRIVER_STS1_HSVDSOVF_MASK)
#define DRIVER_STS1_LSVDSOVF_MASK                                             (0xF0000U)
#define DRIVER_STS1_LSVDSOVF_SHIFT                                            (16U)
#define DRIVER_STS1_LSVDSOVF(x)                                               (((uint32_t)(((uint32_t)(x)) << DRIVER_STS1_LSVDSOVF_SHIFT)) & DRIVER_STS1_LSVDSOVF_MASK)
#define DRIVER_STS1_UVCMPPOSF_MASK                                            (0x800U)
#define DRIVER_STS1_UVCMPPOSF_SHIFT                                           (11U)
#define DRIVER_STS1_UVCMPPOSF(x)                                              (((uint32_t)(((uint32_t)(x)) << DRIVER_STS1_UVCMPPOSF_SHIFT)) & DRIVER_STS1_UVCMPPOSF_MASK)
#define DRIVER_STS1_UVCMPNEGF_MASK                                            (0x400U)
#define DRIVER_STS1_UVCMPNEGF_SHIFT                                           (10U)
#define DRIVER_STS1_UVCMPNEGF(x)                                              (((uint32_t)(((uint32_t)(x)) << DRIVER_STS1_UVCMPNEGF_SHIFT)) & DRIVER_STS1_UVCMPNEGF_MASK)
#define DRIVER_STS1_WTCMPPOSF_MASK                                            (0x200U)
#define DRIVER_STS1_WTCMPPOSF_SHIFT                                           (9U)
#define DRIVER_STS1_WTCMPPOSF(x)                                              (((uint32_t)(((uint32_t)(x)) << DRIVER_STS1_WTCMPPOSF_SHIFT)) & DRIVER_STS1_WTCMPPOSF_MASK)
#define DRIVER_STS1_WTCMPNEGF_MASK                                            (0x100U)
#define DRIVER_STS1_WTCMPNEGF_SHIFT                                           (8U)
#define DRIVER_STS1_WTCMPNEGF(x)                                              (((uint32_t)(((uint32_t)(x)) << DRIVER_STS1_WTCMPNEGF_SHIFT)) & DRIVER_STS1_WTCMPNEGF_MASK)
#define DRIVER_STS1_VDDPUVF_MASK                                              (0x20U)
#define DRIVER_STS1_VDDPUVF_SHIFT                                             (5U)
#define DRIVER_STS1_VDDPUVF(x)                                                (((uint32_t)(((uint32_t)(x)) << DRIVER_STS1_VDDPUVF_SHIFT)) & DRIVER_STS1_VDDPUVF_MASK)
#define DRIVER_STS1_OVCF_MASK                                                 (0x10U)
#define DRIVER_STS1_OVCF_SHIFT                                                (4U)
#define DRIVER_STS1_OVCF(x)                                                   (((uint32_t)(((uint32_t)(x)) << DRIVER_STS1_OVCF_SHIFT)) & DRIVER_STS1_OVCF_MASK)
#define DRIVER_STS1_OVTF_MASK                                                 (0x8U)
#define DRIVER_STS1_OVTF_SHIFT                                                (3U)
#define DRIVER_STS1_OVTF(x)                                                   (((uint32_t)(((uint32_t)(x)) << DRIVER_STS1_OVTF_SHIFT)) & DRIVER_STS1_OVTF_MASK)
#define DRIVER_STS1_VSOVF_MASK                                                (0x4U)
#define DRIVER_STS1_VSOVF_SHIFT                                               (2U)
#define DRIVER_STS1_VSOVF(x)                                                  (((uint32_t)(((uint32_t)(x)) << DRIVER_STS1_VSOVF_SHIFT)) & DRIVER_STS1_VSOVF_MASK)
#define DRIVER_STS1_VSUVF_MASK                                                (0x2U)
#define DRIVER_STS1_VSUVF_SHIFT                                               (1U)
#define DRIVER_STS1_VSUVF(x)                                                  (((uint32_t)(((uint32_t)(x)) << DRIVER_STS1_VSUVF_SHIFT)) & DRIVER_STS1_VSUVF_MASK)
#define DRIVER_STS1_VDDAUVF_MASK                                              (0x1U)
#define DRIVER_STS1_VDDAUVF_SHIFT                                             (0U)
#define DRIVER_STS1_VDDAUVF(x)                                                (((uint32_t)(((uint32_t)(x)) << DRIVER_STS1_VDDAUVF_SHIFT)) & DRIVER_STS1_VDDAUVF_MASK)
/* DRIVER_INTEN Register */
#define DRIVER_INTEN_OVERTEMPIE_MASK                                          (0xF0000000U)
#define DRIVER_INTEN_OVERTEMPIE_SHIFT                                         (28U)
#define DRIVER_INTEN_OVERTEMPIE(x)                                            (((uint32_t)(((uint32_t)(x)) << DRIVER_INTEN_OVERTEMPIE_SHIFT)) & DRIVER_INTEN_OVERTEMPIE_MASK)
#define DRIVER_INTEN_MALDETIE_MASK                                            (0xF000000U)
#define DRIVER_INTEN_MALDETIE_SHIFT                                           (24U)
#define DRIVER_INTEN_MALDETIE(x)                                              (((uint32_t)(((uint32_t)(x)) << DRIVER_INTEN_MALDETIE_SHIFT)) & DRIVER_INTEN_MALDETIE_MASK)
#define DRIVER_INTEN_HSVDSOVIE_MASK                                           (0xF00000U)
#define DRIVER_INTEN_HSVDSOVIE_SHIFT                                          (20U)
#define DRIVER_INTEN_HSVDSOVIE(x)                                             (((uint32_t)(((uint32_t)(x)) << DRIVER_INTEN_HSVDSOVIE_SHIFT)) & DRIVER_INTEN_HSVDSOVIE_MASK)
#define DRIVER_INTEN_LSVDSOVIE_MASK                                           (0xF0000U)
#define DRIVER_INTEN_LSVDSOVIE_SHIFT                                          (16U)
#define DRIVER_INTEN_LSVDSOVIE(x)                                             (((uint32_t)(((uint32_t)(x)) << DRIVER_INTEN_LSVDSOVIE_SHIFT)) & DRIVER_INTEN_LSVDSOVIE_MASK)
#define DRIVER_INTEN_UVCMPPOSIE_MASK                                          (0x800U)
#define DRIVER_INTEN_UVCMPPOSIE_SHIFT                                         (11U)
#define DRIVER_INTEN_UVCMPPOSIE(x)                                            (((uint32_t)(((uint32_t)(x)) << DRIVER_INTEN_UVCMPPOSIE_SHIFT)) & DRIVER_INTEN_UVCMPPOSIE_MASK)
#define DRIVER_INTEN_UVCMPNEGIE_MASK                                          (0x400U)
#define DRIVER_INTEN_UVCMPNEGIE_SHIFT                                         (10U)
#define DRIVER_INTEN_UVCMPNEGIE(x)                                            (((uint32_t)(((uint32_t)(x)) << DRIVER_INTEN_UVCMPNEGIE_SHIFT)) & DRIVER_INTEN_UVCMPNEGIE_MASK)
#define DRIVER_INTEN_WTCMPPOSIE_MASK                                          (0x200U)
#define DRIVER_INTEN_WTCMPPOSIE_SHIFT                                         (9U)
#define DRIVER_INTEN_WTCMPPOSIE(x)                                            (((uint32_t)(((uint32_t)(x)) << DRIVER_INTEN_WTCMPPOSIE_SHIFT)) & DRIVER_INTEN_WTCMPPOSIE_MASK)
#define DRIVER_INTEN_WTCMPNEGIE_MASK                                          (0x100U)
#define DRIVER_INTEN_WTCMPNEGIE_SHIFT                                         (8U)
#define DRIVER_INTEN_WTCMPNEGIE(x)                                            (((uint32_t)(((uint32_t)(x)) << DRIVER_INTEN_WTCMPNEGIE_SHIFT)) & DRIVER_INTEN_WTCMPNEGIE_MASK)
#define DRIVER_INTEN_VDDPUVIE_MASK                                            (0x20U)
#define DRIVER_INTEN_VDDPUVIE_SHIFT                                           (5U)
#define DRIVER_INTEN_VDDPUVIE(x)                                              (((uint32_t)(((uint32_t)(x)) << DRIVER_INTEN_VDDPUVIE_SHIFT)) & DRIVER_INTEN_VDDPUVIE_MASK)
#define DRIVER_INTEN_OVCIE_MASK                                               (0x10U)
#define DRIVER_INTEN_OVCIE_SHIFT                                              (4U)
#define DRIVER_INTEN_OVCIE(x)                                                 (((uint32_t)(((uint32_t)(x)) << DRIVER_INTEN_OVCIE_SHIFT)) & DRIVER_INTEN_OVCIE_MASK)
#define DRIVER_INTEN_OVTIE_MASK                                               (0x8U)
#define DRIVER_INTEN_OVTIE_SHIFT                                              (3U)
#define DRIVER_INTEN_OVTIE(x)                                                 (((uint32_t)(((uint32_t)(x)) << DRIVER_INTEN_OVTIE_SHIFT)) & DRIVER_INTEN_OVTIE_MASK)
#define DRIVER_INTEN_VSOVIE_MASK                                              (0x4U)
#define DRIVER_INTEN_VSOVIE_SHIFT                                             (2U)
#define DRIVER_INTEN_VSOVIE(x)                                                (((uint32_t)(((uint32_t)(x)) << DRIVER_INTEN_VSOVIE_SHIFT)) & DRIVER_INTEN_VSOVIE_MASK)
#define DRIVER_INTEN_VSUVIE_MASK                                              (0x2U)
#define DRIVER_INTEN_VSUVIE_SHIFT                                             (1U)
#define DRIVER_INTEN_VSUVIE(x)                                                (((uint32_t)(((uint32_t)(x)) << DRIVER_INTEN_VSUVIE_SHIFT)) & DRIVER_INTEN_VSUVIE_MASK)
#define DRIVER_INTEN_VDDAUVIE_MASK                                            (0x1U)
#define DRIVER_INTEN_VDDAUVIE_SHIFT                                           (0U)
#define DRIVER_INTEN_VDDAUVIE(x)                                              (((uint32_t)(((uint32_t)(x)) << DRIVER_INTEN_VDDAUVIE_SHIFT)) & DRIVER_INTEN_VDDAUVIE_MASK)
/* DRIVER_FILT Register */
#define DRIVER_FILT_VDDPUVFILT_MASK                                           (0x60U)
#define DRIVER_FILT_VDDPUVFILT_SHIFT                                          (5U)
#define DRIVER_FILT_VDDPUVFILT(x)                                             (((uint32_t)(((uint32_t)(x)) << DRIVER_FILT_VDDPUVFILT_SHIFT)) & DRIVER_FILT_VDDPUVFILT_MASK)
#define DRIVER_FILT_OVCFILT_MASK                                              (0x10U)
#define DRIVER_FILT_OVCFILT_SHIFT                                             (4U)
#define DRIVER_FILT_OVCFILT(x)                                                (((uint32_t)(((uint32_t)(x)) << DRIVER_FILT_OVCFILT_SHIFT)) & DRIVER_FILT_OVCFILT_MASK)
#define DRIVER_FILT_OVTFILT_MASK                                              (0x8U)
#define DRIVER_FILT_OVTFILT_SHIFT                                             (3U)
#define DRIVER_FILT_OVTFILT(x)                                                (((uint32_t)(((uint32_t)(x)) << DRIVER_FILT_OVTFILT_SHIFT)) & DRIVER_FILT_OVTFILT_MASK)
#define DRIVER_FILT_VSOVFILT_MASK                                             (0x4U)
#define DRIVER_FILT_VSOVFILT_SHIFT                                            (2U)
#define DRIVER_FILT_VSOVFILT(x)                                               (((uint32_t)(((uint32_t)(x)) << DRIVER_FILT_VSOVFILT_SHIFT)) & DRIVER_FILT_VSOVFILT_MASK)
#define DRIVER_FILT_VSUVFILT_MASK                                             (0x2U)
#define DRIVER_FILT_VSUVFILT_SHIFT                                            (1U)
#define DRIVER_FILT_VSUVFILT(x)                                               (((uint32_t)(((uint32_t)(x)) << DRIVER_FILT_VSUVFILT_SHIFT)) & DRIVER_FILT_VSUVFILT_MASK)
#define DRIVER_FILT_VDDAUVFILT_MASK                                           (0x1U)
#define DRIVER_FILT_VDDAUVFILT_SHIFT                                          (0U)
#define DRIVER_FILT_VDDAUVFILT(x)                                             (((uint32_t)(((uint32_t)(x)) << DRIVER_FILT_VDDAUVFILT_SHIFT)) & DRIVER_FILT_VDDAUVFILT_MASK)
/* DRIVER_PROT Register */
#define DRIVER_PROT_PROTPRI_MASK                                              (0x70000000U)
#define DRIVER_PROT_PROTPRI_SHIFT                                             (28U)
#define DRIVER_PROT_PROTPRI(x)                                                (((uint32_t)(((uint32_t)(x)) << DRIVER_PROT_PROTPRI_SHIFT)) & DRIVER_PROT_PROTPRI_MASK)
#define DRIVER_PROT_VDSSW_MASK                                                (0x1000000U)
#define DRIVER_PROT_VDSSW_SHIFT                                               (24U)
#define DRIVER_PROT_VDSSW(x)                                                  (((uint32_t)(((uint32_t)(x)) << DRIVER_PROT_VDSSW_SHIFT)) & DRIVER_PROT_VDSSW_MASK)
#define DRIVER_PROT_TVDS_MASK                                                 (0x300000U)
#define DRIVER_PROT_TVDS_SHIFT                                                (20U)
#define DRIVER_PROT_TVDS(x)                                                   (((uint32_t)(((uint32_t)(x)) << DRIVER_PROT_TVDS_SHIFT)) & DRIVER_PROT_TVDS_MASK)
#define DRIVER_PROT_UVDSEN_MASK                                               (0x80000U)
#define DRIVER_PROT_UVDSEN_SHIFT                                              (19U)
#define DRIVER_PROT_UVDSEN(x)                                                 (((uint32_t)(((uint32_t)(x)) << DRIVER_PROT_UVDSEN_SHIFT)) & DRIVER_PROT_UVDSEN_MASK)
#define DRIVER_PROT_VVDSEN_MASK                                               (0x40000U)
#define DRIVER_PROT_VVDSEN_SHIFT                                              (18U)
#define DRIVER_PROT_VVDSEN(x)                                                 (((uint32_t)(((uint32_t)(x)) << DRIVER_PROT_VVDSEN_SHIFT)) & DRIVER_PROT_VVDSEN_MASK)
#define DRIVER_PROT_WVDSEN_MASK                                               (0x20000U)
#define DRIVER_PROT_WVDSEN_SHIFT                                              (17U)
#define DRIVER_PROT_WVDSEN(x)                                                 (((uint32_t)(((uint32_t)(x)) << DRIVER_PROT_WVDSEN_SHIFT)) & DRIVER_PROT_WVDSEN_MASK)
#define DRIVER_PROT_TVDSEN_MASK                                               (0x10000U)
#define DRIVER_PROT_TVDSEN_SHIFT                                              (16U)
#define DRIVER_PROT_TVDSEN(x)                                                 (((uint32_t)(((uint32_t)(x)) << DRIVER_PROT_TVDSEN_SHIFT)) & DRIVER_PROT_TVDSEN_MASK)
#define DRIVER_PROT_HSVDSPSEL_MASK                                            (0xC000U)
#define DRIVER_PROT_HSVDSPSEL_SHIFT                                           (14U)
#define DRIVER_PROT_HSVDSPSEL(x)                                              (((uint32_t)(((uint32_t)(x)) << DRIVER_PROT_HSVDSPSEL_SHIFT)) & DRIVER_PROT_HSVDSPSEL_MASK)
#define DRIVER_PROT_LSVDSPSEL_MASK                                            (0x3000U)
#define DRIVER_PROT_LSVDSPSEL_SHIFT                                           (12U)
#define DRIVER_PROT_LSVDSPSEL(x)                                              (((uint32_t)(((uint32_t)(x)) << DRIVER_PROT_LSVDSPSEL_SHIFT)) & DRIVER_PROT_LSVDSPSEL_MASK)
#define DRIVER_PROT_VDDPUVPSEL_MASK                                           (0xC00U)
#define DRIVER_PROT_VDDPUVPSEL_SHIFT                                          (10U)
#define DRIVER_PROT_VDDPUVPSEL(x)                                             (((uint32_t)(((uint32_t)(x)) << DRIVER_PROT_VDDPUVPSEL_SHIFT)) & DRIVER_PROT_VDDPUVPSEL_MASK)
#define DRIVER_PROT_OVCPSEL_MASK                                              (0x300U)
#define DRIVER_PROT_OVCPSEL_SHIFT                                             (8U)
#define DRIVER_PROT_OVCPSEL(x)                                                (((uint32_t)(((uint32_t)(x)) << DRIVER_PROT_OVCPSEL_SHIFT)) & DRIVER_PROT_OVCPSEL_MASK)
#define DRIVER_PROT_OVTPSEL_MASK                                              (0xC0U)
#define DRIVER_PROT_OVTPSEL_SHIFT                                             (6U)
#define DRIVER_PROT_OVTPSEL(x)                                                (((uint32_t)(((uint32_t)(x)) << DRIVER_PROT_OVTPSEL_SHIFT)) & DRIVER_PROT_OVTPSEL_MASK)
#define DRIVER_PROT_VSOVPSEL_MASK                                             (0x30U)
#define DRIVER_PROT_VSOVPSEL_SHIFT                                            (4U)
#define DRIVER_PROT_VSOVPSEL(x)                                               (((uint32_t)(((uint32_t)(x)) << DRIVER_PROT_VSOVPSEL_SHIFT)) & DRIVER_PROT_VSOVPSEL_MASK)
#define DRIVER_PROT_VSUVPSEL_MASK                                             (0xCU)
#define DRIVER_PROT_VSUVPSEL_SHIFT                                            (2U)
#define DRIVER_PROT_VSUVPSEL(x)                                               (((uint32_t)(((uint32_t)(x)) << DRIVER_PROT_VSUVPSEL_SHIFT)) & DRIVER_PROT_VSUVPSEL_MASK)
#define DRIVER_PROT_VDDAUVPSEL_MASK                                           (0x3U)
#define DRIVER_PROT_VDDAUVPSEL_SHIFT                                          (0U)
#define DRIVER_PROT_VDDAUVPSEL(x)                                             (((uint32_t)(((uint32_t)(x)) << DRIVER_PROT_VDDAUVPSEL_SHIFT)) & DRIVER_PROT_VDDAUVPSEL_MASK)
/* DRIVER_PHASE Register */
#define DRIVER_PHASE_USRC_MASK                                                (0xF0000000U)
#define DRIVER_PHASE_USRC_SHIFT                                               (28U)
#define DRIVER_PHASE_USRC(x)                                                  (((uint32_t)(((uint32_t)(x)) << DRIVER_PHASE_USRC_SHIFT)) & DRIVER_PHASE_USRC_MASK)
#define DRIVER_PHASE_VSRC_MASK                                                (0xF000000U)
#define DRIVER_PHASE_VSRC_SHIFT                                               (24U)
#define DRIVER_PHASE_VSRC(x)                                                  (((uint32_t)(((uint32_t)(x)) << DRIVER_PHASE_VSRC_SHIFT)) & DRIVER_PHASE_VSRC_MASK)
#define DRIVER_PHASE_WSRC_MASK                                                (0xF00000U)
#define DRIVER_PHASE_WSRC_SHIFT                                               (20U)
#define DRIVER_PHASE_WSRC(x)                                                  (((uint32_t)(((uint32_t)(x)) << DRIVER_PHASE_WSRC_SHIFT)) & DRIVER_PHASE_WSRC_MASK)
#define DRIVER_PHASE_TSRC_MASK                                                (0xF0000U)
#define DRIVER_PHASE_TSRC_SHIFT                                               (16U)
#define DRIVER_PHASE_TSRC(x)                                                  (((uint32_t)(((uint32_t)(x)) << DRIVER_PHASE_TSRC_SHIFT)) & DRIVER_PHASE_TSRC_MASK)
#define DRIVER_PHASE_UDRVEN_MASK                                              (0x8U)
#define DRIVER_PHASE_UDRVEN_SHIFT                                             (3U)
#define DRIVER_PHASE_UDRVEN(x)                                                (((uint32_t)(((uint32_t)(x)) << DRIVER_PHASE_UDRVEN_SHIFT)) & DRIVER_PHASE_UDRVEN_MASK)
#define DRIVER_PHASE_VDRVEN_MASK                                              (0x4U)
#define DRIVER_PHASE_VDRVEN_SHIFT                                             (2U)
#define DRIVER_PHASE_VDRVEN(x)                                                (((uint32_t)(((uint32_t)(x)) << DRIVER_PHASE_VDRVEN_SHIFT)) & DRIVER_PHASE_VDRVEN_MASK)
#define DRIVER_PHASE_WDRVEN_MASK                                              (0x2U)
#define DRIVER_PHASE_WDRVEN_SHIFT                                             (1U)
#define DRIVER_PHASE_WDRVEN(x)                                                (((uint32_t)(((uint32_t)(x)) << DRIVER_PHASE_WDRVEN_SHIFT)) & DRIVER_PHASE_WDRVEN_MASK)
#define DRIVER_PHASE_TDRVEN_MASK                                              (0x1U)
#define DRIVER_PHASE_TDRVEN_SHIFT                                             (0U)
#define DRIVER_PHASE_TDRVEN(x)                                                (((uint32_t)(((uint32_t)(x)) << DRIVER_PHASE_TDRVEN_SHIFT)) & DRIVER_PHASE_TDRVEN_MASK)
/* DRIVER_CP Register */
#define DRIVER_CP_FRACDIVVAL_MASK                                             (0xF000000U)
#define DRIVER_CP_FRACDIVVAL_SHIFT                                            (24U)
#define DRIVER_CP_FRACDIVVAL(x)                                               (((uint32_t)(((uint32_t)(x)) << DRIVER_CP_FRACDIVVAL_SHIFT)) & DRIVER_CP_FRACDIVVAL_MASK)
#define DRIVER_CP_FRACDIVEN_MASK                                              (0x10000U)
#define DRIVER_CP_FRACDIVEN_SHIFT                                             (16U)
#define DRIVER_CP_FRACDIVEN(x)                                                (((uint32_t)(((uint32_t)(x)) << DRIVER_CP_FRACDIVEN_SHIFT)) & DRIVER_CP_FRACDIVEN_MASK)
#define DRIVER_CP_DRVMOD_MASK                                                 (0x30U)
#define DRIVER_CP_DRVMOD_SHIFT                                                (4U)
#define DRIVER_CP_DRVMOD(x)                                                   (((uint32_t)(((uint32_t)(x)) << DRIVER_CP_DRVMOD_SHIFT)) & DRIVER_CP_DRVMOD_MASK)
#define DRIVER_CP_CPCLKEN_MASK                                                (0x1U)
#define DRIVER_CP_CPCLKEN_SHIFT                                               (0U)
#define DRIVER_CP_CPCLKEN(x)                                                  (((uint32_t)(((uint32_t)(x)) << DRIVER_CP_CPCLKEN_SHIFT)) & DRIVER_CP_CPCLKEN_MASK)
/* DRIVER_ILS Register */
#define DRIVER_ILS_CSCAL_MASK                                                 (0x40000U)
#define DRIVER_ILS_CSCAL_SHIFT                                                (18U)
#define DRIVER_ILS_CSCAL(x)                                                   (((uint32_t)(((uint32_t)(x)) << DRIVER_ILS_CSCAL_SHIFT)) & DRIVER_ILS_CSCAL_MASK)
#define DRIVER_ILS_CSEN_MASK                                                  (0x20000U)
#define DRIVER_ILS_CSEN_SHIFT                                                 (17U)
#define DRIVER_ILS_CSEN(x)                                                    (((uint32_t)(((uint32_t)(x)) << DRIVER_ILS_CSEN_SHIFT)) & DRIVER_ILS_CSEN_MASK)
#define DRIVER_ILS_EXTSEN_MASK                                                (0x10000U)
#define DRIVER_ILS_EXTSEN_SHIFT                                               (16U)
#define DRIVER_ILS_EXTSEN(x)                                                  (((uint32_t)(((uint32_t)(x)) << DRIVER_ILS_EXTSEN_SHIFT)) & DRIVER_ILS_EXTSEN_MASK)
#define DRIVER_ILS_OVCFG_MASK                                                 (0xFFU)
#define DRIVER_ILS_OVCFG_SHIFT                                                (0U)
#define DRIVER_ILS_OVCFG(x)                                                   (((uint32_t)(((uint32_t)(x)) << DRIVER_ILS_OVCFG_SHIFT)) & DRIVER_ILS_OVCFG_MASK)
/* DRIVER_CTRL Register */
#define DRIVER_CTRL_UNLOCK_MASK                                               (0x80000000U)
#define DRIVER_CTRL_UNLOCK_SHIFT                                              (31U)
#define DRIVER_CTRL_UNLOCK(x)                                                 (((uint32_t)(((uint32_t)(x)) << DRIVER_CTRL_UNLOCK_SHIFT)) & DRIVER_CTRL_UNLOCK_MASK)
#define DRIVER_CTRL_LOCK_MASK                                                 (0x40000000U)
#define DRIVER_CTRL_LOCK_SHIFT                                                (30U)
#define DRIVER_CTRL_LOCK(x)                                                   (((uint32_t)(((uint32_t)(x)) << DRIVER_CTRL_LOCK_SHIFT)) & DRIVER_CTRL_LOCK_MASK)
#define DRIVER_CTRL_LSCSEN_MASK                                               (0x100U)
#define DRIVER_CTRL_LSCSEN_SHIFT                                              (8U)
#define DRIVER_CTRL_LSCSEN(x)                                                 (((uint32_t)(((uint32_t)(x)) << DRIVER_CTRL_LSCSEN_SHIFT)) & DRIVER_CTRL_LSCSEN_MASK)
#define DRIVER_CTRL_PARALLEL_MASK                                             (0xF0U)
#define DRIVER_CTRL_PARALLEL_SHIFT                                            (4U)
#define DRIVER_CTRL_PARALLEL(x)                                               (((uint32_t)(((uint32_t)(x)) << DRIVER_CTRL_PARALLEL_SHIFT)) & DRIVER_CTRL_PARALLEL_MASK)
#define DRIVER_CTRL_PULLDOWNB_MASK                                            (0x4U)
#define DRIVER_CTRL_PULLDOWNB_SHIFT                                           (2U)
#define DRIVER_CTRL_PULLDOWNB(x)                                              (((uint32_t)(((uint32_t)(x)) << DRIVER_CTRL_PULLDOWNB_SHIFT)) & DRIVER_CTRL_PULLDOWNB_MASK)
#define DRIVER_CTRL_VDDPEN_MASK                                               (0x2U)
#define DRIVER_CTRL_VDDPEN_SHIFT                                              (1U)
#define DRIVER_CTRL_VDDPEN(x)                                                 (((uint32_t)(((uint32_t)(x)) << DRIVER_CTRL_VDDPEN_SHIFT)) & DRIVER_CTRL_VDDPEN_MASK)
#define DRIVER_CTRL_EN_MASK                                                   (0x1U)
#define DRIVER_CTRL_EN_SHIFT                                                  (0U)
#define DRIVER_CTRL_EN(x)                                                     (((uint32_t)(((uint32_t)(x)) << DRIVER_CTRL_EN_SHIFT)) & DRIVER_CTRL_EN_MASK)
/* DRIVER_IDAC Register */
#define DRIVER_IDAC_WTIDACVAL_MASK                                            (0xFF000000U)
#define DRIVER_IDAC_WTIDACVAL_SHIFT                                           (24U)
#define DRIVER_IDAC_WTIDACVAL(x)                                              (((uint32_t)(((uint32_t)(x)) << DRIVER_IDAC_WTIDACVAL_SHIFT)) & DRIVER_IDAC_WTIDACVAL_MASK)
#define DRIVER_IDAC_UVIDACVAL_MASK                                            (0xFF0000U)
#define DRIVER_IDAC_UVIDACVAL_SHIFT                                           (16U)
#define DRIVER_IDAC_UVIDACVAL(x)                                              (((uint32_t)(((uint32_t)(x)) << DRIVER_IDAC_UVIDACVAL_SHIFT)) & DRIVER_IDAC_UVIDACVAL_MASK)
#define DRIVER_IDAC_IDACSEL_MASK                                              (0xF00U)
#define DRIVER_IDAC_IDACSEL_SHIFT                                             (8U)
#define DRIVER_IDAC_IDACSEL(x)                                                (((uint32_t)(((uint32_t)(x)) << DRIVER_IDAC_IDACSEL_SHIFT)) & DRIVER_IDAC_IDACSEL_MASK)
#define DRIVER_IDAC_IDACREF_MASK                                              (0x30U)
#define DRIVER_IDAC_IDACREF_SHIFT                                             (4U)
#define DRIVER_IDAC_IDACREF(x)                                                (((uint32_t)(((uint32_t)(x)) << DRIVER_IDAC_IDACREF_SHIFT)) & DRIVER_IDAC_IDACREF_MASK)
#define DRIVER_IDAC_IDACEN_MASK                                               (0x1U)
#define DRIVER_IDAC_IDACEN_SHIFT                                              (0U)
#define DRIVER_IDAC_IDACEN(x)                                                 (((uint32_t)(((uint32_t)(x)) << DRIVER_IDAC_IDACEN_SHIFT)) & DRIVER_IDAC_IDACEN_MASK)
/* DRIVER_CMP Register */
#define DRIVER_CMP_CMPFILT_MASK                                               (0x7U)
#define DRIVER_CMP_CMPFILT_SHIFT                                              (0U)
#define DRIVER_CMP_CMPFILT(x)                                                 (((uint32_t)(((uint32_t)(x)) << DRIVER_CMP_CMPFILT_SHIFT)) & DRIVER_CMP_CMPFILT_MASK)
/* DRIVER_STEP Register */
#define DRIVER_STEP_FDPOL_MASK                                                (0xF0000000U)
#define DRIVER_STEP_FDPOL_SHIFT                                               (28U)
#define DRIVER_STEP_FDPOL(x)                                                  (((uint32_t)(((uint32_t)(x)) << DRIVER_STEP_FDPOL_SHIFT)) & DRIVER_STEP_FDPOL_MASK)
#define DRIVER_STEP_PWMONPOL_MASK                                             (0xF000000U)
#define DRIVER_STEP_PWMONPOL_SHIFT                                            (24U)
#define DRIVER_STEP_PWMONPOL(x)                                               (((uint32_t)(((uint32_t)(x)) << DRIVER_STEP_PWMONPOL_SHIFT)) & DRIVER_STEP_PWMONPOL_MASK)
#define DRIVER_STEP_FDFORCE_MASK                                              (0xF00000U)
#define DRIVER_STEP_FDFORCE_SHIFT                                             (20U)
#define DRIVER_STEP_FDFORCE(x)                                                (((uint32_t)(((uint32_t)(x)) << DRIVER_STEP_FDFORCE_SHIFT)) & DRIVER_STEP_FDFORCE_MASK)
#define DRIVER_STEP_PWMONFORCE_MASK                                           (0xF0000U)
#define DRIVER_STEP_PWMONFORCE_SHIFT                                          (16U)
#define DRIVER_STEP_PWMONFORCE(x)                                             (((uint32_t)(((uint32_t)(x)) << DRIVER_STEP_PWMONFORCE_SHIFT)) & DRIVER_STEP_PWMONFORCE_MASK)
#define DRIVER_STEP_TRIFORCE_MASK                                             (0xF000U)
#define DRIVER_STEP_TRIFORCE_SHIFT                                            (12U)
#define DRIVER_STEP_TRIFORCE(x)                                               (((uint32_t)(((uint32_t)(x)) << DRIVER_STEP_TRIFORCE_SHIFT)) & DRIVER_STEP_TRIFORCE_MASK)
#define DRIVER_STEP_TRICFG_MASK                                               (0xF00U)
#define DRIVER_STEP_TRICFG_SHIFT                                              (8U)
#define DRIVER_STEP_TRICFG(x)                                                 (((uint32_t)(((uint32_t)(x)) << DRIVER_STEP_TRICFG_SHIFT)) & DRIVER_STEP_TRICFG_MASK)
#define DRIVER_STEP_TRIFORCEEN_MASK                                           (0x80U)
#define DRIVER_STEP_TRIFORCEEN_SHIFT                                          (7U)
#define DRIVER_STEP_TRIFORCEEN(x)                                             (((uint32_t)(((uint32_t)(x)) << DRIVER_STEP_TRIFORCEEN_SHIFT)) & DRIVER_STEP_TRIFORCEEN_MASK)
#define DRIVER_STEP_TRISYNC_MASK                                              (0x40U)
#define DRIVER_STEP_TRISYNC_SHIFT                                             (6U)
#define DRIVER_STEP_TRISYNC(x)                                                (((uint32_t)(((uint32_t)(x)) << DRIVER_STEP_TRISYNC_SHIFT)) & DRIVER_STEP_TRISYNC_MASK)
#define DRIVER_STEP_UVTRIEN_MASK                                              (0x20U)
#define DRIVER_STEP_UVTRIEN_SHIFT                                             (5U)
#define DRIVER_STEP_UVTRIEN(x)                                                (((uint32_t)(((uint32_t)(x)) << DRIVER_STEP_UVTRIEN_SHIFT)) & DRIVER_STEP_UVTRIEN_MASK)
#define DRIVER_STEP_WTTRIEN_MASK                                              (0x10U)
#define DRIVER_STEP_WTTRIEN_SHIFT                                             (4U)
#define DRIVER_STEP_WTTRIEN(x)                                                (((uint32_t)(((uint32_t)(x)) << DRIVER_STEP_WTTRIEN_SHIFT)) & DRIVER_STEP_WTTRIEN_MASK)
#define DRIVER_STEP_UVEN_MASK                                                 (0x8U)
#define DRIVER_STEP_UVEN_SHIFT                                                (3U)
#define DRIVER_STEP_UVEN(x)                                                   (((uint32_t)(((uint32_t)(x)) << DRIVER_STEP_UVEN_SHIFT)) & DRIVER_STEP_UVEN_MASK)
#define DRIVER_STEP_WTEN_MASK                                                 (0x4U)
#define DRIVER_STEP_WTEN_SHIFT                                                (2U)
#define DRIVER_STEP_WTEN(x)                                                   (((uint32_t)(((uint32_t)(x)) << DRIVER_STEP_WTEN_SHIFT)) & DRIVER_STEP_WTEN_MASK)
#define DRIVER_STEP_UVFDEN_MASK                                               (0x2U)
#define DRIVER_STEP_UVFDEN_SHIFT                                              (1U)
#define DRIVER_STEP_UVFDEN(x)                                                 (((uint32_t)(((uint32_t)(x)) << DRIVER_STEP_UVFDEN_SHIFT)) & DRIVER_STEP_UVFDEN_MASK)
#define DRIVER_STEP_WTFDEN_MASK                                               (0x1U)
#define DRIVER_STEP_WTFDEN_SHIFT                                              (0U)
#define DRIVER_STEP_WTFDEN(x)                                                 (((uint32_t)(((uint32_t)(x)) << DRIVER_STEP_WTFDEN_SHIFT)) & DRIVER_STEP_WTFDEN_MASK)
/* DRIVER_FDCNT Register */
#define DRIVER_FDCNT_UVFDCNT_MASK                                             (0xFFFF0000U)
#define DRIVER_FDCNT_UVFDCNT_SHIFT                                            (16U)
#define DRIVER_FDCNT_UVFDCNT(x)                                               (((uint32_t)(((uint32_t)(x)) << DRIVER_FDCNT_UVFDCNT_SHIFT)) & DRIVER_FDCNT_UVFDCNT_MASK)
#define DRIVER_FDCNT_WTFDCNT_MASK                                             (0xFFFFU)
#define DRIVER_FDCNT_WTFDCNT_SHIFT                                            (0U)
#define DRIVER_FDCNT_WTFDCNT(x)                                               (((uint32_t)(((uint32_t)(x)) << DRIVER_FDCNT_WTFDCNT_SHIFT)) & DRIVER_FDCNT_WTFDCNT_MASK)
/* DRIVER_TRIM Register */
#define DRIVER_TRIM_ILSGAINTRIM_MASK                                          (0x1F000000U)
#define DRIVER_TRIM_ILSGAINTRIM_SHIFT                                         (24U)
#define DRIVER_TRIM_ILSGAINTRIM(x)                                            (((uint32_t)(((uint32_t)(x)) << DRIVER_TRIM_ILSGAINTRIM_SHIFT)) & DRIVER_TRIM_ILSGAINTRIM_MASK)
#define DRIVER_TRIM_SLWRTTRIM_MASK                                            (0xF00000U)
#define DRIVER_TRIM_SLWRTTRIM_SHIFT                                           (20U)
#define DRIVER_TRIM_SLWRTTRIM(x)                                              (((uint32_t)(((uint32_t)(x)) << DRIVER_TRIM_SLWRTTRIM_SHIFT)) & DRIVER_TRIM_SLWRTTRIM_MASK)
#define DRIVER_TRIM_VDDPTRIM_MASK                                             (0xF0000U)
#define DRIVER_TRIM_VDDPTRIM_SHIFT                                            (16U)
#define DRIVER_TRIM_VDDPTRIM(x)                                               (((uint32_t)(((uint32_t)(x)) << DRIVER_TRIM_VDDPTRIM_SHIFT)) & DRIVER_TRIM_VDDPTRIM_MASK)
#define DRIVER_TRIM_ILSOCTRIM_MASK                                            (0x1F00U)
#define DRIVER_TRIM_ILSOCTRIM_SHIFT                                           (8U)
#define DRIVER_TRIM_ILSOCTRIM(x)                                              (((uint32_t)(((uint32_t)(x)) << DRIVER_TRIM_ILSOCTRIM_SHIFT)) & DRIVER_TRIM_ILSOCTRIM_MASK)
#define DRIVER_TRIM_IDACTRIM_MASK                                             (0xF8U)
#define DRIVER_TRIM_IDACTRIM_SHIFT                                            (3U)
#define DRIVER_TRIM_IDACTRIM(x)                                               (((uint32_t)(((uint32_t)(x)) << DRIVER_TRIM_IDACTRIM_SHIFT)) & DRIVER_TRIM_IDACTRIM_MASK)
#define DRIVER_TRIM_OCDETTRIM_MASK                                            (0x3U)
#define DRIVER_TRIM_OCDETTRIM_SHIFT                                           (0U)
#define DRIVER_TRIM_OCDETTRIM(x)                                              (((uint32_t)(((uint32_t)(x)) << DRIVER_TRIM_OCDETTRIM_SHIFT)) & DRIVER_TRIM_OCDETTRIM_MASK)
/* DRIVER_CAL Register */
#define DRIVER_CAL_GAIN_MASK                                                  (0xFF00U)
#define DRIVER_CAL_GAIN_SHIFT                                                 (8U)
#define DRIVER_CAL_GAIN(x)                                                    (((uint32_t)(((uint32_t)(x)) << DRIVER_CAL_GAIN_SHIFT)) & DRIVER_CAL_GAIN_MASK)
#define DRIVER_CAL_OFFSET_MASK                                                (0xFFU)
#define DRIVER_CAL_OFFSET_SHIFT                                               (0U)
#define DRIVER_CAL_OFFSET(x)                                                  (((uint32_t)(((uint32_t)(x)) << DRIVER_CAL_OFFSET_SHIFT)) & DRIVER_CAL_OFFSET_MASK)


/*!
 * @}
 */ /* end of group DRIVER_Register_Masks */


/*!
 * @}
 */ /* end of group DRIVER_Peripheral_Access_Layer */


/* -------------------------------------------------------------
   -- WDG Peripheral Access Layer
   ------------------------------------------------------------- */

/*!
 * @addtogroup WDG_Peripheral_Access_Layer WDG Peripheral Access Layer
 * @{
 */


/** WDG - Size of Registers Arrays */

/* WDG Register Layout Typedef */
typedef struct {
    __O  uint32_t SVCR;                                                       /**< Service Code Register, offset: 0x0000 */
    __IO uint32_t CR;                                                         /**< Control Register, offset: 0x0004 */
    __IO uint32_t LR;                                                         /**< Lock Register, offset: 0x0008 */
    __IO uint32_t TOVR;                                                       /**< Timer Overflow Value, offset: 0x000C */
    __IO uint32_t WVR;                                                        /**< Window Value Register, offset: 0x0010 */
    __I  uint32_t CNTCVR;                                                     /**< Counter Current Value Register, offset: 0x0014 */
    __IO uint32_t INTF;                                                       /**< Interrupt Flag register, offset: 0x0018 */

} WDG_Type, *WDG_MemMapPtr;

/** Number of instances of the WDG module. */
#define WDG_INSTANCE_COUNT                                                    (1u)

/* WDG0  base address */
#define WDG0_BASE                                                             (0x40052000)
#define WDG0                                                                  ((WDG_Type *)(WDG0_BASE))

/** Array initializer of WDG peripheral base addresses */
#define WDG_BASE_ADDRS                                                        { WDG0_BASE }
/** Array initializer of WDG peripheral base pointers */
#define WDG_BASE_PTRS                                                         { WDG0 }
/** Number of interrupt vector arrays for the WDG module. */
#define WDG_IRQS_ARR_COUNT                                                    (1u)
/** Number of interrupt channels for the WDG. */
#define WDG_IRQS_CH_COUNT                                                     (1u)
/** Interrupt vectors for the WDG peripheral type */
#define WDG_IRQS                                                              { WDG_IRQn }

/* WDG Register Mask */
/* WDG_SVCR Register */
#define WDG_SVCR_SVCODE_MASK                                                  (0xFFFFU)
#define WDG_SVCR_SVCODE_SHIFT                                                 (0U)
#define WDG_SVCR_SVCODE(x)                                                    (((uint32_t)(((uint32_t)(x)) << WDG_SVCR_SVCODE_SHIFT)) & WDG_SVCR_SVCODE_MASK)
/* WDG_CR Register */
#define WDG_CR_RIA_MASK                                                       (0x80U)
#define WDG_CR_RIA_SHIFT                                                      (7U)
#define WDG_CR_RIA(x)                                                         (((uint32_t)(((uint32_t)(x)) << WDG_CR_RIA_SHIFT)) & WDG_CR_RIA_MASK)
#define WDG_CR_WIN_MASK                                                       (0x40U)
#define WDG_CR_WIN_SHIFT                                                      (6U)
#define WDG_CR_WIN(x)                                                         (((uint32_t)(((uint32_t)(x)) << WDG_CR_WIN_SHIFT)) & WDG_CR_WIN_MASK)
#define WDG_CR_IBR_MASK                                                       (0x20U)
#define WDG_CR_IBR_SHIFT                                                      (5U)
#define WDG_CR_IBR(x)                                                         (((uint32_t)(((uint32_t)(x)) << WDG_CR_IBR_SHIFT)) & WDG_CR_IBR_MASK)
#define WDG_CR_CLKSRC_MASK                                                    (0x8U)
#define WDG_CR_CLKSRC_SHIFT                                                   (3U)
#define WDG_CR_CLKSRC(x)                                                      (((uint32_t)(((uint32_t)(x)) << WDG_CR_CLKSRC_SHIFT)) & WDG_CR_CLKSRC_MASK)
#define WDG_CR_DSDIS_MASK                                                     (0x4U)
#define WDG_CR_DSDIS_SHIFT                                                    (2U)
#define WDG_CR_DSDIS(x)                                                       (((uint32_t)(((uint32_t)(x)) << WDG_CR_DSDIS_SHIFT)) & WDG_CR_DSDIS_MASK)
#define WDG_CR_DBGDIS_MASK                                                    (0x2U)
#define WDG_CR_DBGDIS_SHIFT                                                   (1U)
#define WDG_CR_DBGDIS(x)                                                      (((uint32_t)(((uint32_t)(x)) << WDG_CR_DBGDIS_SHIFT)) & WDG_CR_DBGDIS_MASK)
#define WDG_CR_EN_MASK                                                        (0x1U)
#define WDG_CR_EN_SHIFT                                                       (0U)
#define WDG_CR_EN(x)                                                          (((uint32_t)(((uint32_t)(x)) << WDG_CR_EN_SHIFT)) & WDG_CR_EN_MASK)
/* WDG_LR Register */
#define WDG_LR_HL_MASK                                                        (0x2U)
#define WDG_LR_HL_SHIFT                                                       (1U)
#define WDG_LR_HL(x)                                                          (((uint32_t)(((uint32_t)(x)) << WDG_LR_HL_SHIFT)) & WDG_LR_HL_MASK)
#define WDG_LR_SL_MASK                                                        (0x1U)
#define WDG_LR_SL_SHIFT                                                       (0U)
#define WDG_LR_SL(x)                                                          (((uint32_t)(((uint32_t)(x)) << WDG_LR_SL_SHIFT)) & WDG_LR_SL_MASK)
/* WDG_TOVR Register */
#define WDG_TOVR_TO_MASK                                                      (0xFFFFU)
#define WDG_TOVR_TO_SHIFT                                                     (0U)
#define WDG_TOVR_TO(x)                                                        (((uint32_t)(((uint32_t)(x)) << WDG_TOVR_TO_SHIFT)) & WDG_TOVR_TO_MASK)
/* WDG_WVR Register */
#define WDG_WVR_WIN_MASK                                                      (0xFFFFU)
#define WDG_WVR_WIN_SHIFT                                                     (0U)
#define WDG_WVR_WIN(x)                                                        (((uint32_t)(((uint32_t)(x)) << WDG_WVR_WIN_SHIFT)) & WDG_WVR_WIN_MASK)
/* WDG_CNTCVR Register */
#define WDG_CNTCVR_CVAL_MASK                                                  (0xFFFFU)
#define WDG_CNTCVR_CVAL_SHIFT                                                 (0U)
#define WDG_CNTCVR_CVAL(x)                                                    (((uint32_t)(((uint32_t)(x)) << WDG_CNTCVR_CVAL_SHIFT)) & WDG_CNTCVR_CVAL_MASK)
/* WDG_INTF Register */
#define WDG_INTF_IF_MASK                                                      (0x1U)
#define WDG_INTF_IF_SHIFT                                                     (0U)
#define WDG_INTF_IF(x)                                                        (((uint32_t)(((uint32_t)(x)) << WDG_INTF_IF_SHIFT)) & WDG_INTF_IF_MASK)


/*!
 * @}
 */ /* end of group WDG_Register_Masks */


/*!
 * @}
 */ /* end of group WDG_Peripheral_Access_Layer */


/* -------------------------------------------------------------
   -- CFMU Peripheral Access Layer
   ------------------------------------------------------------- */

/*!
 * @addtogroup CFMU_Peripheral_Access_Layer CFMU Peripheral Access Layer
 * @{
 */


/** CFMU - Size of Registers Arrays */

/* CFMU Register Layout Typedef */
typedef struct {
    __IO uint32_t CTRL;                                                       /**< Control Register, offset: 0x0000 */
    __IO uint32_t CLK_CNT;                                                    /**< Clock Count Register, offset: 0x0004 */
    __IO uint32_t REFCLK_CNT;                                                 /**< Reference Clock Count Register, offset: 0x0008 */
    __IO uint32_t CLK_TGTCNT;                                                 /**< Target Count of to-be-measured Clock Register, offset: 0x000C */
    __IO uint32_t REFCLK_TGTCNT;                                              /**< Reference Clock Target Count Register, offset: 0x0010 */
    __I  uint32_t STS;                                                        /**< Status Register, offset: 0x0014 */
    __I  uint32_t REFCLK_CNT_SAVED;                                           /**< Reference Clock Count Saved Register, offset: 0x0018 */

} CFMU_Type, *CFMU_MemMapPtr;

/** Number of instances of the CFMU module. */
#define CFMU_INSTANCE_COUNT                                                   (1u)

/* CFMU0  base address */
#define CFMU0_BASE                                                            (0x40077000)
#define CFMU0                                                                 ((CFMU_Type *)(CFMU0_BASE))

/** Array initializer of CFMU peripheral base addresses */
#define CFMU_BASE_ADDRS                                                       { CFMU0_BASE }
/** Array initializer of CFMU peripheral base pointers */
#define CFMU_BASE_PTRS                                                        { CFMU0 }

/* CFMU Register Mask */
/* CFMU_CTRL Register */
#define CFMU_CTRL_CLK_SEL_MASK                                                (0x7F00U)
#define CFMU_CTRL_CLK_SEL_SHIFT                                               (8U)
#define CFMU_CTRL_CLK_SEL(x)                                                  (((uint32_t)(((uint32_t)(x)) << CFMU_CTRL_CLK_SEL_SHIFT)) & CFMU_CTRL_CLK_SEL_MASK)
#define CFMU_CTRL_CLK_DIV_MASK                                                (0xFFU)
#define CFMU_CTRL_CLK_DIV_SHIFT                                               (0U)
#define CFMU_CTRL_CLK_DIV(x)                                                  (((uint32_t)(((uint32_t)(x)) << CFMU_CTRL_CLK_DIV_SHIFT)) & CFMU_CTRL_CLK_DIV_MASK)
/* CFMU_CLK_CNT Register */
#define CFMU_CLK_CNT_CNT_MASK                                                 (0xFFFFU)
#define CFMU_CLK_CNT_CNT_SHIFT                                                (0U)
#define CFMU_CLK_CNT_CNT(x)                                                   (((uint32_t)(((uint32_t)(x)) << CFMU_CLK_CNT_CNT_SHIFT)) & CFMU_CLK_CNT_CNT_MASK)
/* CFMU_REFCLK_CNT Register */
#define CFMU_REFCLK_CNT_CNT_MASK                                              (0xFFFFU)
#define CFMU_REFCLK_CNT_CNT_SHIFT                                             (0U)
#define CFMU_REFCLK_CNT_CNT(x)                                                (((uint32_t)(((uint32_t)(x)) << CFMU_REFCLK_CNT_CNT_SHIFT)) & CFMU_REFCLK_CNT_CNT_MASK)
/* CFMU_CLK_TGTCNT Register */
#define CFMU_CLK_TGTCNT_CNT_MASK                                              (0xFFFFU)
#define CFMU_CLK_TGTCNT_CNT_SHIFT                                             (0U)
#define CFMU_CLK_TGTCNT_CNT(x)                                                (((uint32_t)(((uint32_t)(x)) << CFMU_CLK_TGTCNT_CNT_SHIFT)) & CFMU_CLK_TGTCNT_CNT_MASK)
/* CFMU_REFCLK_TGTCNT Register */
#define CFMU_REFCLK_TGTCNT_CNT_MASK                                           (0xFFFFU)
#define CFMU_REFCLK_TGTCNT_CNT_SHIFT                                          (0U)
#define CFMU_REFCLK_TGTCNT_CNT(x)                                             (((uint32_t)(((uint32_t)(x)) << CFMU_REFCLK_TGTCNT_CNT_SHIFT)) & CFMU_REFCLK_TGTCNT_CNT_MASK)
/* CFMU_STS Register */
#define CFMU_STS_CLK_CNT_START_MASK                                           (0x4U)
#define CFMU_STS_CLK_CNT_START_SHIFT                                          (2U)
#define CFMU_STS_CLK_CNT_START(x)                                             (((uint32_t)(((uint32_t)(x)) << CFMU_STS_CLK_CNT_START_SHIFT)) & CFMU_STS_CLK_CNT_START_MASK)
#define CFMU_STS_CLK_CNT_STOP_MASK                                            (0x2U)
#define CFMU_STS_CLK_CNT_STOP_SHIFT                                           (1U)
#define CFMU_STS_CLK_CNT_STOP(x)                                              (((uint32_t)(((uint32_t)(x)) << CFMU_STS_CLK_CNT_STOP_SHIFT)) & CFMU_STS_CLK_CNT_STOP_MASK)
#define CFMU_STS_REFCLK_CNT_STOP_MASK                                         (0x1U)
#define CFMU_STS_REFCLK_CNT_STOP_SHIFT                                        (0U)
#define CFMU_STS_REFCLK_CNT_STOP(x)                                           (((uint32_t)(((uint32_t)(x)) << CFMU_STS_REFCLK_CNT_STOP_SHIFT)) & CFMU_STS_REFCLK_CNT_STOP_MASK)
/* CFMU_REFCLK_CNT_SAVED Register */
#define CFMU_REFCLK_CNT_SAVED_CNT_MASK                                        (0xFFFFU)
#define CFMU_REFCLK_CNT_SAVED_CNT_SHIFT                                       (0U)
#define CFMU_REFCLK_CNT_SAVED_CNT(x)                                          (((uint32_t)(((uint32_t)(x)) << CFMU_REFCLK_CNT_SAVED_CNT_SHIFT)) & CFMU_REFCLK_CNT_SAVED_CNT_MASK)


/*!
 * @}
 */ /* end of group CFMU_Register_Masks */


/*!
 * @}
 */ /* end of group CFMU_Peripheral_Access_Layer */




/* ----------------------------------------------------------------------------
   -- Device Peripheral Access Layer for YTM32Z1ME0
   ---------------------------------------------------------------------------- */

/*!
 * @addtogroup Peripheral_access_layer_YTM32Z1ME0 Device Peripheral Access Layer for YTM32Z1ME0
 * @{
 */

/* @brief This module covers memory mapped registers available on SoC */

/* ----------------------------------------------------------------------------
   -- Backward Compatibility for YTM32Z1ME0
   ---------------------------------------------------------------------------- */

/*!
 * @addtogroup Backward_Compatibility_Symbols_YTM32Z1ME0 Backward Compatibility for YTM32Z1ME0
 * @{
 */

/* No backward compatibility issues. */

/*!
 * @}
 */ /* end of group Backward_Compatibility_Symbols_YTM32Z1ME0 */


#else /* #if !defined(YTM32Z1ME0_H) */
  /* There is already included the same memory map. Check if it is compatible (has the same major version) */
  #if (MCU_MEM_MAP_VERSION != 0x0100u)
    #if (!defined(MCU_MEM_MAP_SUPPRESS_VERSION_WARNING))
      #warning There are included two not compatible versions of memory maps. Please check possible differences.
    #endif /* (!defined(MCU_MEM_MAP_SUPPRESS_VERSION_WARNING)) */
  #endif /* (MCU_MEM_MAP_VERSION != 0x0100u) */
#endif  /* #if !defined(YTM32Z1ME0_H) */

/* YTM32Z1ME0.h, eof. */
/*******************************************************************************
* EOF
*******************************************************************************/
